/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
 
parser grammar SvaParser;

options {
  tokenVocab   = VerilogLexer;
  k            = 3;
  output       = AST;
  ASTLabelType = CommonTree;
}

tokens {
  AST_ACTIVITY;
  AST_ACTIVITY_TYPE;
  AST_ASSIGN_STATEMENT;
  AST_ASSIGN_TYPE;
  AST_ASSIGNMENT;
  AST_ATTRIBUTE;
  AST_ATTRIBUTED_ITEM;
  AST_ATTRIBUTED_STATEMENT;
  AST_ATTRIBUTED_UNIT;
  AST_ATTRIBUTES;
  AST_AUTOMATIC;
  AST_BINARY_OPERATION;
  AST_BLOCK_GENERATE;
  AST_BLOCK_ITEMS;
  AST_BLOCK_STATEMENT;
  AST_BLOCK_TYPE;
  AST_CASE_GENERATE;
  AST_CASE_GENERATE_ITEM;
  AST_CASE_GENERATE_ITEMS;
  AST_CASE_STATEMENT;
  AST_CASE_STATEMENT_ITEM;
  AST_CASE_STATEMENT_ITEMS;
  AST_CASE_TYPE;
  AST_CONCATENATION;
  AST_CONTINUOUS_ASSIGNMENT;
  AST_DATA_TYPE;
  AST_DECLARATION;
  AST_DEFINE_PARAMETER;
  AST_DELAY;
  AST_DELAYED_STATEMENT;
  AST_DIMENSIONS;
  AST_DISABLE_STATEMENT;
  AST_DUMMY;
  AST_EDGE;
  AST_ELEMENT;
  AST_ELEMENT_TYPE;
  AST_EVENT;
  AST_EVENT_CONTROL;
  AST_EVENTS;
  AST_EXPRESSION;
  AST_EXPRESSIONS;
  AST_FUNCTION_CALL;
  AST_GENERATE;
  AST_GENERATE_ITEM;
  AST_GENERATE_ITEMS;
  AST_GENERATE_ITEMS;
  AST_IF_GENERATE;
  AST_IF_STATEMENT;
  AST_INSTANCE;
  AST_INSTANTIATION;
  AST_INTERFACE;
  AST_LOOP_GENERATE;
  AST_LOOP_STATEMENT;
  AST_LOOP_TYPE;
  AST_MINTYPMAX_EXPRESSION;
  AST_MODIFIERS;
  AST_MODULE;
  AST_MODULE_ITEMS;
  AST_MODULE_TYPE;
  AST_NAME;
  AST_NULL_STATEMENT;
  AST_NUMBER;
  AST_OPERATION;
  AST_PARAMETER_TYPE;
  AST_PARAMETRIZATION;
  AST_PATH;
  AST_PATH_DECLARATION;
  AST_PATH_DESCRIPTION;
  AST_PATH_IF;
  AST_PATH_ITEM;
  AST_PATH_REFERENCE;
  AST_PATH_THEN;
  AST_PATH_TYPE;
  AST_PORT;
  AST_PORT_CONNECTION;
  AST_PORT_CONNECTIONS;
  AST_PROCEDURE;
  AST_PROCEDURE_ITEMS;
  AST_PROCEDURE_TYPE;
  AST_PULSESTYLE;
  AST_PULSESTYLE_TYPE;
  AST_RANGE;
  AST_RANGE_TYPE;
  AST_REFERENCE;
  AST_REFERENCES;
  AST_REPEAT_EVENTS;
  AST_REPLICATION;
  AST_ROOT;
  AST_SELECTION;
  AST_SHOWCANCELLED;
  AST_SHOWCANCELLED_TYPE;
  AST_SIGN;
  AST_SPECIFY;
  AST_STATEMENT;
  AST_STATEMENTS;
  AST_STRENGTH;
  AST_STRING;
  AST_TABLE;
  AST_TABLE_EDGE;
  AST_TABLE_ENTRY;
  AST_TABLE_LEVELS;
  AST_TASK_STATEMENT;
  AST_TERNARY_OPERATION;
  AST_TRIGGER_STATEMENT;
  AST_TYPE;
  AST_UNARY_OPERATION;
  AST_WAIT_STATEMENT;

//--------------------------------------------------------------------------------------------------
// SVA tokens
//--------------------------------------------------------------------------------------------------
  AST_CONCURRENT_ASSERTION;
  AST_CONCURRENT_ASSERTION_STATEMENT;
  AST_ASSERT_PROPERTY_STMT;
  AST_ASSUME_PROPERTY_STMT;
  AST_ACTIONS;
  AST_PROPERTY_SPEC;
  AST_CYCLE_DELAY_RANGE;
  AST_SEQUENCE_MATCH_ITEM;
  AST_CONSEC_REPETITION;
  AST_NON_CONSEC_REPETITION;
  AST_GOTO_REPETITION;
  AST_DELAY_RANGE;
  AST_VAR_DIMENSIONS;
  AST_INC_DEC;
  AST_ASSERTION_ITEM;
  AST_DEFERRED_ASSERTION;
  AST_PROCEDURAL_ASSERTION;
  AST_IMMEDIATE_ASSERTION;
  AST_IMM_ASSERT;
  AST_IMM_ASSUME;
  AST_PORT_DIRECTION;
  AST_PORT_DIMENSION;
  AST_PORT_ARG;
  AST_SVA_MATCH_EXPR;
  AST_SVA_REPEATED_EXPR;
  AST_SVA_DELAYED_EXPR;
  AST_SVA_THROUGHOUT_EXPR;
  AST_SVA_BINARY_EXPR;
  AST_SVA_EVENT_EXPR;
  AST_SVA_FIRST_MATCH_EXPR;
  AST_SVA_STRENGTH_EXPR;
  AST_SVA_NEXTTIME_EXPR;
  AST_SVA_NEG_EXPR;
  AST_SVA_IMPL_FOLL_EXPR;
  AST_SVA_ACC_REJ_EXPR;
  AST_SVA_TEMP_EXPR;
  AST_SVA_ITE_EXPR;
  AST_SVA_CASE_EXPR;
  AST_SVA_EVENT_EXPR;
  AST_SVA_REFERENCE;
  AST_PROPERTY_CASE_ITEM;
  AST_CONDITION;
  AST_VALUES;
  AST_THEN;
  AST_ELSE;
//--------------------------------------------------------------------------------------------------
// Dummy tokens for Verilog rules
//--------------------------------------------------------------------------------------------------
  DUMMY_attributes;
  DUMMY_net_assignment;
  DUMMY_expression;
  DUMMY_identifier;
  DUMMY_range_expression;
  DUMMY_port_identifier;
  DUMMY_block_identifier;
  DUMMY_lvalue;
  DUMMY_hierarchical_identifier;
  DUMMY_statement;
  DUMMY_system_function_call;
  DUMMY_primary;
  DUMMY_event_expression;
  DUMMY_event_control;
  DUMMY_function_call;
  DUMMY_reference;
  DUMMY_null_statement;
  DUMMY_logical_or_operator;
  DUMMY_logical_and_operator;
  DUMMY_bitwise_or_operator;
  DUMMY_bitwise_xor_operator;
  DUMMY_bitwise_and_operator;
  DUMMY_equality_operator;
  DUMMY_compare_operator;
  DUMMY_shift_operator;
  DUMMY_additive_operator;
  DUMMY_multiplicative_operator;
  DUMMY_power_operator;
//--------------------------------------------------------------------------------------------------
}

//==================================================================================================
// SystemVerilog Assertions Grammar
//==================================================================================================

//==================================================================================================
// A.2.10 Assertion declarations
//==================================================================================================

//==================================================================================================
concurrent_assertion_statement
  : (stmt=assert_property_statement
  | stmt=assume_property_statement)
//--------------------------------------------------------------------------------------------------
-> ^(AST_CONCURRENT_ASSERTION_STATEMENT $stmt)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
assert_property_statement
    : KW_ASSERT KW_PROPERTY LPAREN spec=property_spec RPAREN stmt=action_block
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ASSERT_PROPERTY_STMT
        $spec
        $stmt
    ) // AST_ASSERT_PROPERTY_STMT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
assume_property_statement
    : KW_ASSUME KW_PROPERTY LPAREN spec=property_spec RPAREN stmt=action_block
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ASSUME_PROPERTY_STMT
        $spec
        $stmt
    ) // AST_ASSUME_PROPERTY_STMT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
action_block
    : success_stmt=statement (KW_ELSE fail_stmt=statement)?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ACTIONS
        ^(AST_THEN $success_stmt)
        ^(AST_ELSE ($fail_stmt)?)
    ) // AST_ACTIONS
//--------------------------------------------------------------------------------------------------
    | null_statement KW_ELSE fail_stmt=statement
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ACTIONS
        ^(AST_THEN null_statement)
        ^(AST_ELSE $fail_stmt)
    ) // AST_ACTIONS
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
list_of_arguments
    : sva_port_connection  (COMMA sva_port_connection )*
//--------------------------------------------------------------------------------------------------
 -> (sva_port_connection)+
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_port_connection
    : DOT identifier LPAREN ( actual_arg )? RPAREN
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PORT_CONNECTION
        identifier
        ^(AST_PORT_ARG ( actual_arg )? )
    ) // AST_PORT_CONNECTION
//--------------------------------------------------------------------------------------------------
    | actual_arg
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PORT_CONNECTION
        ^(AST_NAME )
        ^(AST_PORT_ARG actual_arg)
    ) // AST_PORT_CONNECTION
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
actual_arg
    : property_expr
    | event_expression
;

//==================================================================================================
assertion_item_declaration
    : property_declaration
    | sequence_declaration
;

//==================================================================================================
property_declaration
    : KW_PROPERTY identifier ( LPAREN ( property_port_list )? RPAREN )? SEMI
        ( assertion_variable_declaration )*
          property_spec ( SEMI )?
            KW_ENDPROPERTY ( COLON identifier )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DECLARATION
        ^(AST_TYPE KW_PROPERTY)
        identifier
        (property_port_list)?
        (assertion_variable_declaration)*
        property_spec
    ) // AST_DECLARATION
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
property_port_list
    : property_port_item ( COMMA property_port_item )*
//--------------------------------------------------------------------------------------------------
 -> (property_port_item)+
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
property_port_item
    : attrs=attributes ( KW_LOCAL ( KW_INPUT )? )? (property_formal_type)?
        port_assignment (COMMA attributes port_assignment)*
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PORT
        ^(AST_PORT_DIRECTION (KW_LOCAL)? (KW_INPUT)?)
        ^(AST_TYPE (property_formal_type)?)
        port_assignment
        ^(AST_ATTRIBUTES (attributes)?)
    )+ // AST_PORT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
property_formal_type
    : type=sequence_formal_type
//--------------------------------------------------------------------------------------------------
 -> $type
//--------------------------------------------------------------------------------------------------
  | KW_PROPERTY
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE KW_PROPERTY)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
property_spec
    : ( event_control )? ( disable_condition )? property_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PROPERTY_SPEC
        ( event_control )?
        (disable_condition)?
        property_expr
    ) // AST_PROPERTY_SPEC
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
disable_condition
    : KW_DISABLE KW_IFF LPAREN expression RPAREN
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DISABLE_STATEMENT
        expression
    ) // AST_DISABLE_STATEMENT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_reference
    : path=hierarchical_identifier LPAREN (args=list_of_arguments)? RPAREN
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_REFERENCE
        $path
        ($args)?
    ) // AST_SVA_REFERENCE
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
match_items
    : ( COMMA sequence_match_item )+
//--------------------------------------------------------------------------------------------------
  -> sequence_match_item+
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_match_expr
    : sva_expr (-> sva_expr)
        (match_items
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_MATCH_EXPR
        sva_expr
        match_items
    ) // AST_SVA_MATCH_EXPR
//--------------------------------------------------------------------------------------------------
        )?
;

//==================================================================================================
expr_or_reference
    : expression
    | sva_reference
;

//==================================================================================================
sequence_primary
options {backtrack=true;}
    : (LPAREN expression RPAREN verilog_operator)  => expression
    | LPAREN! sequence_match_expr RPAREN!
    | expr_or_reference
;

//==================================================================================================
sva_repeated_expr
    : sequence_primary (-> sequence_primary)
        (boolean_abbrev
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_REPEATED_EXPR
        sequence_primary
        boolean_abbrev
    ) // AST_SVA_REPEATED_EXPR
//--------------------------------------------------------------------------------------------------
        )?
;

//==================================================================================================
sva_delay_expr
@init { boolean isDelayed = false; }
    : first=sva_repeated_expr
        (next_element
          { isDelayed = true; }
        )*
//--------------------------------------------------------------------------------------------------
 -> {isDelayed}?
    ^(AST_SVA_DELAYED_EXPR
        $first
        (next_element)+
    ) // AST_SVA_DELAYED_EXPR
 -> $first
//--------------------------------------------------------------------------------------------------
    | (next_element)+
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_DELAYED_EXPR
        (next_element)+
    ) // AST_SVA_DELAYED_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
next_element
    : cycle_delay_range sequence_expr
;

//==================================================================================================
sva_throughout_expr
    : lhs=sva_delay_expr (-> $lhs)
        (KW_THROUGHOUT rhs=sequence_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_THROUGHOUT_EXPR
        $lhs
        $rhs
    ) // AST_SVA_THROUGHOUT_EXPR
//--------------------------------------------------------------------------------------------------
        )?
;

//==================================================================================================
sva_within_expr
    : sva_throughout_expr (-> sva_throughout_expr)
        (KW_WITHIN rhs=sequence_expr
          -> ^(AST_SVA_BINARY_EXPR KW_WITHIN $sva_within_expr $rhs)
        )*
;

//==================================================================================================
sva_intersect_expr
    : sva_within_expr (-> sva_within_expr)
        (KW_INTERSECT rhs=sequence_expr
          -> ^(AST_SVA_BINARY_EXPR KW_INTERSECT $sva_intersect_expr $rhs)
        )*
;

//==================================================================================================
sva_first_match_expr
    : KW_FIRST_MATCH LPAREN sequence_match_expr RPAREN
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_FIRST_MATCH_EXPR
        sequence_match_expr
    ) // AST_SVA_FIRST_MATCH_EXPR
//--------------------------------------------------------------------------------------------------
;

// TODO: move to primary?
//==================================================================================================
sva_event_seq_expr
    : event_control sequence_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_EVENT_EXPR
        event_control
        sequence_expr
    ) // AST_SVA_EVENT_EXPR
//--------------------------------------------------------------------------------------------------
;

sva_basic_sequence_expr
    : sva_event_seq_expr
    | sva_intersect_expr
    | sva_first_match_expr
;

//==================================================================================================
sva_strength_expr
    : sequence_strength LPAREN sequence_expr RPAREN
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_STRENGTH_EXPR
        sequence_strength
        sequence_expr
    ) // AST_SVA_STRENGTH_EXPR
//--------------------------------------------------------------------------------------------------
    | sva_basic_sequence_expr
;

//==================================================================================================
sva_nexttime_expr
    : nexttime_op (LBRACK expression RBRACK)? sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_NEXTTIME_EXPR
        nexttime_op
       ^(AST_DELAY (expression)?)
        sva_expr
    ) // AST_SVA_NEXTTIME_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_neg_expr
    : KW_NOT sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_NEG_EXPR
        sva_expr
    ) // AST_SVA_NEG_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_unary_expr
    : sva_nexttime_expr
    | sva_neg_expr
    | sva_strength_expr
;

//==================================================================================================
sva_and_expr
    : sva_unary_expr (-> sva_unary_expr)
        (KW_AND rhs=sva_expr
          -> ^(AST_SVA_BINARY_EXPR KW_AND $sva_and_expr $rhs)
        )*
;

//==================================================================================================
sva_or_expr
    : sva_and_expr (-> sva_and_expr)
        (KW_OR rhs=sva_expr
          -> ^(AST_SVA_BINARY_EXPR KW_OR $sva_or_expr $rhs)
        )*
;

//==================================================================================================
sva_iff_expr
    : lhs=sva_or_expr (-> $lhs)
        (KW_IFF rhs=sva_expr
          -> ^(AST_SVA_BINARY_EXPR KW_IFF $lhs $rhs)
        )?
;

//==================================================================================================
sva_bin_expr
    : lhs=sva_iff_expr (-> $lhs)
        (prop_binary_op rhs=sva_expr
          -> ^(AST_SVA_BINARY_EXPR prop_binary_op $lhs $rhs)
        )?
;

//==================================================================================================
sva_impl_follow_expr
    : lhs=sva_bin_expr (-> $lhs)
        (impl_follow_op rhs=sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_IMPL_FOLL_EXPR
        impl_follow_op
        $lhs
        $rhs
    ) // AST_SVA_IMPL_FOLL_EXPR
//--------------------------------------------------------------------------------------------------
        )?
;

//==================================================================================================
sva_accept_reject_expr
    : accept_reject_op LPAREN expression RPAREN sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_ACC_REJ_EXPR
        accept_reject_op
        ^(AST_CONDITION expression)
        sva_expr
    ) // AST_SVA_ACC_REJ_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_always_expr
    : KW_ALWAYS (LBRACK delay_range RBRACK)? sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_TEMP_EXPR
        KW_ALWAYS
        ^(AST_DELAY (delay_range)?)
        sva_expr
    ) // AST_SVA_TEMP_EXPR
//--------------------------------------------------------------------------------------------------
    | KW_S_ALWAYS LBRACK delay_range RBRACK sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_TEMP_EXPR
        KW_S_ALWAYS
        ^(AST_DELAY delay_range)
        sva_expr
    ) // AST_SVA_TEMP_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_eventually_expr
    : KW_EVENTUALLY LBRACK delay_range RBRACK sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_TEMP_EXPR
        KW_EVENTUALLY
        ^(AST_DELAY delay_range)
        sva_expr
    ) // AST_SVA_TEMP_EXPR
//--------------------------------------------------------------------------------------------------
    | KW_S_EVENTUALLY (LBRACK delay_range RBRACK)? sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_TEMP_EXPR
        KW_S_EVENTUALLY
        ^(AST_DELAY (delay_range)?)
        sva_expr
    ) // AST_SVA_TEMP_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_if_else_expr
    : KW_IF LPAREN expression RPAREN alt1=sva_expr ( KW_ELSE alt2=sva_expr )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_ITE_EXPR
        ^(AST_CONDITION expression)
        $alt1
        ($alt2)?
    ) // AST_SVA_ITE_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_case_expr
    : KW_CASE LPAREN expression RPAREN property_case_item ( property_case_item )* KW_ENDCASE
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_CASE_EXPR
        expression
        (property_case_item)+
    ) // AST_SVA_CASE_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_event_expr
    : event_control sva_expr
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SVA_EVENT_EXPR
        event_control
        sva_expr
    ) // AST_SVA_EVENT_EXPR
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sva_expr
    : sva_impl_follow_expr
    | sva_if_else_expr
    | sva_case_expr
    | sva_always_expr
    | sva_eventually_expr
    | sva_accept_reject_expr
    | sva_event_expr
;

property_expr
    : sva_expr
;

//==================================================================================================
sequence_strength
    : KW_STRONG
    | KW_WEAK
;

//==================================================================================================
nexttime_op
    : KW_NEXTTIME
    | KW_S_NEXTTIME
;

//==================================================================================================
prop_binary_op
    : KW_UNTIL
    | KW_UNTIL_WITH
    | KW_S_UNTIL
    | KW_S_UNTIL_WITH
    | KW_IMPLIES
;

//==================================================================================================
impl_follow_op
    : OVER_FOLLOW
    | NON_OVER_FOLLOW
    | OVER_IMPL
    | NON_OVER_IMPL
;

//==================================================================================================
accept_reject_op
    : KW_ACCEPT_ON
    | KW_REJECT_ON
    | KW_SYNC_ACCEPT_ON
    | KW_SYNC_REJECT_ON
;

//==================================================================================================
verilog_operator
    : logical_or_operator
    | logical_and_operator
    | bitwise_or_operator
    | bitwise_xor_operator
    | bitwise_and_operator
    | equality_operator
    | compare_operator
    | shift_operator
    | additive_operator
    | multiplicative_operator
    | power_operator
;

//==================================================================================================
property_case_item
    : expression ( COMMA expression )* COLON sva_expr ( SEMI )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PROPERTY_CASE_ITEM
        ^(AST_VALUES (expression)+)
        sva_expr
    ) // AST_PROPERTY_CASE_ITEM
//--------------------------------------------------------------------------------------------------
    | KW_DEFAULT ( COLON )? sva_expr ( SEMI )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PROPERTY_CASE_ITEM
        KW_DEFAULT
        sva_expr
    ) // AST_PROPERTY_CASE_ITEM
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_declaration
    : KW_SEQUENCE name=identifier ( LPAREN ( sequence_port_list )? RPAREN )? SEMI
        ( assertion_variable_declaration )*
          sequence_expr ( SEMI )?
            KW_ENDSEQUENCE ( COLON identifier )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DECLARATION
        ^(AST_TYPE KW_SEQUENCE)
        $name
        (sequence_port_list)?
        (assertion_variable_declaration)*
        sequence_expr
    ) // AST_DECLARATION
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_port_list
    : sequence_port_item ( COMMA sequence_port_item )*
//--------------------------------------------------------------------------------------------------
 -> (sequence_port_item)+
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_port_item
    : attrs=attributes ( local=KW_LOCAL ( sequence_lvar_port_direction )? )? (sequence_formal_type)?
        port_assignment ( COMMA attributes port_assignment )*
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PORT
        ^(AST_PORT_DIRECTION ($local)? (sequence_lvar_port_direction)?)
        ^(AST_TYPE (sequence_formal_type)?)
        port_assignment
        ^(AST_ATTRIBUTES (attributes)?)
    )+ // AST_PORT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
port_assignment
    : port_identifier ( range_expression )* ( ASSIGN actual_arg )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ELEMENT
        port_identifier
        ^(AST_DIMENSIONS (range_expression)* )
        ^(AST_PORT_ARG (actual_arg)? )
    )
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_lvar_port_direction
    : KW_INPUT
    | KW_INOUT
    | KW_OUTPUT
;

//==================================================================================================
sequence_formal_type
    : type=data_type
//--------------------------------------------------------------------------------------------------
 -> $type
//--------------------------------------------------------------------------------------------------
    | KW_SEQUENCE
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE KW_SEQUENCE)
//--------------------------------------------------------------------------------------------------
    | KW_UNTYPED
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE KW_UNTYPED)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
sequence_expr
    : sva_or_expr
;

//==================================================================================================
cycle_delay_range
    : DELAY primary
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CYCLE_DELAY_RANGE primary)
//--------------------------------------------------------------------------------------------------
    | DELAY LBRACK del_expr=delay_range RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CYCLE_DELAY_RANGE $del_expr )
//--------------------------------------------------------------------------------------------------
    | DELAY LBRACK sign=range_sign RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CYCLE_DELAY_RANGE $sign)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
range_sign
    : STAR
    | PLUS
;

//==================================================================================================
sequence_match_item
  : (item=net_assignment
  | item=inc_or_dec_expression
  | item=function_call
  | item=system_function_call)
//--------------------------------------------------------------------------------------------------
-> ^(AST_SEQUENCE_MATCH_ITEM $item)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
boolean_abbrev
    : consecutive_repetition
    | non_consecutive_repetition
    | goto_repetition
;

//==================================================================================================
consecutive_repetition
    : LBRACK STAR dim=delay_range RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CONSEC_REPETITION $dim)
//--------------------------------------------------------------------------------------------------
    | LBRACK STAR RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CONSEC_REPETITION STAR)
//--------------------------------------------------------------------------------------------------
    | LBRACK PLUS RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CONSEC_REPETITION PLUS)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
non_consecutive_repetition
    : LBRACK ASSIGN delay_range RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_NON_CONSEC_REPETITION delay_range)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
goto_repetition
    : LBRACK TRIGGER delay_range RBRACK
//--------------------------------------------------------------------------------------------------
 -> ^(AST_GOTO_REPETITION delay_range)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
delay_range
    : range_expression (COLON DOLLAR)?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DELAY_RANGE
        range_expression
        (COLON DOLLAR)?
    ) // AST_DELAY_RANGE
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
assertion_variable_declaration
    : var_data_type variable_decl_assignment ( COMMA variable_decl_assignment )* SEMI
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DECLARATION
        var_data_type
        variable_decl_assignment
    )+ // AST_DECLARATION
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
var_data_type
    : data_type
    | KW_VAR! data_type
;

//==================================================================================================
signing
    : sign=(KW_SIGNED | KW_UNSIGNED)
//--------------------------------------------------------------------------------------------------
 -> ^(AST_SIGN $sign)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
data_type
    : type=integer_vector_type ( signing )? ( LBRACK dim=range_expression RBRACK )*
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE
        $type
        (signing)?
        ($dim)*
    ) // AST_DATA_TYPE
//--------------------------------------------------------------------------------------------------
    | type=integer_atom_type ( signing )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE
        $type
        (signing)?
    ) // AST_DATA_TYPE
//--------------------------------------------------------------------------------------------------
    | type=non_integer_type
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DATA_TYPE $type)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
integer_atom_type
    : KW_BYTE
    | KW_SHORTINT
    | KW_INT
    | KW_LONGINT
    | KW_INTEGER
    | KW_TIME
;

//==================================================================================================
integer_vector_type
    : KW_BIT
    | KW_LOGIC
    | KW_REG
;

//==================================================================================================
non_integer_type
    : KW_SHORTREAL
    | KW_REAL
    | KW_REALTIME
;

//==================================================================================================
inc_or_dec_operator
    : INC
    | DEC
;

//==================================================================================================
variable_decl_assignment
    : identifier (LBRACK dim=range_expression RBRACK)* ( ASSIGN expression )?
//--------------------------------------------------------------------------------------------------
 -> ^(AST_ASSIGNMENT
        identifier
        ^(AST_VAR_DIMENSIONS ($dim)*)
        (expression)?
    ) // AST_ASSIGNMENT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
inc_or_dec_expression
    : op=inc_or_dec_operator attrs=attributes val=lvalue
//--------------------------------------------------------------------------------------------------
 -> ^(AST_INC_DEC
        $val
        $op
        ^(AST_ATTRIBUTES ($attrs)*)
    ) // AST_INC_DEC
//--------------------------------------------------------------------------------------------------
    | val=lvalue attrs=attributes op=inc_or_dec_operator
//--------------------------------------------------------------------------------------------------
 -> ^(AST_INC_DEC
        $val
        $op
        ^(AST_ATTRIBUTES ($attrs)*)
    ) // AST_INC_DEC
//--------------------------------------------------------------------------------------------------
;


//==================================================================================================
//A.6.10 Assertion statements
//==================================================================================================

//==================================================================================================
assertion_item
    : ( name=block_identifier COLON )?
      (item=concurrent_assertion_statement
//--------------------------------------------------------------------------------------------------
 -> ^(AST_CONCURRENT_ASSERTION
        ($name)?
        $item
    ) // AST_CONCURRENT_ASSERTION
//--------------------------------------------------------------------------------------------------
    | item=deferred_immediate_assertion_statement
//--------------------------------------------------------------------------------------------------
 -> ^(AST_DEFERRED_ASSERTION
        ($name)?
        $item
    )
//--------------------------------------------------------------------------------------------------
      )
;

//==================================================================================================
procedural_assertion_statement
    : ( name=block_identifier COLON )? (stmt=concurrent_assertion_statement
    | stmt=immediate_assertion_statement)
//--------------------------------------------------------------------------------------------------
 -> ^(AST_PROCEDURAL_ASSERTION
        ($name)?
        $stmt
    )
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
immediate_assertion_statement
    : (stmt=simple_immediate_assertion_statement
    | stmt=deferred_immediate_assertion_statement)
//--------------------------------------------------------------------------------------------------
 -> ^(AST_IMMEDIATE_ASSERTION $stmt)
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
simple_immediate_assertion_statement
    : simple_immediate_assert_statement
    | simple_immediate_assume_statement
;

//==================================================================================================
simple_immediate_assert_statement
    : KW_ASSERT LPAREN expression RPAREN action_block
//--------------------------------------------------------------------------------------------------
 -> ^(AST_IMM_ASSERT
        ^(AST_EXPRESSION expression)
        action_block
    ) // AST_IMM_ASSERT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
simple_immediate_assume_statement
    : KW_ASSUME LPAREN expression RPAREN action_block
//--------------------------------------------------------------------------------------------------
 -> ^(AST_IMM_ASSUME
        ^(AST_EXPRESSION expression)
        action_block
    ) // AST_IMM_ASSUME
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
deferred_immediate_assertion_statement
    : deferred_immediate_assert_statement
    | deferred_immediate_assume_statement
;

//==================================================================================================
deferred_immediate_assert_statement
    : (KW_ASSERT SHARP NUMBER LPAREN expression RPAREN action_block
    | KW_ASSERT KW_FINAL LPAREN expression RPAREN action_block)
//--------------------------------------------------------------------------------------------------
 -> ^(AST_IMM_ASSERT
        ^(AST_EXPRESSION expression)
        action_block
    ) // AST_IMM_ASSERT
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
deferred_immediate_assume_statement
    : (KW_ASSUME SHARP NUMBER LPAREN expression RPAREN action_block
    | KW_ASSUME KW_FINAL LPAREN expression RPAREN action_block)
//--------------------------------------------------------------------------------------------------
 -> ^(AST_IMM_ASSUME
        ^(AST_EXPRESSION expression)
        action_block
    ) // AST_IMM_ASSUME
//--------------------------------------------------------------------------------------------------
;

//==================================================================================================
// Verilog rules to be overriden in VerilogParser grammar
//==================================================================================================
attributes
    : DUMMY_attributes;
net_assignment
    : DUMMY_net_assignment;
expression
    : DUMMY_expression;
identifier
    : DUMMY_identifier;
range_expression
    : DUMMY_range_expression;
port_identifier
    : DUMMY_port_identifier;
block_identifier
    : DUMMY_block_identifier;
lvalue
    : DUMMY_lvalue;
hierarchical_identifier
    : DUMMY_hierarchical_identifier;
statement
    : DUMMY_statement;
system_function_call
    : DUMMY_system_function_call;
primary
    : DUMMY_primary;
event_expression
    : DUMMY_event_expression;
event_control
    : DUMMY_event_control;
function_call
    : DUMMY_function_call;
reference
    : DUMMY_reference;
null_statement
    : DUMMY_null_statement;
logical_or_operator
    : DUMMY_logical_or_operator;
logical_and_operator
    : DUMMY_logical_and_operator;
bitwise_or_operator
    : DUMMY_bitwise_or_operator;
bitwise_xor_operator
    : DUMMY_bitwise_xor_operator;
bitwise_and_operator
    : DUMMY_bitwise_and_operator;
equality_operator
    : DUMMY_equality_operator;
compare_operator
    : DUMMY_compare_operator;
shift_operator
    : DUMMY_shift_operator;
additive_operator
    : DUMMY_additive_operator;
multiplicative_operator
    : DUMMY_multiplicative_operator;
power_operator
    : DUMMY_power_operator;



