/*
 * Copyright 2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.backends.design.checker;

import ru.ispras.verilog.parser.VerilogDesignBackend;
import ru.ispras.verilog.parser.elaborator.VerilogDesign;
import ru.ispras.verilog.parser.elaborator.VerilogProcess;
import ru.ispras.verilog.parser.model.VerilogStatement;
import ru.ispras.verilog.parser.walker.VerilogTreeWalker;

/**
 * {@link AssignCheckerModule} implements a Verilog assign checker.
 * <p>It checks assign problems according to a IEEE Standard 1364-2005 requirements.</p>
 *
 * @author <a href="mailto:aadanilov_3@edu.hse.ru">Alexey Danilov</a>
 */
public final class AssignCheckerModule extends VerilogDesignBackend {

  /**
   * Creates a back-end.
   *
   * @param name the back-end name.
   */
  public AssignCheckerModule(final String name) {
    super(name);
  }

  @Override
  public void start(final VerilogDesign design) {
    for (final VerilogProcess process : design.getProcesses()) {

      final VerilogStatement stmt = process.getStatement();
      final VerilogTreeWalker walker = new VerilogTreeWalker(stmt,
          new AssignCheckerModuleVisitor());
      walker.start();
    }
  }
}
