/*
 * Copyright 2019-2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.backends.design.typecast;

import ru.ispras.verilog.parser.VerilogDesignBackend;
import ru.ispras.verilog.parser.elaborator.VerilogDesign;
import ru.ispras.verilog.parser.elaborator.VerilogProcess;
import ru.ispras.verilog.parser.model.VerilogStatement;
import ru.ispras.verilog.parser.walker.VerilogTreeWalker;

/**
 * {@link VerilogTypeCaster} implements a Verilog type caster.
 * <p>It casts types of statement fields
 * according to the IEEE Standard 1364-2005 standard requirements.</p>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class VerilogTypeCaster extends VerilogDesignBackend {

  /**
   * Creates a back-end.
   *
   * @param name the back-end name.
   */
  public VerilogTypeCaster(final String name) {
    super(name);
  }

  @Override
  public void start(final VerilogDesign design) {

    for (final VerilogProcess process : design.getProcesses()) {

      final VerilogStatement stmt = process.getStatement();
      final VerilogTreeWalker walker = new VerilogTreeWalker(stmt, new VerilogTypeCastVisitor());
      walker.start();
    }
  }
}
