/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.elaborator;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.VerilogStatement;

/**
 * {@link VerilogProcess} represents instantiated processes.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogProcess {
  public enum Type {
    ALWAYS,
    INITIAL,
    ASSIGN,
    ASSERTION
  }

  private final Type type;
  private final VerilogStatement statement;

  public VerilogProcess(final Type type, final VerilogStatement statement) {
    InvariantChecks.checkNotNull(type);
    InvariantChecks.checkNotNull(statement);

    this.type = type;
    this.statement = statement;
  }

  public Type getType() {
    return type;
  }

  public boolean isAlways() {
    return type == Type.ALWAYS;
  }

  public boolean isInitial() {
    return type == Type.INITIAL;
  }

  public boolean isAssign() {
    return type == Type.ASSIGN;
  }

  public boolean isAssertion() {
    return type == Type.ASSERTION;
  }

  public VerilogStatement getStatement() {
    return statement;
  }
}
