/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogBlockGenerate} represents generate blocks.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogBlockGenerate extends VerilogNode {
  public static final Tag TAG = Tag.BLOCK_GENERATE;
  public static final EnumSet<Tag> TAGS_CHILDREN = VerilogGenerate.TAGS_CHILDREN;

  /**
   * Creates a generate block.
   * 
   * @param parent the node parent.
   */
  public VerilogBlockGenerate(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.NAMED_PSEUDO_BLOCK, parent);
  }

  /**
   * Creates a copy of the generate block.
   * 
   * @param other the generate block to be copied.
   * @param parent the parent node.
   */
  public VerilogBlockGenerate(final VerilogBlockGenerate other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public VerilogBlockGenerate clone() {
    return new VerilogBlockGenerate(this, getParentNode());
  }
}
