/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;

import java.util.EnumSet;

/**
 * {@link VerilogCode} represents source code.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogCode extends VerilogNode {
  public static final Tag TAG = Tag.CODE;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(Tag.MODULE, Tag.PROCEDURE);

  /** Source code file name. */
  private String fileName;

  /**
   * Creates a source code node.
   */
  public VerilogCode() {
    super(TAG, TAGS_CHILDREN, NodeKind.BLOCK);
    this.fileName = null;
  }

  /**
   * Creates a copy of the source code node.
   * 
   * @param other the source code entity to be copied.
   */
  public VerilogCode(final VerilogCode other) {
    super(other, null /* This node is a root */);
  }

  /**
   * Returns the name of the source code file.
   * 
   * @return the file name.
   */
  public String getFileName() {
    return this.fileName;
  }

  /**
   * Sets the name of the source code file.
   * 
   * @param fileName the file name.
   */
  public void setFileName(final String fileName) {
    InvariantChecks.checkNotNull(fileName);
    this.fileName = fileName;
  }

  @Override
  public VerilogCode clone() {
    return new VerilogCode(this);
  }
}
