/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogReference;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

/**
 * {@link VerilogPort} represents port declarations.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class VerilogPort extends VerilogNode {
  public static final Tag TAG = Tag.PORT;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.noneOf(Tag.class);

  /** Port reference. */
  private List<VerilogReference> references = new ArrayList<>();

  /**
   * Creates a port.
   * 
   * @param parent the parent node.
   */
  public VerilogPort(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.OBJECT, parent);
  }

  /**
   * Creates a copy of the port.
   * 
   * @param other the port to be copied.
   * @param parent the parent node.
   */
  public VerilogPort(final VerilogPort other, final VerilogNode parent) {
    super(other, parent);

    for (final VerilogReference reference : other.references) {
      this.references.add(reference.clone());
    }
  }

  /**
   * Returns the port references.
   * 
   * @return the list of port references.
   */
  public List<VerilogReference> getReferences() {
    return references;
  }

  /**
   * Adds the reference to the port references.
   * 
   * @param reference the reference to be added.
   */
  public void addReference(final VerilogReference reference) {
    InvariantChecks.checkNotNull(reference);
    references.add(reference);
  }

  @Override
  public VerilogPort clone() {
    return new VerilogPort(this, getParentNode());
  }
}
