/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogStatement} represents statements.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public abstract class VerilogStatement extends VerilogNode {
  /** All possible statements. */
  public static final EnumSet<Tag> TAGS = EnumSet.of(
      Tag.ASSERTION_STATEMENT,
      Tag.ASSIGN_STATEMENT,
      Tag.BLOCK_STATEMENT,
      Tag.CASE_STATEMENT,
      Tag.DELAYED_STATEMENT,
      Tag.DISABLE_STATEMENT,
      Tag.IF_STATEMENT,
      Tag.LOOP_STATEMENT,
      Tag.NULL_STATEMENT,
      Tag.TASK_STATEMENT,
      Tag.TRIGGER_STATEMENT,
      Tag.WAIT_STATEMENT
  );

  /**
   * Creates a statement.
   * 
   * @param tag the statement tag.
   * @param childrenTags the children tags.
   * @param kind the statement kind.
   * @param parent the parent node.
   */
  public VerilogStatement(
      final Tag tag,
      final EnumSet<Tag> childrenTags,
      final NodeKind kind,
      final VerilogNode parent) {
    super(tag, childrenTags, kind, parent);
  }

  /**
   * Creates a statement with no children.
   * 
   * @param tag the statement tag.
   * @param parent the parent node.
   */
  public VerilogStatement(final Tag tag, final VerilogNode parent) {
    this(tag, EnumSet.noneOf(Tag.class), NodeKind.ELEMENT, parent);
  }

  /**
   * Creates a null statement.
   * 
   * @param parent the parent node.
   */
  public VerilogStatement(final VerilogNode parent) {
    this(Tag.NULL_STATEMENT, parent);
  }

  /**
   * Creates a copy of the statement.
   * 
   * @param other the statement to be copied.
   * @param parent the parent node.
   */
  public VerilogStatement(final VerilogStatement other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public abstract VerilogStatement clone();
}
