/*
 * Copyright 2013-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License") {} you may not use this file
 * except in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sample;

import ru.ispras.verilog.parser.VerilogSyntaxBackend;
import ru.ispras.verilog.parser.VerilogTranslator;
import ru.ispras.verilog.parser.walker.VerilogNodeVisitor;

/**
 * {@link VerilogPrinter} illustrates development of a Verilog back-end.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogPrinter {
  private static final boolean DEBUG_COMMENTS_ENABLED = true;

  /**
   * Launches the Verilog translator with the Verilog printer as a back-end.
   * 
   * @param args the command line parameters.
   */
  public static void main(final String[] args) {
    final VerilogTranslator translator = new VerilogTranslator();
    final VerilogNodeVisitor visitor = new VerilogNodePrinter(DEBUG_COMMENTS_ENABLED);

    // Register the AST-level printer.
    translator.add(new VerilogSyntaxBackend("printer", visitor));

    // Register the design-level printer.
    translator.add(new VerilogDesignPrinter("printer"));

    // Parse a description and print it.
    translator.start(args);
  }
}
