/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva;

import ru.ispras.verilog.parser.model.VerilogNode;
import ru.ispras.verilog.parser.sva.basis.SvaProperty;

import java.util.EnumSet;

/**
 * {@link SvaPropertyDeclaration} represents the declaration of a property.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaPropertyDeclaration extends SvaDeclaration {

  public static final Tag TAG = Tag.PROPERTY_DECLARATION;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(Tag.DECLARATION, Tag.SVA_INSTANCE);

  /** Property specification. */
  private SvaProperty property;

  /**
   * Creates a new property declaration.
   *
   * @param parent - the specified parent node.
   */
  public SvaPropertyDeclaration(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.CLASS, parent);

  }

  /**
   * Creates a new property declaration using the other.
   *
   * @param other - the specified property declaration to copy.
   * @param parent - the specified parent node.
   */
  private SvaPropertyDeclaration(final SvaPropertyDeclaration other, final VerilogNode parent) {
    super(other, parent);
    this.property = other.property != null ? other.property.clone() : null;

  }

  @Override
  public SvaPropertyDeclaration clone() {
    return new SvaPropertyDeclaration(this, getParentNode());
  }

  /**
   * Returns the property specification.
   *
   * @return the property specification.
   */
  public SvaProperty getProperty() {
    return property;
  }

  /**
   * Sets the property specification.
   *
   * @param property - the specified property specification.
   */
  public void setProperty(final SvaProperty property) {
    this.property = property;
  }

}
