/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva;

import ru.ispras.verilog.parser.model.VerilogNode;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

import java.util.EnumSet;

/**
 * {@link SvaSequenceDeclaration} represents the declaration of a sequence.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaSequenceDeclaration extends SvaDeclaration {

  public static final Tag TAG = Tag.SEQUENCE_DECLARATION;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(Tag.DECLARATION, Tag.SVA_INSTANCE);

  /** Sequence expression. */
  private VerilogExpression expression;

  /**
   * Creates a new sequence declaration.
   *
   * @param parent - the specified parent node.
   */
  public SvaSequenceDeclaration(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.CLASS, parent);
  }

  /**
   * Creates a new sequence declaration using the other.
   *
   * @param other - the specified sequence declaration to copy.
   * @param parent - the specified parent node.
   */
  private SvaSequenceDeclaration(final SvaSequenceDeclaration other, final VerilogNode parent) {
    super(other, parent);
    this.expression = other.expression != null ? other.expression.clone() : null;
  }

  @Override
  public SvaSequenceDeclaration clone() {
    return new SvaSequenceDeclaration(this, getParentNode());
  }

  /**
   * Returns the sequence expression.
   *
   * @return the sequence expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets the sequence expression.
   *
   * @param expression - the specified expression.
   */
  public void setExpression(VerilogExpression expression) {
    this.expression = expression;
  }

}
