/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.basis;

/**
 * {@link SvaRepetition} represents the SVA repetition operation.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaRepetition extends SvaRange {

  /** Repetition type. */
  private SvaTemporalOperations type;

  /**
   * Creates a new repetition.
   */
  public SvaRepetition() {
    super();
  }

  /**
   * Copy constructor.
   *
   * @param other - the repetition to copy.
   */
  private SvaRepetition(final SvaRepetition other) {
    super(other);
    this.type = other.type;
  }

  /**
   * Constructs a new repetition using the range.
   *
   * @param other - the specified {@link SvaRange}.
   */
  public SvaRepetition(final SvaRange other) {
    super(other);
  }

  /**
   * Returns the repetition type.
   *
   * @return the repetition type.
   */
  public SvaTemporalOperations getType() {
    return type;
  }

  /**
   * Sets the repetition type.
   *
   * @param type - the specified type.
   */
  public void setType(final SvaTemporalOperations type) {
    this.type = type;
  }

  @Override
  public SvaRepetition clone() {
    return new SvaRepetition(this);
  }
}
