/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.basis;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.VerilogAssignment;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

/**
 * {@link SvaSequenceMatchAction} represents the action performed on a sequence match.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaSequenceMatchAction {

  /** Match action. */
  private Object action;

  /**
   * Creates a new sequence match action.
   */
  public SvaSequenceMatchAction() { }

  /**
   * Copy constructor.
   *
   * @param other - action to copy.
   */
  private SvaSequenceMatchAction(final SvaSequenceMatchAction other) {
    InvariantChecks.checkNotNull(other);
    this.action = other.isAssignment()
        ? ((VerilogAssignment) other.action).clone()
        : other.isCall() ? ((VerilogExpression) other.action).clone() : null;
  }

  /**
   * Shows whether the action is a method call.
   *
   * @return {@code true} if the action is a call, {@code false} otherwise.
   */
  public boolean isCall() {
    return action instanceof VerilogExpression;
  }

  /**
   * Shows whether the action is an assignment.
   *
   * @return {@code true} if the action is an assignment, {@code false} otherwise.
   */
  public boolean isAssignment() {
    return action instanceof VerilogAssignment;
  }

  /**
   * Sets the match action.
   *
   * @param action - the specified action.
   */
  public void setAction(final Object action) {
    this.action = action;
  }

  /**
   * Returns the match action.
   *
   * @return the match action.
   */
  public Object getAction() {
    return action;
  }

  @Override
  public SvaSequenceMatchAction clone() {
    return new SvaSequenceMatchAction(this);
  }
}
