/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.basis;

/**
 * {@link SvaTemporalOperations} represents the SVA sequence/property operations.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public enum SvaTemporalOperations {
  // Highest priority
  //------------------------------------------------------------------------------------------------
  // Sequence operators
  //------------------------------------------------------------------------------------------------
  CONSECUTIVE_REP,
  NON_CONSECUTIVE_REP,
  GOTO_REP,
  //------------------------------------------------------------------------------------------------
  DELAY,
  //------------------------------------------------------------------------------------------------
  THROUGHOUT,
  //------------------------------------------------------------------------------------------------
  WITHIN,
  //------------------------------------------------------------------------------------------------
  INTERSECT,
  //------------------------------------------------------------------------------------------------
  FIRST_MATCH,
  //------------------------------------------------------------------------------------------------
  // Property operators
  //------------------------------------------------------------------------------------------------
  STRONG,
  WEAK,
  NOT,
  NEXTTIME,
  S_NEXTTIME,
  //------------------------------------------------------------------------------------------------
  AND,
  //------------------------------------------------------------------------------------------------
  OR,
  //------------------------------------------------------------------------------------------------
  IFF,
  //------------------------------------------------------------------------------------------------
  UNTIL,
  S_UNTIL,
  UNTIL_WITH,
  S_UNTIL_WITH,
  IMPLIES,
  //------------------------------------------------------------------------------------------------
  OVERLAPPED_IMPL,
  NON_OVERLAPPED_IMPL,
  OVERLAPPED_FOLLOW,
  NON_OVERLAPPED_FOLLOW,
  //------------------------------------------------------------------------------------------------
  ALWAYS,
  S_ALWAYS,
  RANGED_ALWAYS,
  EVENTUALLY,
  S_EVENTUALLY,
  RANGED_S_EVENTUALLY,
  ACCEPT_ON,
  REJECT_ON,
  SYNC_ACCEPT_ON,
  SYNC_REJECT_ON,
  ITE,
  CASE,
  //------------------------------------------------------------------------------------------------
  // Lowest priority
  RANGE,
  SEQ_INSTANCE,
  PROP_INSTANCE,
  ZERO_OR_MORE,
  ONE_OR_MORE;

  public static SvaTemporalOperations getRangedOp(final SvaTemporalOperations operation) {
    switch (operation) {
      case ALWAYS:
        return RANGED_ALWAYS;
      case S_EVENTUALLY:
        return RANGED_S_EVENTUALLY;
      default:
        return operation;
    }
  }


}
