/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.expression;

import ru.ispras.verilog.parser.model.basis.VerilogExpression;

/**
 * {@link SvaConditionalExpression} represents the SVA expressions containing logical conditions in
 * addition to operands.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaConditionalExpression extends SvaOperatorExpression {

  /** Condition. */
  private VerilogExpression condition;

  /**
   * Creates a new conditional expression.
   */
  public SvaConditionalExpression() {
    super();
  }

  /**
   * Copy constructor.
   *
   * @param other - the expression to copy.
   */
  private SvaConditionalExpression(final SvaConditionalExpression other) {
    super(other);
    this.condition = other.condition != null ? other.condition.clone() : null;
  }

  /**
   * Returns the condition.
   * @return the condition.
   */
  public VerilogExpression getCondition() {
    return condition;
  }

  /**
   * Sets the condition.
   *
   * @param condition - the specified condition expression.
   */
  public void setCondition(final VerilogExpression condition) {
    this.condition = condition;
  }

  @Override
  public SvaConditionalExpression clone() {
    return new SvaConditionalExpression(this);
  }
}
