/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.expression;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogEvent;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

import java.util.ArrayList;
import java.util.List;

/**
 * {@link SvaEventExpression} represents the SVA expression with event control.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaEventExpression extends VerilogExpression {

  /** Event. */
  private List<VerilogEvent> event;

  /** Expression. */
  private VerilogExpression expression;

  /**
   * Creates a new event expression.
   */
  public SvaEventExpression() { }

  /**
   * Copy constructor.
   *
   * @param other - the event expression to copy.
   */
  private SvaEventExpression(final SvaEventExpression other) {
    super(other);
    this.expression = other.expression != null ? other.expression.clone() : null;
    event = new ArrayList<>();
    if (other.event != null) {
      for (VerilogEvent otherEvent : other.event) {
        event.add(otherEvent.clone());
      }
    }
  }

  /**
   * Returns the event control.
   *
   * @return the list of {@link VerilogEvent}.
   */
  public List<VerilogEvent> getEvent() {
    return event;
  }

  /**
   * Sets the event control.
   *
   * @param event - the list of {@link VerilogEvent}.
   */
  public void setEvent(final List<VerilogEvent> event) {
    this.event = event;
  }

  /**
   * Returns the expression.
   *
   * @return the expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets the expression.
   *
   * @param expression - the specified expression.
   */
  public void setExpression(final VerilogExpression expression) {
    this.expression = expression;
  }

  @Override
  public void setNode(final Node node) {
    InvariantChecks.checkNotNull(node);
    final Object oldUserData = node.getUserData();
    super.setNode(node);
    // Set the old user data for node that is passed from another rule.
    node.setUserData(oldUserData);
  }

  @Override
  public SvaEventExpression clone() {
    return new SvaEventExpression(this);
  }

}
