/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.expression;

import ru.ispras.verilog.parser.sva.basis.SvaRange;

/**
 * {@link SvaRangedExpression} represents the SVA expressions containing range in addition to the
 * operands.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaRangedExpression extends SvaOperatorExpression {

  /** Range. */
  private SvaRange range;

  /**
   * Creates a new ranged expression.
   */
  public SvaRangedExpression() {
    super();
  }

  /**
   * Copy constructor.
   *
   * @param other - the ranged expression to copy.
   */
  private SvaRangedExpression(final SvaRangedExpression other) {
    super(other);
    this.range = other.range != null ? other.range.clone() : null;
  }

  /**
   * Returns the range.
   *
   * @return the range.
   */
  public SvaRange getRange() {
    return range;
  }

  /**
   * Sets the range.
   *
   * @param range - the specified range.
   */
  public void setRange(final SvaRange range) {
    this.range = range;
  }

  @Override
  public SvaRangedExpression clone() {
    return new SvaRangedExpression(this);
  }

}
