/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.sva.expression;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

/**
 * {@link SvaStrengthExpression} represents the sequence expression with strength specification.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SvaStrengthExpression extends VerilogExpression {

  /**
   * Strength specifications.
   */
  public enum Strength {
    STRONG,
    WEAK
  }

  /** Sequence expression. */
  private VerilogExpression sequence;

  /** Strength specification. */
  private Strength strength;

  /**
   * Creates a new strength expression.
   */
  public SvaStrengthExpression() { }

  /**
   * Copy constructor.
   *
   * @param other - the expression to copy.
   */
  private SvaStrengthExpression(final SvaStrengthExpression other) {
    super(other);
    this.strength = other.strength;
    this.sequence = other.sequence != null ? other.sequence.clone() : null;
  }

  /**
   * Returns the sequence expression.
   *
   * @return the sequence expression.
   */
  public VerilogExpression getSequence() {
    return sequence;
  }

  /**
   * Sets the sequence expression.
   *
   * @param sequence - the specified sequence expression.
   */
  public void setSequence(final VerilogExpression sequence) {
    this.sequence = sequence;
  }

  /**
   * Returns the strength specification.
   *
   * @return the strength specification.
   */
  public Strength getStrength() {
    return strength;
  }

  /**
   * Sets the strength specification.
   *
   * @param strength - the specified strength specification.
   */
  public void setStrength(final Strength strength) {
    this.strength = strength;
  }

  @Override
  public SvaStrengthExpression clone() {
    return new SvaStrengthExpression(this);
  }

}
