/*
 * Copyright 2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.transformer;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.transformer.NodeTransformer;
import ru.ispras.fortress.transformer.Transformer;
import ru.ispras.fortress.transformer.TransformerRule;
import ru.ispras.verilog.parser.elaborator.VerilogBinding;
import ru.ispras.verilog.parser.elaborator.VerilogContext;
import ru.ispras.verilog.parser.model.util.ModelUtils;
import ru.ispras.verilog.parser.sva.basis.SvaTemporalOperations;

import java.util.Map;

/**
 * {@link VerilogTransformerSvaInstance} represents the transformer of SVA instances.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class VerilogTransformerSvaInstance extends VerilogTransformer {

  private VerilogContext context;

  public VerilogTransformerSvaInstance(final VerilogContext context) {
    this.context = context;
  }

  @Override
  public Node transform(final Node node) {

    final Map<String, VerilogBinding> bindings = context.getBindings();
    final boolean[] success = {false};

    final TransformerRule rule = new TransformerRule() {

      @Override
      public boolean isApplicable(final Node node) {
        return ModelUtils.isSvaInstance(node);
      }

      @Override
      public Node apply(Node node) {
        final String name = context.toString();

        if (bindings.containsKey(name)) {
          success[0] = true;
          return bindings.get(name).getRhs();
        }

        return node;
      }
    };

    final NodeTransformer transformer = new NodeTransformer();
    transformer.addRule(SvaTemporalOperations.SEQ_INSTANCE, rule);
    transformer.addRule(SvaTemporalOperations.PROP_INSTANCE, rule);

    Node transformedNode = Transformer.transform(node, transformer);
    while (success[0]) {
      success[0] = false;
      transformedNode = Transformer.transform(transformedNode, transformer);
    }

    return transformedNode;
  }

}
