/*
 * Copyright 2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.transformer;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.transformer.NodeProvider;
import ru.ispras.fortress.transformer.Transformer;
import ru.ispras.verilog.parser.sva.SvaInstantiation;

import java.util.Map;

/**
 * {@link VerilogTransformerSvaStatement} represents the transformer of assertion statements.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class VerilogTransformerSvaStatement extends VerilogTransformer {

  private Map<String, SvaInstantiation> instances;

  public VerilogTransformerSvaStatement(final Map<String, SvaInstantiation> instances) {
    this.instances = instances;
  }

  @Override
  public Node transform(final Node node) {

    final boolean[] success = {false};

    final NodeProvider nodeProvider = variable -> {
      final String varName = variable.getName();
      if (instances.containsKey(varName)) {
        final SvaInstantiation instance = instances.get(varName);
        success[0] = true;
        return instance.getNode();
      }
      return new NodeVariable(variable);
    };

    Node transformedNode = Transformer.substitute(node, nodeProvider);
    while (success[0]) {
      success[0] = false;
      transformedNode = Transformer.substitute(transformedNode, nodeProvider);
    }

    return transformedNode;
  }

}
