/*
 * Copyright 2019-2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.verilog.parser.util;

import ru.ispras.verilog.parser.VerilogFrontend;

import java.util.List;

/**
 * This class is used by {@link VerilogFrontend} to expand define structures of Verilog code.
 *
 * @author <a href="mailto:aadanilov_3@edu.hse.ru">Alexey Danilov</a>
 */
public class DefineStructure {

  private String value;

  private final boolean parameterized;

  /**
   * Constructor with parameter with means is define structure parameterized or not.
   * @param parameterized The parameterized structure flag.
   */
  public DefineStructure(boolean parameterized) {
    this.parameterized = parameterized;
  }

  /**
   * Returns string with entered params.
   *
   * @param params list of params.
   * @return String form of the specified parameters.
   */
  public String getString(final List<String> params) {
    if (!parameterized) {
      return value;
    }
    return String.format(value, params.toArray());
  }

  public void setValue(String value) {
    this.value = value;
  }
}
