/*
 * Copyright 2012-2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.util;

import org.antlr.runtime.ANTLRFileStream;
import ru.ispras.verilog.parser.VerilogLogger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.LinkedList;
import java.util.StringTokenizer;

/**
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class IncludeFileFinder {
  private LinkedList<String> dirs = new LinkedList<>();

  public void addPath(final String path) {
    dirs.add(path);
  }

  public void addPaths(final String paths) {
    final StringTokenizer tokenizer = new StringTokenizer(paths, File.pathSeparator);

    while (tokenizer.hasMoreTokens()) {
      final String path = tokenizer.nextToken();
      addPath(path);
    }
  }

  public File openFile(final String filename) {
    File file = new File(filename);

    if (!file.exists() && !file.isAbsolute()) {
      for (final String path : dirs) {
        String filepath = path + File.separator + filename;

        if ((file = new File(filepath)).exists()) {
          return file;
        }
      }
    }

    return file;
  }

  public InputStream openFileAsStream(final String filename) {
    try {
      final File file = openFile(filename);
      return file.exists() ? new FileInputStream(file.getAbsolutePath()) : null;
    } catch (final Exception e) {
      e.printStackTrace();
      return null;
    }
  }

  public BufferedReader openFileAsReader(final String filename) {
    final InputStream stream = openFileAsStream(filename);
    VerilogLogger.debug("Stream " + stream);
    return stream != null ? new BufferedReader(new InputStreamReader(stream)) : null;
  }

  public ANTLRFileStream openFileAsAntlrStream(final String filename) {
    try {
      final File file = openFile(filename);
      return file.exists() ? new ANTLRFileStream(file.getAbsolutePath()) : null;
    } catch (final Exception e) {
      e.printStackTrace();
      return null;
    }
  }
}
