/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.verilog.parser.util;

import ru.ispras.verilog.parser.VerilogFrontend;

/**
 * This class is used by {@link VerilogFrontend} to find loops in inclusion.
 *
 * @author <a href="mailto:aadanilov_3@edu.hse.ru">Alexey Danilov</a>
 */
public class IncludeStructure {

  private String source;

  private String filename;

  /**
   * All arg constructor.
   *
   * @param filename name of included file
   * @param source name of file where filename was included
   */
  public IncludeStructure(String filename, String source) {
    this.filename = filename;
    this.source = source;
  }

  /**
   * Returns name of included file
   *
   * @return name of included file.
   */
  public String getFilename() {
    return filename;
  }

  /**
   * Returns name of file where filename was included
   *
   * @return name of file where we include filename.
   */
  public String getSource() {
    return source;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj.getClass() == this.getClass()) {
      return ((IncludeStructure) obj).filename.equals(filename);
    }

    if (obj.getClass() == String.class) {
      return obj.equals(filename);
    }

    return false;
  }

  @Override
  public int hashCode() {
    return filename.hashCode();
  }
}
