/*
 * Copyright 2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.verilog.parser.util;

import ru.ispras.fortress.expression.Node;
import ru.ispras.verilog.parser.transformer.VerilogTransformerVariableSubstitute;

/**
 * This class is used by {@link VerilogTransformerVariableSubstitute} save information about indexes
 * which is out of bound in Verilog bit vector.
 *
 * @author <a href="mailto:aadanilov_3@edu.hse.ru">Alexey Danilov</a>
 */
public class IndexOutOfBoundStructure {

  private final int index;

  private final int lowerBound;

  private final int upperBound;

  private final Node vector;

  /**
   * All arg constructor.
   *
   * @param index      index value that is out of bound
   * @param vector     Verilog bit vector used to identify the name
   * @param upperBound upper bound of the bit vector
   * @param lowerBound lower bound of the bit vector
   */
  public IndexOutOfBoundStructure(int index, Node vector, int upperBound, int lowerBound) {
    this.index = index;
    this.vector = vector;
    this.upperBound = upperBound;
    this.lowerBound = lowerBound;
  }

  /**
   * Method returns is this object equal to another object.
   *
   * @param obj object that will be checked
   * @return true if object is equal false if not
   */
  @Override
  public boolean equals(Object obj) {

    if (obj instanceof IndexOutOfBoundStructure) {
      return ((IndexOutOfBoundStructure) obj).index == index
          && ((IndexOutOfBoundStructure) obj).lowerBound == lowerBound
          && ((IndexOutOfBoundStructure) obj).upperBound == upperBound
          && ((IndexOutOfBoundStructure) obj).vector.toString().equals(vector.toString());
    }

    return false;
  }

  /**
   * Returns error message with description which index is out of bound.
   *
   * @return error message
   */
  @Override
  public String toString() {
    return String.format("Index: %s is out of range [%s:%s] in %s", index, lowerBound, upperBound,
        vector.toString());
  }
}
