/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.util;

import ru.ispras.verilog.parser.sample.VerilogPrinter;

import java.util.LinkedList;
import java.util.List;

/**
 * Basic class for test cases are aimed at collections of Verilog modules, i.e. benchmarks.
 * <p>Benchmarks a located at the same basic directory and can contain sub-directories.</p>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class VerilogBenchmarkTest {

  /**
   * Returns the full path to basic test directory.
   * @return The full path to basic test directory.
   */
  public abstract String getTestDirPath();

  /**
   * Prints the elaborated structure of the module with the specified arguments.
   * @param module The module name.
   * @param args VerilogPrinter arguments.
   */
  protected void runTest(
      final String module,
      final String ... args) {

    final List<String> list = new LinkedList<>();

    if (module != null) {
      list.add("--module-name");
      list.add(module);
    }

    for (final String arg : args) {

      if (isHdlSourceFile(arg)) {

        // This arg contains paths(s) to file(s), it should be enhanced
        list.add(getTestDirPath() + arg);
      } else {
        list.add(arg);
      }
    }

    VerilogPrinter.main(list.toArray(new String[0]));
  }

  private boolean isHdlSourceFile(final String arg) {
    return arg.endsWith(".v") || arg.endsWith(".vh") || arg.endsWith(".h") || arg.endsWith(".sv");
  }
}
