/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License") you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.walker;

import ru.ispras.verilog.parser.model.VerilogActivity;
import ru.ispras.verilog.parser.model.VerilogAssign;
import ru.ispras.verilog.parser.model.VerilogAssignStatement;
import ru.ispras.verilog.parser.model.VerilogAssignment;
import ru.ispras.verilog.parser.model.VerilogAttribute;
import ru.ispras.verilog.parser.model.VerilogBlockGenerate;
import ru.ispras.verilog.parser.model.VerilogBlockStatement;
import ru.ispras.verilog.parser.model.VerilogCaseGenerate;
import ru.ispras.verilog.parser.model.VerilogCaseGenerateItem;
import ru.ispras.verilog.parser.model.VerilogCaseStatement;
import ru.ispras.verilog.parser.model.VerilogCaseStatementItem;
import ru.ispras.verilog.parser.model.VerilogCode;
import ru.ispras.verilog.parser.model.VerilogDeclaration;
import ru.ispras.verilog.parser.model.VerilogDefineParameter;
import ru.ispras.verilog.parser.model.VerilogDelayedStatement;
import ru.ispras.verilog.parser.model.VerilogDisableStatement;
import ru.ispras.verilog.parser.model.VerilogGenerate;
import ru.ispras.verilog.parser.model.VerilogIfGenerate;
import ru.ispras.verilog.parser.model.VerilogIfGenerateBranch;
import ru.ispras.verilog.parser.model.VerilogIfStatement;
import ru.ispras.verilog.parser.model.VerilogIfStatementBranch;
import ru.ispras.verilog.parser.model.VerilogInstantiation;
import ru.ispras.verilog.parser.model.VerilogLoopGenerate;
import ru.ispras.verilog.parser.model.VerilogLoopStatement;
import ru.ispras.verilog.parser.model.VerilogModule;
import ru.ispras.verilog.parser.model.VerilogNullStatement;
import ru.ispras.verilog.parser.model.VerilogPathDeclaration;
import ru.ispras.verilog.parser.model.VerilogPort;
import ru.ispras.verilog.parser.model.VerilogPortConnection;
import ru.ispras.verilog.parser.model.VerilogProcedure;
import ru.ispras.verilog.parser.model.VerilogPulseStyle;
import ru.ispras.verilog.parser.model.VerilogShowCancelled;
import ru.ispras.verilog.parser.model.VerilogSpecify;
import ru.ispras.verilog.parser.model.VerilogTable;
import ru.ispras.verilog.parser.model.VerilogTableEntry;
import ru.ispras.verilog.parser.model.VerilogTaskStatement;
import ru.ispras.verilog.parser.model.VerilogTriggerStatement;
import ru.ispras.verilog.parser.model.VerilogWaitStatement;
import ru.ispras.verilog.parser.sva.SvaAssertionStatement;
import ru.ispras.verilog.parser.sva.SvaInstantiation;
import ru.ispras.verilog.parser.sva.SvaPropertyDeclaration;
import ru.ispras.verilog.parser.sva.SvaSequenceDeclaration;

/**
 * This class implements an empty Verilog node visitor.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class VerilogEmptyVisitor extends VerilogNodeVisitor {

  @Override
  public Result onActivityBegin(final VerilogActivity node) {
    return Result.OK;
  }

  @Override
  public Result onActivityEnd(final VerilogActivity node) {
    return Result.OK;
  }

  @Override
  public Result onAssignBegin(final VerilogAssign node) {
    return Result.OK;
  }

  @Override
  public Result onAssignEnd(final VerilogAssign node) {
    return Result.OK;
  }

  @Override
  public Result onAssignStatementBegin(final VerilogAssignStatement node) {
    return Result.OK;
  }

  @Override
  public Result onAssignStatementEnd(final VerilogAssignStatement node) {
    return Result.OK;
  }

  @Override
  public Result onAssignmentBegin(final VerilogAssignment node) {
    return Result.OK;
  }

  @Override
  public Result onAssignmentEnd(final VerilogAssignment node) {
    return Result.OK;
  }

  @Override
  public Result onAttributeBegin(final VerilogAttribute node) {
    return Result.OK;
  }

  @Override
  public Result onAttributeEnd(final VerilogAttribute node) {
    return Result.OK;
  }

  @Override
  public Result onBlockGenerateBegin(final VerilogBlockGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onBlockGenerateEnd(final VerilogBlockGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onBlockStatementBegin(final VerilogBlockStatement node) {
    return Result.OK;
  }

  @Override
  public Result onBlockStatementEnd(final VerilogBlockStatement node) {
    return Result.OK;
  }

  @Override
  public Result onCaseGenerateBegin(final VerilogCaseGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onCaseGenerateEnd(final VerilogCaseGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onCaseGenerateItemBegin(final VerilogCaseGenerateItem node) {
    return Result.OK;
  }

  @Override
  public Result onCaseGenerateItemEnd(final VerilogCaseGenerateItem node) {
    return Result.OK;
  }

  @Override
  public Result onCaseStatementBegin(final VerilogCaseStatement node) {
    return Result.OK;
  }

  @Override
  public Result onCaseStatementEnd(final VerilogCaseStatement node) {
    return Result.OK;
  }

  @Override
  public Result onCaseStatementItemBegin(final VerilogCaseStatementItem node) {
    return Result.OK;
  }

  @Override
  public Result onCaseStatementItemEnd(final VerilogCaseStatementItem node) {
    return Result.OK;
  }

  @Override
  public Result onCodeBegin(final VerilogCode node) {
    return Result.OK;
  }

  @Override
  public Result onCodeEnd(final VerilogCode node) {
    return Result.OK;
  }

  @Override
  public Result onDeclarationBegin(final VerilogDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onDeclarationEnd(final VerilogDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onDefineParameterBegin(final VerilogDefineParameter node) {
    return Result.OK;
  }

  @Override
  public Result onDefineParameterEnd(final VerilogDefineParameter node) {
    return Result.OK;
  }

  @Override
  public Result onDelayedStatementBegin(final VerilogDelayedStatement node) {
    return Result.OK;
  }

  @Override
  public Result onDelayedStatementEnd(final VerilogDelayedStatement node) {
    return Result.OK;
  }

  @Override
  public Result onDisableStatementBegin(final VerilogDisableStatement node) {
    return Result.OK;
  }

  @Override
  public Result onDisableStatementEnd(final VerilogDisableStatement node) {
    return Result.OK;
  }

  @Override
  public Result onGenerateBegin(final VerilogGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onGenerateEnd(final VerilogGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onIfGenerateBegin(final VerilogIfGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onIfGenerateEnd(final VerilogIfGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onIfGenerateBranchBegin(final VerilogIfGenerateBranch node) {
    return Result.OK;
  }

  @Override
  public Result onIfGenerateBranchEnd(final VerilogIfGenerateBranch node) {
    return Result.OK;
  }

  @Override
  public Result onIfStatementBegin(final VerilogIfStatement node) {
    return Result.OK;
  }

  @Override
  public Result onIfStatementEnd(final VerilogIfStatement node) {
    return Result.OK;
  }

  @Override
  public Result onIfStatementBranchBegin(final VerilogIfStatementBranch node) {
    return Result.OK;
  }

  @Override
  public Result onIfStatementBranchEnd(final VerilogIfStatementBranch node) {
    return Result.OK;
  }

  @Override
  public Result onInstantiationBegin(final VerilogInstantiation node) {
    return Result.OK;
  }

  @Override
  public Result onInstantiationEnd(final VerilogInstantiation node) {
    return Result.OK;
  }

  @Override
  public Result onLoopGenerateBegin(final VerilogLoopGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onLoopGenerateEnd(final VerilogLoopGenerate node) {
    return Result.OK;
  }

  @Override
  public Result onLoopStatementBegin(final VerilogLoopStatement node) {
    return Result.OK;
  }

  @Override
  public Result onLoopStatementEnd(final VerilogLoopStatement node) {
    return Result.OK;
  }

  @Override
  public Result onModuleBegin(final VerilogModule node) {
    return Result.OK;
  }

  @Override
  public Result onModuleEnd(final VerilogModule node) {
    return Result.OK;
  }

  @Override
  public Result onNullStatementBegin(final VerilogNullStatement node) {
    return Result.OK;
  }

  @Override
  public Result onNullStatementEnd(final VerilogNullStatement node) {
    return Result.OK;
  }

  @Override
  public Result onPathDeclarationBegin(final VerilogPathDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onPathDeclarationEnd(final VerilogPathDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onPortBegin(final VerilogPort node) {
    return Result.OK;
  }

  @Override
  public Result onPortEnd(final VerilogPort node) {
    return Result.OK;
  }

  @Override
  public Result onPortConnectionBegin(final VerilogPortConnection node) {
    return Result.OK;
  }

  @Override
  public Result onPortConnectionEnd(final VerilogPortConnection node) {
    return Result.OK;
  }

  @Override
  public Result onProcedureBegin(final VerilogProcedure node) {
    return Result.OK;
  }

  @Override
  public Result onProcedureEnd(final VerilogProcedure node) {
    return Result.OK;
  }

  @Override
  public Result onPulseStyleBegin(final VerilogPulseStyle node) {
    return Result.OK;
  }

  @Override
  public Result onPulseStyleEnd(final VerilogPulseStyle node) {
    return Result.OK;
  }

  @Override
  public Result onShowCancelledBegin(final VerilogShowCancelled node) {
    return Result.OK;
  }

  @Override
  public Result onShowCancelledEnd(final VerilogShowCancelled node) {
    return Result.OK;
  }

  @Override
  public Result onSpecifyBegin(final VerilogSpecify node) {
    return Result.OK;
  }

  @Override
  public Result onSpecifyEnd(final VerilogSpecify node) {
    return Result.OK;
  }

  @Override
  public Result onTableBegin(final VerilogTable node) {
    return Result.OK;
  }

  @Override
  public Result onTableEnd(final VerilogTable node) {
    return Result.OK;
  }

  @Override
  public Result onTableEntryBegin(final VerilogTableEntry node) {
    return Result.OK;
  }

  @Override
  public Result onTableEntryEnd(final VerilogTableEntry node) {
    return Result.OK;
  }

  @Override
  public Result onTaskStatementBegin(final VerilogTaskStatement node) {
    return Result.OK;
  }

  @Override
  public Result onTaskStatementEnd(final VerilogTaskStatement node) {
    return Result.OK;
  }

  @Override
  public Result onTriggerStatementBegin(final VerilogTriggerStatement node) {
    return Result.OK;
  }

  @Override
  public Result onTriggerStatementEnd(final VerilogTriggerStatement node) {
    return Result.OK;
  }

  @Override
  public Result onWaitStatementBegin(final VerilogWaitStatement node) {
    return Result.OK;
  }

  @Override
  public Result onWaitStatementEnd(final VerilogWaitStatement node) {
    return Result.OK;
  }

  @Override
  public Result onSequenceDeclarationBegin(final SvaSequenceDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onSequenceDeclarationEnd(final SvaSequenceDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onPropertyDeclarationBegin(final SvaPropertyDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onPropertyDeclarationEnd(final SvaPropertyDeclaration node) {
    return Result.OK;
  }

  @Override
  public Result onAssertionStatementBegin(final SvaAssertionStatement node) {
    return Result.OK;
  }

  @Override
  public Result onAssertionStatementEnd(final SvaAssertionStatement node) {
    return Result.OK;
  }

  @Override
  public Result onSvaInstanceBegin(final SvaInstantiation node) {
    return Result.OK;
  }

  @Override
  public Result onSvaInstanceEnd(final SvaInstantiation node) {
    return Result.OK;
  }

}
