/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;
import ru.ispras.verilog.parser.util.VerilogBenchmarkTest;

import java.io.File;
import java.nio.file.Paths;

/**
 * Verilog2Smv\VIS benchmark jUnit test cases.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class Verilog2SmvVisTestSuite extends VerilogBenchmarkTest {

  /* Am2901 */

  @Test
  public void runTest_Am2901_am2901() {
    runTest("am2901", Paths.get("Am2901", "am2901.v").toString());
  }

  /* Am2910 */

  @Test
  public void runTest_Am2910_am2910_p1() {
    runTest("am2910", Paths.get("Am2910", "am2910_p1.v").toString());
  }

  @Test
  public void runTest_Am2910_am2910_p2() {
    runTest("am2910", Paths.get("Am2910", "am2910_p2.v").toString());
  }

  @Test
  public void runTest_Am2910_am2910_p3() {
    runTest("am2910", Paths.get("Am2910", "am2910_p3.v").toString());
  }

  @Test
  public void runTest_Am2910_am2910_p4() {
    runTest("am2910", Paths.get("Am2910", "am2910_p4.v").toString());
  }

  /* Bpb */

  @Test
  public void runTest_Bpb_bpbs_p1() {
    runTest("branchPredictionBuffer", Paths.get("Bpb", "bpbs_p1.v").toString());
  }

  @Test
  public void runTest_Bpb_bpbs_p2() {
    runTest("branchPredictionBuffer", Paths.get("Bpb", "bpbs_p2.v").toString());
  }

  @Test
  public void runTest_Bpb_bpbs_p3() {
    runTest("branchPredictionBuffer", Paths.get("Bpb", "bpbs_p3.v").toString());
  }

  @Test
  public void runTest_Bpb_bpbs_p4() {
    runTest("branchPredictionBuffer", Paths.get("Bpb", "bpbs_p4.v").toString());
  }

  /* BufAl */

  @Test
  public void runTest_BufAl_buf_bug() {
    runTest("buffer_alloc", Paths.get("BufAl", "buf_bug.v").toString());
  }

  @Test
  public void runTest_BufAl_bufferAlloc() {
    runTest("buffer_alloc", Paths.get("BufAl", "bufferAlloc.v").toString());
  }

  /* Field */

  @Test
  public void runTest_Field_field5() {
    runTest("field5", Paths.get("Field", "field5.v").toString());
  }

  /* FIFOs */

  @Test
  public void runTest_Fifos_Fifos() {
    runTest("compareFIFOs", Paths.get("FIFOs", "FIFOs.v").toString());
  }

  /* FourByFour */

  @Test
  public void runTest_FourByFour_two_p1() {
    runTest("twoByFour", Paths.get("FourByFour", "two_p1.v").toString());
  }

  @Test
  public void runTest_FourByFour_two_p2() {
    runTest("twoByFour", Paths.get("FourByFour", "two_p2.v").toString());
  }

  /* Ibuf */

  @Test
  public void runTest_Ibuf_ibuf() {
    runTest("iqc", Paths.get("Ibuf", "ibuf.v").toString());
  }

  /* Miim */

  @Test
  public void runTest_Miim_vMiim_p1() {
    runTest("miim", Paths.get("Miim", "vMiim_p1.v").toString());
  }

  @Test
  public void runTest_Miim_vMiim_p2() {
    runTest("miim", Paths.get("Miim", "vMiim_p2.v").toString());
  }

  /* Palu */

  @Test
  public void runTest_Palu_palu() {
    runTest("palu", Paths.get("Palu", "palu.v").toString());
  }

  /* Rotate */

  @Test
  public void runTest_Rotate_rotate32() {
    runTest("rotate", Paths.get("Rotate", "rotate32.v").toString());
  }

  /* s1269 */

  @Test
  public void runTest_s1269_s1269_p1() {
    runTest("s1269", Paths.get("s1269", "s1269b_p1.v").toString());
  }

  @Test
  public void runTest_s1269_s1269_p2() {
    runTest("s1269", Paths.get("s1269", "s1269b_p2.v").toString());
  }

  @Test
  public void runTest_s1269_s1269_p3() {
    runTest("s1269", Paths.get("s1269", "s1269b_p3.v").toString());
  }

  @Test
  public void runTest_s1269_s1269_p4() {
    runTest("s1269", Paths.get("s1269", "s1269b_p4.v").toString());
  }

  @Test
  public void runTest_s1269_s1269_p5() {
    runTest("s1269", Paths.get("s1269", "s1269b_p5.v").toString());
  }

  /* Sampleq */
  @Test
  public void runTest_Sampleq_twoFifo1_p1() {
    runTest("sampleq", Paths.get("Sampleq", "twoFifo1_p1.v").toString());
  }

  @Test
  public void runTest_Sampleq_twoFifo1_p2() {
    runTest("sampleq", Paths.get("Sampleq", "twoFifo1_p2.v").toString());
  }

  @Test
  public void runTest_Sampleq_twoFifo1_p3() {
    runTest("sampleq", Paths.get("Sampleq", "twoFifo1_p3.v").toString());
  }

  /* Silver-bcu */

  @Test
  public void runTest_Silver_bcu_bcuvis32() {
    runTest("bcu", Paths.get("Silver-bcu", "bcuvis32.v").toString());
  }

  /* Silver-fru */

  @Test
  public void runTest_Silver_fru_includes_const() {
    runTest(null, Paths.get("Silver-fru", "includes", "const.v").toString());
  }

  @Test
  public void runTest_Silver_fru_includes_decode() {
    runTest(null, Paths.get("Silver-fru", "includes", "decode.v").toString());
  }

  @Test
  public void runTest_Silver_fru_includes_opcode() {
    runTest(null, Paths.get("Silver-fru", "includes", "opcode.v").toString());
  }

  @Test
  public void runTest_Silver_fru_fru32_p1() {
    runTest("fru","--include-path",
        getTestDirPath() + Paths.get("Silver-fru").toString(),
        Paths.get("Silver-fru", "fru32_p1.v").toString());
  }

  @Test
  public void runTest_Silver_fru_fru32_p2() {
    runTest("fru",
        "--include-path",
        getTestDirPath() + Paths.get("Silver-fru").toString(),
        Paths.get("Silver-fru", "fru32_p2.v").toString());
  }

  @Test
  public void runTest_Silver_fru_fru32_p3() {
    runTest("fru",
        "--include-path",
        getTestDirPath() + Paths.get("Silver-fru").toString(),
        Paths.get("Silver-fru", "fru32_p3.v").toString());
  }

  /* Spinner */

  @Test
  public void runTest_Spinner_spinner32() {
    runTest("spinner", Paths.get("Spinner", "spinner32.v").toString());
  }

  /* Vlunc */

  @Test
  public void runTest_Vlunc_vlunc() {
    runTest("lunc", Paths.get("Vlunc", "vlunc.v").toString());
  }

  /* Vsa16 */

  @Test
  public void runTest_Vsa16_vsa16a_p1() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p1.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p2() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p2.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p3() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p3.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p4() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p4.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p5() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p5.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p6() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p6.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p7() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p7.v").toString());
  }

  @Test
  public void runTest_Vsa16_vsa16a_p8() {
    runTest("vsa16a", Paths.get("Vsa16", "vsa16a_p8.v").toString());
  }

  /* VsaR */

  @Test
  public void runTest_VsaR_vsaR_p01() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p01.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p02() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p02.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p03() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p03.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p04() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p04.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p05() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p05.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p06() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p06.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p07() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p07.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p08() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p08.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p09() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p09.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p10() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p10.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p11() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p11.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p12() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p12.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p13() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p13.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p14() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p14.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p15() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p15.v").toString());
  }

  @Test
  public void runTest_VsaR_vsaR_p16() {
    runTest("vsaR", Paths.get("VsaR", "vsaR_p16.v").toString());
  }

  @Override
  public String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "hdl-benchmarks", "hdl", "verilog2smv", "VIS")
            .toString()
        + File.separator;
  }
}
