/*
 * Copyright 2019-2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;
import ru.ispras.verilog.parser.util.VerilogBenchmarkTest;

import java.io.File;
import java.nio.file.Paths;

/**
 * QUIP (Quartus University Interface Program) benchmarks based test cases.
 *
 * @author <a href="mailto:chudnov@ispras.ru">Chudnov Maxim</a>
 */
public class VerilogQuipTestSuite extends VerilogBenchmarkTest {

  /*
   * barrel16
   */

  @Test
  public void runTest_barrel16() {
    runTest("barrel16", Paths.get("barrel16", "barrel16.v").toString());
  }

  /*
   * barrel16a
   */

  @Test
  public void runTest_barrel16a() {
    runTest("barrel16a", Paths.get("barrel16a", "barrel16a.v").toString());
  }

  /*
   * barrel32
   */

  @Test
  public void runTest_barrel32() {
    runTest("barrel32", Paths.get("barrel32", "barrel32.v").toString());
  }

  /*
   * barrel64
   */

  @Test
  public void runTest_barrel64() {
    runTest("barrel64", Paths.get("barrel64", "barrel64.v").toString());
  }

  /*
   * fip_cordic_cla
   */

  // TODO: ERROR: [Internal] Descriptor for 'Adder.Add.shiftedcarry' has not been found
  /*@Test
  public void runTest_cla_adder() {
    runTest("Adder",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "adder.v").toString(),
        Paths.get("fip_cordic_cla", "busmux21.v").toString(),
        Paths.get("fip_cordic_cla", "cla.v").toString(),
        Paths.get("fip_cordic_cla", "compl.v").toString());
  }*/

  @Test
  public void runTest_bitmux21_cla() {
    runTest("BitMux2_1",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "bitmux21.v").toString());
  }

  @Test
  public void runTest_busmux21_cla() {
    runTest("BusMux2_1",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "busmux21.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'CLA.shiftedcarry' has not been found
  /*@Test
  public void runTest_cla() {
    runTest("CLA",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "cla.v").toString());
  }*/

  @Test
  public void runTest_compl_cla() {
    runTest("complement",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "compl.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'fip_cordic_cla.AddX.Add.shiftedcarry' has not been found
  /*@Test
  public void runTest_fip_cordic_cla() {
    runTest("fip_cordic_cla",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "fip_cordic_cla.v").toString(),
        Paths.get("fip_cordic_cla", "mem.v").toString(),
        Paths.get("fip_cordic_cla", "shifter.v").toString(),
        Paths.get("fip_cordic_cla", "adder.v").toString(),
        Paths.get("fip_cordic_cla", "busmux21.v").toString(),
        Paths.get("fip_cordic_cla", "cla.v").toString(),
        Paths.get("fip_cordic_cla", "compl.v").toString());
  }*/

  @Test
  public void runTest_mem_cla() {
    runTest("MEM",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "mem.v").toString());
  }

  @Test
  public void runTest_shifter_cla() {
    runTest("shifter",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_cla").toString(),
        Paths.get("fip_cordic_cla", "shifter.v").toString());
  }

  /*
   * fip_cordic_rca
   */

  @Test
  public void runTest_adder() {
    runTest("Adder",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "adder.v").toString(),
        Paths.get("fip_cordic_rca", "busmux21.v").toString(),
        Paths.get("fip_cordic_rca", "rca.v").toString(),
        Paths.get("fip_cordic_rca", "compl.v").toString());
  }

  @Test
  public void runTest_bitmux21_rca() {
    runTest("BitMux2_1", Paths.get("fip_cordic_rca", "bitmux21.v").toString());
  }

  @Test
  public void runTest_busmux21_rca() {
    runTest("BusMux2_1",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "busmux21.v").toString());
  }

  @Test
  public void runTest_compl_rca() {
    runTest("complement",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "compl.v").toString());
  }

  @Test
  public void runTest_fip_cordic_rca() {
    runTest("fip_cordic_rca", "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "fip_cordic_rca.v").toString(),
        Paths.get("fip_cordic_rca", "mem.v").toString(),
        Paths.get("fip_cordic_rca", "shifter.v").toString(),
        Paths.get("fip_cordic_rca", "adder.v").toString(),
        Paths.get("fip_cordic_rca", "busmux21.v").toString(),
        Paths.get("fip_cordic_rca", "rca.v").toString(),
        Paths.get("fip_cordic_rca", "compl.v").toString());
  }

  @Test
  public void runTest_mem_rca() {
    runTest("MEM",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "mem.v").toString());
  }

  @Test
  public void runTest_rca() {
    runTest("RCA",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "rca.v").toString());
  }

  @Test
  public void runTest_shifter_rca() {
    runTest("shifter",
        "--include-path",
        Paths.get(getTestDirPath(), "fip_cordic_rca").toString(),
        Paths.get("fip_cordic_rca", "shifter.v").toString());
  }

  /*
   * fip_risc8
   */

  @Test
  public void runTest_alu() {
    runTest("alu", Paths.get("fip_risc8", "alu.v").toString());
  }

  @Test
  public void runTest_dram() {
    runTest("dram", Paths.get("fip_risc8", "dram.v").toString());
  }

  @Test
  public void runTest_exp() {
    runTest("exp", Paths.get("fip_risc8", "exp.v").toString());
  }

  @Test
  public void runTest_fip_risc8() {
    runTest("fip_risc8", Paths.get("fip_risc8", "fip_risc8.v").toString(),
        Paths.get("fip_risc8", "regs.v").toString(),
        Paths.get("fip_risc8", "alu.v").toString(),
        Paths.get("fip_risc8", "idec.v").toString(),
        Paths.get("fip_risc8", "dram.v").toString());
  }

  @Test
  public void runTest_idec() {
    runTest("idec", Paths.get("fip_risc8", "idec.v").toString());
  }

  @Test
  public void runTest_pram() {
    runTest("pram", Paths.get("fip_risc8", "pram.v").toString());
  }

  @Test
  public void runTest_regs() {
    runTest("regs", Paths.get("fip_risc8", "regs.v").toString(),
        Paths.get("fip_risc8", "dram.v").toString());
  }

  /*
   * mux8_64bit
   */

  @Test
  public void runTest_mux8_64bit() {
    runTest("mux8_64bit", Paths.get("mux8_64bit", "mux8_64bit.v").toString());
  }

  /*
   * mux8_128bit
   */

  @Test
  public void runTest_mux8_128bit() {
    runTest("mux8_128bit", Paths.get("mux8_128bit", "mux8_128bit.v").toString());
  }

  /*
   * mux32_16bit
   */

  @Test
  public void runTest_mux32_16bit() {
    runTest("mux32_16bit", Paths.get("mux32_16bit", "mux32_16bit.v").toString());
  }

  /*
   * mux64_16bit
   */

  @Test
  public void runTest_mux64_16bit() {
    runTest("mux64_16bit", Paths.get("mux64_16bit", "mux64_16bit.v").toString());
  }

  /*
   * nut_000
   */

  @Test
  public void runTest_dffea_lib() {
    runTest("dffea", Paths.get("nut_000", "dffea_lib.v").toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 2255
  /*@Test
  public void runTest_nut_000() {
    runTest("nut_000", Paths.get("nut_000", "nut_000.v").toString(),
        Paths.get("nut_000", "carry.v").toString(),
        Paths.get("nut_000", "carry_sum.v").toString(),
        Paths.get("nut_000", "cascade.v").toString(),
        Paths.get("nut_000", "dffea_lib.v").toString(),
        Paths.get("nut_000", "lcell.v").toString(),
        Paths.get("nut_000", "lut_output.v").toString(),
        Paths.get("nut_000", "nut_000_lut.v").toString(),
        Paths.get("nut_000", "nut_000_m245.v").toString(),
        Paths.get("nut_000", "nut_000_word_swap.v").toString(),
        Paths.get("nut_000", "soft.v").toString(),
        Paths.get("fip_risc8", "exp.v").toString());
  }*/

  @Test
  public void runTest_nut_000_lut() {
    runTest("nut_000_lut",
        Paths.get("nut_000", "nut_000_lut.v").toString(),
        Paths.get("nut_000", "lut_input.v").toString(),
        Paths.get("nut_000", "lut_output.v").toString());
  }

  @Test
  public void runTest_nut_000_m245() {
    runTest("nut_000_m245", Paths.get("nut_000", "nut_000_m245.v").toString());
  }

  @Test
  public void runTest_nut_000_word_swap() {
    runTest("nut_000_word_swap", Paths.get("nut_000", "nut_000_word_swap.v").toString());
  }

  /*
   * nut_001
   */

  @Test
  public void runTest_dffea_lib_001() {
    runTest("dffea", Paths.get("nut_001", "dffea_lib.v").toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 2200
  /*@Test
  public void runTest_nut_001() {
    runTest("nut_001",
        "--library-file",
        Paths.get(getTestDirPath(), "nut_001", "include.lib").toString(),
        Paths.get("nut_001", "nut_001.v").toString(),
        Paths.get("nut_001", "carry_sum.v").toString(),
        Paths.get("nut_001", "cascade.v").toString(),
        Paths.get("nut_001", "dffea_lib.v").toString(),
        Paths.get("nut_001", "exp.v").toString(),
        Paths.get("nut_001", "lcell.v").toString(),
        Paths.get("nut_001", "lut_output.v").toString(),
        Paths.get("nut_001", "nut_001_deuteronomy.v").toString(),
        Paths.get("nut_001", "nut_001_lut.v").toString(),
        Paths.get("nut_001", "nut_001_rom_tattoo.v").toString(),
        Paths.get("nut_001", "soft.v").toString());
  }*/

  @Test
  public void runTest_nut_001_deuteronomy() {
    runTest("nut_001_deuteronomy", Paths.get("nut_001", "nut_001_deuteronomy.v").toString());
  }

  @Test
  public void runTest_nut_001_lut() {
    runTest("nut_001_lut",
        Paths.get("nut_001", "nut_001_lut.v").toString(),
        Paths.get("nut_001", "lut_input.v").toString(),
        Paths.get("nut_001", "lut_output.v").toString());
  }

  @Test
  public void runTest_nut_001_rom_tattoo() {
    runTest("nut_001_rom_tattoo",
        "--library-file",
        Paths.get(getTestDirPath(), "nut_001", "include.lib").toString(),
        Paths.get("nut_001", "nut_001_rom_tattoo.v").toString());
  }

  /*
   * nut_002
   */

  @Test
  public void runTest_dffea_lib_002() {
    runTest("dffea", Paths.get("nut_002", "dffea_lib.v").toString());
  }

  //TODO: ERROR: Starting points limit has been exhausted: 1080
  /*@Test
  public void runTest_nut_002() {
    runTest("nut_002",
        Paths.get("nut_002", "nut_002.v").toString(),
        Paths.get("nut_002", "carry.v").toString(),
        Paths.get("nut_002", "carry_sum.v").toString(),
        Paths.get("nut_002", "dffea_lib.v").toString(),
        Paths.get("nut_002", "lut_output.v").toString(),
        Paths.get("nut_002", "nut_002_hagrid_neon.v").toString(),
        Paths.get("nut_002", "nut_002_lut.v").toString(),
        Paths.get("nut_002", "nut_002_m270.v").toString(),
        Paths.get("nut_002", "nut_002_m47.v").toString(),
        Paths.get("nut_002", "nut_002_rarest.v").toString(),
        Paths.get("nut_002", "soft.v").toString());
  }*/

  @Test
  public void runTest_nut_002_hagrid_neon() {
    runTest("nut_002_hagrid_neon", Paths.get("nut_002", "nut_002_hagrid_neon.v").toString());
  }

  @Test
  public void runTest_nut_002_lut() {
    runTest("nut_002_lut",
        Paths.get("nut_002", "nut_002_lut.v").toString(),
        Paths.get("nut_002", "lut_input.v").toString(),
        Paths.get("nut_002", "lut_output.v").toString());
  }

  @Test
  public void runTest_nut_002_m47() {
    runTest("nut_002_m47", Paths.get("nut_002", "nut_002_m47.v").toString());
  }

  @Test
  public void runTest_nut_002_m270() {
    runTest("nut_002_m270", Paths.get("nut_002", "nut_002_m270.v").toString());
  }

  @Test
  public void runTest_nut_002_rarest() {
    runTest("nut_002_rarest", Paths.get("nut_002", "nut_002_rarest.v").toString());
  }

  /*
   * nut_003
   */

  @Test
  public void runTest_dffea_lib_003() {
    runTest("dffea", Paths.get("nut_003", "dffea_lib.v").toString());
  }

  //TODO: ERROR: Starting points limit has been exhausted: 1040
  /*@Test
  public void runTest_nut_003() {
    runTest("nut_003", Paths.get("nut_003", "nut_003.v").toString(),
        Paths.get("nut_003", "carry.v").toString(),
        Paths.get("nut_003", "carry_sum.v").toString(),
        Paths.get("nut_003", "cascade.v").toString(),
        Paths.get("nut_003", "dffea_lib.v").toString(),
        Paths.get("nut_003", "lut_output.v").toString(),
        Paths.get("nut_003", "nut_003_lut.v").toString(),
        Paths.get("nut_003", "nut_003_m178.v").toString(),
        Paths.get("nut_003", "nut_003_spastic_pilot.v").toString());
  }*/

  @Test
  public void runTest_nut_003_lut() {
    runTest("nut_003_lut",
        Paths.get("nut_003", "nut_003_lut.v").toString(),
        Paths.get("nut_003", "lut_input.v").toString(),
        Paths.get("nut_003", "lut_output.v").toString());
  }

  @Test
  public void runTest_nut_003_m178() {
    runTest("nut_003_m178", Paths.get("nut_003", "nut_003_m178.v").toString());
  }

  @Test
  public void runTest_nut_003_spastic_pilot() {
    runTest("nut_003_spastic_pilot", Paths.get("nut_003", "nut_003_spastic_pilot.v").toString());
  }

  /*
   * nut_004
   */

  @Test
  public void runTest_dffea_lib_004() {
    runTest("dffea", Paths.get("nut_003", "dffea_lib.v").toString());
  }

  //TODO: Starting points limit has been exhausted: 1305
  /*@Test
  public void runTest_nut_004() {
    runTest("nut_004",
        "--library-file",
        Paths.get(getTestDirPath(), "nut_004", "include.lib").toString(),
        Paths.get("nut_004", "nut_004.v").toString(),
        Paths.get("nut_004", "carry.v").toString(),
        Paths.get("nut_004", "carry_sum.v").toString(),
        Paths.get("nut_004", "cascade.v").toString(),
        Paths.get("nut_004", "dffea_lib.v").toString(),
        Paths.get("nut_004", "exp.v").toString(),
        Paths.get("nut_004", "lcell.v").toString(),
        Paths.get("nut_004", "lut_output.v").toString(),
        Paths.get("nut_004", "nut_004_lut.v").toString(),
        Paths.get("nut_004", "nut_004_m149.v").toString(),
        Paths.get("nut_004", "nut_004_rom_jesus.v").toString(),
        Paths.get("nut_004", "nut_004_word_swap.v").toString(),
        Paths.get("nut_004", "soft.v").toString());
  }*/

  @Test
  public void runTest_nut_004_lut() {
    runTest("nut_004_lut",
        Paths.get("nut_004", "nut_004_lut.v").toString(),
        Paths.get("nut_004", "lut_input.v").toString(),
        Paths.get("nut_004", "lut_output.v").toString());
  }

  @Test
  public void runTest_nut_004_m149() {
    runTest("nut_004_m149", Paths.get("nut_004", "nut_004_m149.v").toString());
  }

  @Test
  public void runTest_nut_004_rom_jesus() {
    runTest("nut_004_rom_jesus",
        "--library-file",
        Paths.get(getTestDirPath(), "nut_004", "include.lib").toString(),
        Paths.get("nut_004", "nut_004_rom_jesus.v").toString());
  }

  @Test
  public void runTest_nut_004_word_swap() {
    runTest("nut_004_word_swap", Paths.get("nut_004", "nut_004_word_swap.v").toString());
  }

  /*
   * oc_aes_core
   */

  @Test
  public void runTest_aes_key_expand_128() {
    runTest("aes_key_expand_128",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core").toString(),
        Paths.get("oc_aes_core", "aes_key_expand_128.v").toString(),
        Paths.get("oc_aes_core", "aes_sbox.v").toString(),
        Paths.get("oc_aes_core", "aes_rcon.v").toString());
  }

  @Test
  public void runTest_aes_rcon() {
    runTest("aes_rcon",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core").toString(),
        Paths.get("oc_aes_core", "aes_rcon.v").toString());
  }

  @Test
  public void runTest_aes_sbox() {
    runTest("aes_sbox",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core").toString(),
        Paths.get("oc_aes_core", "aes_sbox.v").toString());
  }

  @Test
  public void runTest_oc_aes_core() {
    runTest("oc_aes_core",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core").toString(),
        Paths.get("oc_aes_core", "oc_aes_core.v").toString(),
        Paths.get("oc_aes_core", "aes_key_expand_128.v").toString(),
        Paths.get("oc_aes_core", "aes_sbox.v").toString(),
        Paths.get("oc_aes_core", "aes_rcon.v").toString());
  }

  /*
   * oc_aes_core_inv
   */

  @Test
  public void runTest_aes_inv_sbox() {
    runTest("aes_inv_sbox",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core_inv").toString(),
        Paths.get("oc_aes_core_inv", "aes_inv_sbox.v").toString());
  }

  @Test
  public void runTest_aes_key_expand_128_inv() {
    runTest("aes_key_expand_128",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core_inv").toString(),
        Paths.get("oc_aes_core_inv", "aes_key_expand_128.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_sbox.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_rcon.v").toString());
  }

  @Test
  public void runTest_aes_rcon_inv() {
    runTest("aes_rcon",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core_inv").toString(),
        Paths.get("oc_aes_core_inv", "aes_rcon.v").toString());
  }

  @Test
  public void runTest_aes_sbox_inv() {
    runTest("aes_sbox",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core_inv").toString(),
        Paths.get("oc_aes_core_inv", "aes_sbox.v").toString());
  }

  @Test
  public void runTest_oc_aes_core_inv() {
    runTest("oc_aes_core_inv",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aes_core_inv").toString(),
        Paths.get("oc_aes_core_inv", "oc_aes_core_inv.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_key_expand_128.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_sbox.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_rcon.v").toString(),
        Paths.get("oc_aes_core_inv", "aes_inv_sbox.v").toString());
  }

  /*
   * oc_aquarius
   */

  @Test
  public void runTest_clk_gate() {
    runTest("clk_gate",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "clk_gate.v").toString());
  }

  @Test
  public void runTest_cpu() {
    runTest("cpu",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "cpu.v").toString(),
        Paths.get("oc_aquarius", "mem.v").toString(),
        Paths.get("oc_aquarius", "decode.v").toString(),
        Paths.get("oc_aquarius", "mult.v").toString(),
        Paths.get("oc_aquarius", "datapath.v").toString(),
        Paths.get("oc_aquarius", "register.v").toString());
  }

  @Test
  public void runTest_datapath() {
    runTest("datapath",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "datapath.v").toString(),
        Paths.get("oc_aquarius", "register.v").toString());
  }

  @Test
  public void runTest_decode() {
    runTest("decode",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "decode.v").toString());
  }

  @Test
  public void runTest_mem() {
    runTest("mem",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "mem.v").toString());
  }

  @Test
  public void runTest_memory() {
    runTest("memory",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "memory.v").toString());
  }

  @Test
  public void runTest_mult() {
    runTest("mult",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "mult.v").toString());
  }

  @Test
  public void runTest_oc_aquarius() {
    runTest("oc_aquarius",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "oc_aquarius.v").toString(),
        Paths.get("oc_aquarius", "cpu.v").toString(),
        Paths.get("oc_aquarius", "memory.v").toString(),
        Paths.get("oc_aquarius", "pio.v").toString(),
        Paths.get("oc_aquarius", "uart.v").toString(),
        Paths.get("oc_aquarius", "sys.v").toString(),
        Paths.get("oc_aquarius", "mem.v").toString(),
        Paths.get("oc_aquarius", "decode.v").toString(),
        Paths.get("oc_aquarius", "mult.v").toString(),
        Paths.get("oc_aquarius", "datapath.v").toString(),
        Paths.get("oc_aquarius", "sasc_top.v").toString(),
        Paths.get("oc_aquarius", "sasc_brg.v").toString(),
        Paths.get("oc_aquarius", "clk_gate.v").toString(),
        Paths.get("oc_aquarius", "register.v").toString(),
        Paths.get("oc_aquarius", "sasc_fifo4.v").toString());
  }

  @Test
  public void runTest_pio() {
    runTest("pio",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "pio.v").toString());
  }

  @Test
  public void runTest_register() {
    runTest("register",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "register.v").toString());
  }

  @Test
  public void runTest_sasc_brg() {
    runTest("sasc_brg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "sasc_brg.v").toString());
  }

  @Test
  public void runTest_sasc_fifo4() {
    runTest("sasc_fifo4",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "sasc_fifo4.v").toString());
  }

  @Test
  public void runTest_sasc_top() {
    runTest("sasc_top",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "sasc_top.v").toString(),
        Paths.get("oc_aquarius", "sasc_fifo4.v").toString());
  }

  @Test
  public void runTest_sys() {
    runTest("sys",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "sys.v").toString(),
        Paths.get("oc_aquarius", "clk_gate.v").toString());
  }

  @Test
  public void runTest_uart() {
    runTest("uart", "--include-path", Paths.get(getTestDirPath(), "oc_aquarius").toString(),
        Paths.get("oc_aquarius", "uart.v").toString(),
        Paths.get("oc_aquarius", "sasc_top.v").toString(),
        Paths.get("oc_aquarius", "sasc_brg.v").toString(),
        Paths.get("oc_aquarius", "sasc_fifo4.v").toString());
  }

  /*
   * oc_des_area_opt
   */

  @Test
  public void runTest_crp() {
    runTest("crp", Paths.get("oc_des_area_opt", "crp.v").toString(),
        Paths.get("oc_des_area_opt", "sbox1.v").toString(),
        Paths.get("oc_des_area_opt", "sbox2.v").toString(),
        Paths.get("oc_des_area_opt", "sbox3.v").toString(),
        Paths.get("oc_des_area_opt", "sbox4.v").toString(),
        Paths.get("oc_des_area_opt", "sbox5.v").toString(),
        Paths.get("oc_des_area_opt", "sbox6.v").toString(),
        Paths.get("oc_des_area_opt", "sbox7.v").toString(),
        Paths.get("oc_des_area_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_des_test() {
    runTest("test", Paths.get("oc_des_area_opt", "des_test.v").toString(),
        Paths.get("oc_des_area_opt", "oc_des_area_opt.v").toString(),
        Paths.get("oc_des_area_opt", "crp.v").toString(),
        Paths.get("oc_des_area_opt", "key_sel.v").toString(),
        Paths.get("oc_des_area_opt", "sbox1.v").toString(),
        Paths.get("oc_des_area_opt", "sbox2.v").toString(),
        Paths.get("oc_des_area_opt", "sbox3.v").toString(),
        Paths.get("oc_des_area_opt", "sbox4.v").toString(),
        Paths.get("oc_des_area_opt", "sbox5.v").toString(),
        Paths.get("oc_des_area_opt", "sbox6.v").toString(),
        Paths.get("oc_des_area_opt", "sbox7.v").toString(),
        Paths.get("oc_des_area_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_key_sel() {
    runTest("key_sel", Paths.get("oc_des_area_opt", "key_sel.v").toString());
  }

  @Test
  public void runTest_oc_des_area_opt() {
    runTest("oc_des_area_opt",
        Paths.get("oc_des_area_opt", "oc_des_area_opt.v").toString(),
        Paths.get("oc_des_area_opt", "crp.v").toString(),
        Paths.get("oc_des_area_opt", "key_sel.v").toString(),
        Paths.get("oc_des_area_opt", "sbox1.v").toString(),
        Paths.get("oc_des_area_opt", "sbox2.v").toString(),
        Paths.get("oc_des_area_opt", "sbox3.v").toString(),
        Paths.get("oc_des_area_opt", "sbox4.v").toString(),
        Paths.get("oc_des_area_opt", "sbox5.v").toString(),
        Paths.get("oc_des_area_opt", "sbox6.v").toString(),
        Paths.get("oc_des_area_opt", "sbox7.v").toString(),
        Paths.get("oc_des_area_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_sbox1() {
    runTest("sbox1", Paths.get("oc_des_area_opt", "sbox1.v").toString());
  }

  @Test
  public void runTest_sbox2() {
    runTest("sbox2", Paths.get("oc_des_area_opt", "sbox2.v").toString());
  }

  @Test
  public void runTest_sbox3() {
    runTest("sbox3", Paths.get("oc_des_area_opt", "sbox3.v").toString());
  }

  @Test
  public void runTest_sbox4() {
    runTest("sbox4", Paths.get("oc_des_area_opt", "sbox4.v").toString());
  }

  @Test
  public void runTest_sbox5() {
    runTest("sbox5", Paths.get("oc_des_area_opt", "sbox5.v").toString());
  }

  @Test
  public void runTest_sbox6() {
    runTest("sbox6", Paths.get("oc_des_area_opt", "sbox6.v").toString());
  }

  @Test
  public void runTest_sbox7() {
    runTest("sbox7", Paths.get("oc_des_area_opt", "sbox7.v").toString());
  }

  @Test
  public void runTest_sbox8() {
    runTest("sbox8", Paths.get("oc_des_area_opt", "sbox8.v").toString());
  }

  /*
   * oc_des_des3area
   */

  @Test
  public void runTest_crp_3area() {
    runTest("crp", Paths.get("oc_des_des3area", "crp.v").toString(),
        Paths.get("oc_des_des3area", "sbox1.v").toString(),
        Paths.get("oc_des_des3area", "sbox2.v").toString(),
        Paths.get("oc_des_des3area", "sbox3.v").toString(),
        Paths.get("oc_des_des3area", "sbox4.v").toString(),
        Paths.get("oc_des_des3area", "sbox5.v").toString(),
        Paths.get("oc_des_des3area", "sbox6.v").toString(),
        Paths.get("oc_des_des3area", "sbox7.v").toString(),
        Paths.get("oc_des_des3area", "sbox8.v").toString());
  }

  @Test
  public void runTest_des3() {
    runTest("des3", Paths.get("oc_des_des3area", "des3.v").toString(),
        Paths.get("oc_des_des3area", "oc_des_des3area.v").toString(),
        Paths.get("oc_des_des3area", "crp.v").toString(),
        Paths.get("oc_des_des3area", "key_sel3.v").toString(),
        Paths.get("oc_des_des3area", "sbox1.v").toString(),
        Paths.get("oc_des_des3area", "sbox2.v").toString(),
        Paths.get("oc_des_des3area", "sbox3.v").toString(),
        Paths.get("oc_des_des3area", "sbox4.v").toString(),
        Paths.get("oc_des_des3area", "sbox5.v").toString(),
        Paths.get("oc_des_des3area", "sbox6.v").toString(),
        Paths.get("oc_des_des3area", "sbox7.v").toString(),
        Paths.get("oc_des_des3area", "sbox8.v").toString());
  }

  @Test
  public void runTest_key_sel3() {
    runTest("key_sel3", Paths.get("oc_des_des3area", "key_sel3.v").toString());
  }

  @Test
  public void runTest_oc_des_des3area() {
    runTest("oc_des_des3area",
        Paths.get("oc_des_des3area", "oc_des_des3area.v").toString(),
        Paths.get("oc_des_des3area", "crp.v").toString(),
        Paths.get("oc_des_des3area", "key_sel3.v").toString(),
        Paths.get("oc_des_des3area", "des3.v").toString(),
        Paths.get("oc_des_des3area", "sbox1.v").toString(),
        Paths.get("oc_des_des3area", "sbox2.v").toString(),
        Paths.get("oc_des_des3area", "sbox3.v").toString(),
        Paths.get("oc_des_des3area", "sbox4.v").toString(),
        Paths.get("oc_des_des3area", "sbox5.v").toString(),
        Paths.get("oc_des_des3area", "sbox6.v").toString(),
        Paths.get("oc_des_des3area", "sbox7.v").toString(),
        Paths.get("oc_des_des3area", "sbox8.v").toString());
  }

  @Test
  public void runTest_sbox1_3area() {
    runTest("sbox1", Paths.get("oc_des_des3area", "sbox1.v").toString());
  }

  @Test
  public void runTest_sbox2_3area() {
    runTest("sbox2", Paths.get("oc_des_des3area", "sbox2.v").toString());
  }

  @Test
  public void runTest_sbox3_3area() {
    runTest("sbox3", Paths.get("oc_des_des3area", "sbox3.v").toString());
  }

  @Test
  public void runTest_sbox4_3area() {
    runTest("sbox4", Paths.get("oc_des_des3area", "sbox4.v").toString());
  }

  @Test
  public void runTest_sbox5_3area() {
    runTest("sbox5", Paths.get("oc_des_des3area", "sbox5.v").toString());
  }

  @Test
  public void runTest_sbox6_3area() {
    runTest("sbox6", Paths.get("oc_des_des3area", "sbox6.v").toString());
  }

  @Test
  public void runTest_sbox7_3area() {
    runTest("sbox7", Paths.get("oc_des_des3area", "sbox7.v").toString());
  }

  @Test
  public void runTest_sbox8_3area() {
    runTest("sbox8", Paths.get("oc_des_des3area", "sbox8.v").toString());
  }

  /*
   * oc_des_des3perf
   */

  @Test
  public void runTest_crp_3perf() {
    runTest("crp", Paths.get("oc_des_des3perf", "crp.v").toString(),
        Paths.get("oc_des_des3perf", "sbox1.v").toString(),
        Paths.get("oc_des_des3perf", "sbox2.v").toString(),
        Paths.get("oc_des_des3perf", "sbox3.v").toString(),
        Paths.get("oc_des_des3perf", "sbox4.v").toString(),
        Paths.get("oc_des_des3perf", "sbox5.v").toString(),
        Paths.get("oc_des_des3perf", "sbox6.v").toString(),
        Paths.get("oc_des_des3perf", "sbox7.v").toString(),
        Paths.get("oc_des_des3perf", "sbox8.v").toString());
  }

  @Test
  public void runTest_des_3perf() {
    runTest("des",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_des_des3perf").toString(),
        Paths.get("oc_des_des3perf", "des.v").toString(),
        Paths.get("oc_des_des3perf", "key_sel.v").toString(),
        Paths.get("oc_des_des3perf", "crp.v").toString(),
        Paths.get("oc_des_des3perf", "sbox1.v").toString(),
        Paths.get("oc_des_des3perf", "sbox2.v").toString(),
        Paths.get("oc_des_des3perf", "sbox3.v").toString(),
        Paths.get("oc_des_des3perf", "sbox4.v").toString(),
        Paths.get("oc_des_des3perf", "sbox5.v").toString(),
        Paths.get("oc_des_des3perf", "sbox6.v").toString(),
        Paths.get("oc_des_des3perf", "sbox7.v").toString(),
        Paths.get("oc_des_des3perf", "sbox8.v").toString());
  }

  @Test
  public void runTest_des3_3perf() {
    runTest("des3",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_des_des3perf").toString(),
        Paths.get("oc_des_des3perf", "des3.v").toString(),
        Paths.get("oc_des_des3perf", "des.v").toString(),
        Paths.get("oc_des_des3perf", "oc_des_des3perf.v").toString(),
        Paths.get("oc_des_des3perf", "crp.v").toString(),
        Paths.get("oc_des_des3perf", "key_sel.v").toString(),
        Paths.get("oc_des_des3perf", "sbox1.v").toString(),
        Paths.get("oc_des_des3perf", "sbox2.v").toString(),
        Paths.get("oc_des_des3perf", "sbox3.v").toString(),
        Paths.get("oc_des_des3perf", "sbox4.v").toString(),
        Paths.get("oc_des_des3perf", "sbox5.v").toString(),
        Paths.get("oc_des_des3perf", "sbox6.v").toString(),
        Paths.get("oc_des_des3perf", "sbox7.v").toString(),
        Paths.get("oc_des_des3perf", "sbox8.v").toString());
  }

  @Test
  public void runTest_key_sel_3perf() {
    runTest("key_sel", Paths.get("oc_des_des3perf", "key_sel.v").toString());
  }

  @Test
  public void runTest_oc_des_des3perf() {
    runTest("oc_des_des3perf",
        Paths.get("oc_des_des3perf", "oc_des_des3perf.v").toString(),
        Paths.get("oc_des_des3perf", "crp.v").toString(),
        Paths.get("oc_des_des3perf", "key_sel.v").toString(),
        Paths.get("oc_des_des3perf", "des.v").toString(),
        Paths.get("oc_des_des3perf", "des3.v").toString(),
        Paths.get("oc_des_des3perf", "sbox1.v").toString(),
        Paths.get("oc_des_des3perf", "sbox2.v").toString(),
        Paths.get("oc_des_des3perf", "sbox3.v").toString(),
        Paths.get("oc_des_des3perf", "sbox4.v").toString(),
        Paths.get("oc_des_des3perf", "sbox5.v").toString(),
        Paths.get("oc_des_des3perf", "sbox6.v").toString(),
        Paths.get("oc_des_des3perf", "sbox7.v").toString(),
        Paths.get("oc_des_des3perf", "sbox8.v").toString());
  }

  @Test
  public void runTest_sbox1_3perf() {
    runTest("sbox1", Paths.get("oc_des_des3perf", "sbox1.v").toString());
  }

  @Test
  public void runTest_sbox2_3perf() {
    runTest("sbox2", Paths.get("oc_des_des3perf", "sbox2.v").toString());
  }

  @Test
  public void runTest_sbox3_3perf() {
    runTest("sbox3", Paths.get("oc_des_des3perf", "sbox3.v").toString());
  }

  @Test
  public void runTest_sbox4_3perf() {
    runTest("sbox4", Paths.get("oc_des_des3perf", "sbox4.v").toString());
  }

  @Test
  public void runTest_sbox5_3perf() {
    runTest("sbox5", Paths.get("oc_des_des3perf", "sbox5.v").toString());
  }

  @Test
  public void runTest_sbox6_3perf() {
    runTest("sbox6", Paths.get("oc_des_des3perf", "sbox6.v").toString());
  }

  @Test
  public void runTest_sbox7_3perf() {
    runTest("sbox7", Paths.get("oc_des_des3perf", "sbox7.v").toString());
  }

  @Test
  public void runTest_sbox8_3perf() {
    runTest("sbox8", Paths.get("oc_des_des3perf", "sbox8.v").toString());
  }

  /*
   * oc_des_perf_opt
   */

  @Test
  public void runTest_crp_opt() {
    runTest("crp", Paths.get("oc_des_perf_opt", "crp.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox1.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox2.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox3.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox4.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox5.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox6.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox7.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_des_test_opt() {
    runTest("test", Paths.get("oc_des_perf_opt", "des_test.v").toString(),
        Paths.get("oc_des_des3perf", "des.v").toString(),
        Paths.get("oc_des_perf_opt", "key_sel.v").toString(),
        Paths.get("oc_des_perf_opt", "crp.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox1.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox2.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox3.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox4.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox5.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox6.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox7.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_key_sel_opt() {
    runTest("key_sel", Paths.get("oc_des_perf_opt", "key_sel.v").toString());
  }

  @Test
  public void runTest_oc_des_perf_opt() {
    runTest("oc_des_perf_opt",
        Paths.get("oc_des_perf_opt", "oc_des_perf_opt.v").toString(),
        Paths.get("oc_des_perf_opt", "crp.v").toString(),
        Paths.get("oc_des_perf_opt", "key_sel.v").toString(),
        Paths.get("oc_des_perf_opt", "des_test.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox1.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox2.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox3.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox4.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox5.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox6.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox7.v").toString(),
        Paths.get("oc_des_perf_opt", "sbox8.v").toString());
  }

  @Test
  public void runTest_sbox1_opt() {
    runTest("sbox1", Paths.get("oc_des_perf_opt", "sbox1.v").toString());
  }

  @Test
  public void runTest_sbox2_opt() {
    runTest("sbox2", Paths.get("oc_des_perf_opt", "sbox2.v").toString());
  }

  @Test
  public void runTest_sbox3_opt() {
    runTest("sbox3", Paths.get("oc_des_perf_opt", "sbox3.v").toString());
  }

  @Test
  public void runTest_sbox4_opt() {
    runTest("sbox4", Paths.get("oc_des_perf_opt", "sbox4.v").toString());
  }

  @Test
  public void runTest_sbox5_opt() {
    runTest("sbox5", Paths.get("oc_des_perf_opt", "sbox5.v").toString());
  }

  @Test
  public void runTest_sbox6_opt() {
    runTest("sbox6", Paths.get("oc_des_perf_opt", "sbox6.v").toString());
  }

  @Test
  public void runTest_sbox7_opt() {
    runTest("sbox7", Paths.get("oc_des_perf_opt", "sbox7.v").toString());
  }

  @Test
  public void runTest_sbox8_opt() {
    runTest("sbox8", Paths.get("oc_des_perf_opt", "sbox8.v").toString());
  }

  /*
   * oc_ethernet
   */

  // TODO: ERROR: [Internal] Descriptor for 'dpram_16x32.ram1.sub_wire0' has not been found:
 /* @Test
  public void runTest_dpram_16x32() {
    runTest("dpram_16x32", Paths.get("oc_ethernet", "dpram_16x32.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'dpram_16x32_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram_16x32_bb() {
    runTest("dpram_16x32_bb",
        Paths.get("oc_ethernet", "dpram_16x32_bb.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString());
  }*/

  @Test
  public void runTest_eth_clockgen() {
    runTest("eth_clockgen",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_clockgen.v").toString());
  }

  @Test
  public void runTest_eth_crc() {
    runTest("eth_crc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_crc.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'eth_fifo.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_eth_fifo() {
    runTest("eth_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_fifo.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32_bb.v").toString());
  }*/

  @Test
  public void runTest_eth_maccontrol() {
    runTest("eth_maccontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_maccontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_receivecontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_transmitcontrol.v").toString());
  }

  @Test
  public void runTest_eth_macstatus() {
    runTest("eth_macstatus",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_macstatus.v").toString());
  }

  @Test
  public void runTest_eth_miim() {
    runTest("eth_miim",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_miim.v").toString(),
        Paths.get("oc_ethernet", "eth_clockgen.v").toString(),
        Paths.get("oc_ethernet", "eth_shiftreg.v").toString(),
        Paths.get("oc_ethernet", "eth_outputcontrol.v").toString());
  }

  @Test
  public void runTest_eth_outputcontrol() {
    runTest("eth_outputcontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_outputcontrol.v").toString());
  }

  @Test
  public void runTest_eth_random() {
    runTest("eth_random",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_random.v").toString());
  }

  @Test
  public void runTest_eth_receivecontrol() {
    runTest("eth_receivecontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_receivecontrol.v").toString());
  }

  @Test
  public void runTest_eth_register() {
    runTest("eth_register",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_register.v").toString());
  }

  @Test
  public void runTest_eth_registers() {
    runTest("eth_registers",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_registers.v").toString(),
        Paths.get("oc_ethernet", "eth_register.v").toString());
  }

  @Test
  public void runTest_eth_rxaddrcheck() {
    runTest("eth_rxaddrcheck",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_rxaddrcheck.v").toString());
  }

  @Test
  public void runTest_eth_rxcounters() {
    runTest("eth_rxcounters",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_rxcounters.v").toString());
  }

  @Test
  public void runTest_eth_rxethmac() {
    runTest("eth_rxethmac",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_rxethmac.v").toString(),
        Paths.get("oc_ethernet", "eth_rxstatem.v").toString(),
        Paths.get("oc_ethernet", "eth_rxcounters.v").toString(),
        Paths.get("oc_ethernet", "eth_rxaddrcheck.v").toString(),
        Paths.get("oc_ethernet", "eth_crc.v").toString());
  }

  @Test
  public void runTest_eth_rxstatem() {
    runTest("eth_rxstatem",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_rxstatem.v").toString());
  }

  @Test
  public void runTest_eth_shiftreg() {
    runTest("eth_shiftreg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_shiftreg.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'eth_spram_256x32.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_eth_spram_256x32() {
    runTest("eth_spram_256x32",
        Paths.get("oc_ethernet", "eth_spram_256x32.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'eth_spram_256x32.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_eth_spram_256x32_bb() {
    runTest("eth_spram_256x32_bb",
        Paths.get("oc_ethernet", "eth_spram_256x32_bb.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32.v").toString());
  }*/

  @Test
  public void runTest_eth_transmitcontrol() {
    runTest("eth_transmitcontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_transmitcontrol.v").toString());
  }

  @Test
  public void runTest_eth_txcounters() {
    runTest("eth_txcounters",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_txcounters.v").toString());
  }

  @Test
  public void runTest_eth_txethmac() {
    runTest("eth_txethmac",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_txethmac.v").toString(),
        Paths.get("oc_ethernet", "eth_txcounters.v").toString(),
        Paths.get("oc_ethernet", "eth_txstatem.v").toString(),
        Paths.get("oc_ethernet", "eth_crc.v").toString(),
        Paths.get("oc_ethernet", "eth_random.v").toString());
  }

  @Test
  public void runTest_eth_txstatem() {
    runTest("eth_txstatem",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_txstatem.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'eth_wishbone.bd_ram.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_eth_wishbone() {
    runTest("eth_wishbone",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "eth_wishbone.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32_bb.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32.v").toString(),
        Paths.get("oc_ethernet", "eth_fifo.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'oc_ethernet.wishbone.bd_ram.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_oc_ethernet() {
    runTest("oc_ethernet",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_ethernet").toString(),
        Paths.get("oc_ethernet", "oc_ethernet.v").toString(),
        Paths.get("oc_ethernet", "altsyncram.v").toString(),
        Paths.get("oc_ethernet", "eth_miim.v").toString(),
        Paths.get("oc_ethernet", "eth_registers.v").toString(),
        Paths.get("oc_ethernet", "eth_maccontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_txethmac.v").toString(),
        Paths.get("oc_ethernet", "eth_rxethmac.v").toString(),
        Paths.get("oc_ethernet", "eth_wishbone.v").toString(),
        Paths.get("oc_ethernet", "eth_macstatus.v").toString(),
        Paths.get("oc_ethernet", "eth_clockgen.v").toString(),
        Paths.get("oc_ethernet", "eth_shiftreg.v").toString(),
        Paths.get("oc_ethernet", "eth_outputcontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_register.v").toString(),
        Paths.get("oc_ethernet", "eth_receivecontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_transmitcontrol.v").toString(),
        Paths.get("oc_ethernet", "eth_txcounters.v").toString(),
        Paths.get("oc_ethernet", "eth_txstatem.v").toString(),
        Paths.get("oc_ethernet", "eth_crc.v").toString(),
        Paths.get("oc_ethernet", "eth_random.v").toString(),
        Paths.get("oc_ethernet", "eth_rxstatem.v").toString(),
        Paths.get("oc_ethernet", "eth_rxcounters.v").toString(),
        Paths.get("oc_ethernet", "eth_rxaddrcheck.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32_bb.v").toString(),
        Paths.get("oc_ethernet", "eth_spram_256x32.v").toString(),
        Paths.get("oc_ethernet", "eth_fifo.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32.v").toString(),
        Paths.get("oc_ethernet", "dpram_16x32_bb.v").toString());
  }*/

  /*
   * oc_fcmp
   */

  @Test
  public void runTest_oc_fcmp() {
    runTest("oc_fcmp", Paths.get("oc_fcmp", "oc_fcmp.v").toString());
  }

  /*
   * oc_fpu
   */

  @Test
  public void runTest_except() {
    runTest("except", Paths.get("oc_fpu", "except.v").toString());
  }

  @Test
  public void runTest_oc_fpu() {
    runTest("oc_fpu", Paths.get("oc_fpu", "oc_fpu.v").toString(),
        Paths.get("oc_fpu", "except.v").toString(),
        Paths.get("oc_fpu", "pre_norm.v").toString(),
        Paths.get("oc_fpu", "pre_norm_fmul.v").toString(),
        Paths.get("oc_fpu", "primitives.v").toString(),
        Paths.get("oc_fpu", "post_norm.v").toString());
  }

  @Test
  public void runTest_post_norm() {
    runTest("post_norm", Paths.get("oc_fpu", "post_norm.v").toString());
  }

  @Test
  public void runTest_pre_norm() {
    runTest("pre_norm", Paths.get("oc_fpu", "pre_norm.v").toString());
  }

  @Test
  public void runTest_pre_norm_fmul() {
    runTest("pre_norm_fmul", Paths.get("oc_fpu", "pre_norm_fmul.v").toString());
  }

  @Test
  public void runTest_primitives() {
    runTest("add_sub27", Paths.get("oc_fpu", "primitives.v").toString());
  }

  /*
   * oc_gpio
   */

  @Test
  public void runTest_defines_gpio() {
    runTest(null, Paths.get("oc_gpio", "defines.v").toString());
  }

  @Test
  public void runTest_oc_gpio() {
    runTest("oc_gpio",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_gpio").toString(),
        Paths.get("oc_gpio", "oc_gpio.v").toString());
  }

  /*
   * oc_i2c
   */

  @Test
  public void runTest_i2c_master_bit_ctrl() {
    runTest("i2c_master_bit_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_i2c").toString() ,
        Paths.get("oc_i2c", "i2c_master_bit_ctrl.v").toString());
  }

  @Test
  public void runTest_i2c_master_byt_ctrl() {
    runTest("i2c_master_byte_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_i2c").toString(),
        Paths.get("oc_i2c", "i2c_master_byte_ctrl.v").toString(),
        Paths.get("oc_i2c", "i2c_master_bit_ctrl.v").toString());
  }

  @Test
  public void runTest_oc_i2c() {
    runTest("oc_i2c",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_i2c").toString(),
        Paths.get("oc_i2c", "oc_i2c.v").toString(),
        Paths.get("oc_i2c", "i2c_master_byte_ctrl.v").toString(),
        Paths.get("oc_i2c", "i2c_master_bit_ctrl.v").toString());
  }

  /*
   * oc_mem_ctrl
   */

  @Test
  public void runTest_mc_adr_sel() {
    runTest("mc_adr_sel",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_adr_sel.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_incn_r.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'mc_cs_rf.this_cs' has not been found
  /*@Test
  public void runTest_mc_cs_rf() {
    runTest("mc_cs_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_cs_rf.v").toString());
  }*/

  @Test
  public void runTest_mc_dp() {
    runTest("mc_dp",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_dp.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_rd_fifo.v").toString());
  }

  @Test
  public void runTest_mc_incn_r() {
    runTest("mc_incn_r",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_incn_r.v").toString());
  }

  @Test
  public void runTest_mc_mem_if() {
    runTest("mc_mem_if",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_mem_if.v").toString());
  }

  @Test
  public void runTest_mc_obct() {
    runTest("mc_obct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_obct.v").toString());
  }

  @Test
  public void runTest_mc_obct_top() {
    runTest("mc_obct_top",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_obct_top.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_obct.v").toString());
  }

  @Test
  public void runTest_mc_rd_fifo() {
    runTest("mc_rd_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_rd_fifo.v").toString());
  }

  @Test
  public void runTest_mc_refresh() {
    runTest("mc_refresh",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_refresh.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'mc_rf.u0.this_cs' has not been found
  /*@Test
  public void runTest_mc_rf() {
    runTest("mc_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_rf.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_cs_rf.v").toString());
  }*/

  @Test
  public void runTest_mc_timing() {
    runTest("mc_timing",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_timing.v").toString());
  }

  @Test
  public void runTest_mc_wb_if() {
    runTest("mc_wb_if",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "mc_wb_if.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'oc_mem_ctrl.u0.u0.this_cs' has not been found
  /*@Test
  public void runTest_oc_mem_ctrl() {
    runTest("oc_mem_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_mem_ctrl").toString(),
        Paths.get("oc_mem_ctrl", "oc_mem_ctrl.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_rf.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_adr_sel.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_obct_top.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_dp.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_refresh.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_timing.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_wb_if.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_mem_if.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_cs_rf.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_incn_r.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_obct.v").toString(),
        Paths.get("oc_mem_ctrl", "mc_rd_fifo.v").toString());
  }*/

  /*
   * oc_minirisc
   */

  @Test
  public void runTest_alu_risc() {
    runTest("alu", Paths.get("oc_minirisc", "alu.v").toString(),
        Paths.get("oc_minirisc", "primitives.v").toString());
  }

  @Test
  public void runTest_oc_minirisc() {
    runTest("oc_minirisc", Paths.get("oc_minirisc", "oc_minirisc.v").toString(),
        Paths.get("oc_minirisc", "register_file.v").toString(),
        Paths.get("oc_minirisc", "primitives.v").toString(),
        Paths.get("oc_minirisc", "alu.v").toString(),
        Paths.get("oc_minirisc", "presclr_wdt.v").toString());
  }

  @Test
  public void runTest_presclr_wdt() {
    runTest("presclr_wdt", Paths.get("oc_minirisc", "presclr_wdt.v").toString(),
        Paths.get("oc_minirisc", "primitives.v").toString());
  }

  @Test
  public void runTest_primitives_risc() {
    runTest("mux4_8", Paths.get("oc_minirisc", "primitives.v").toString());
  }

  @Test
  public void runTest_register_file() {
    runTest("register_file",
        Paths.get("oc_minirisc", "register_file.v").toString(),
        Paths.get("oc_minirisc", "primitives.v").toString());
  }

  @Test
  public void runTest_risc_core_top() {
    runTest("oc_minirisc_top",
        Paths.get("oc_minirisc", "risc_core_top.v").toString(),
        Paths.get("oc_minirisc", "alu.v").toString(),
        Paths.get("oc_minirisc", "presclr_wdt.v").toString(),
        Paths.get("oc_minirisc", "primitives.v").toString(),
        Paths.get("oc_minirisc", "ramb4_s4_s4.v").toString(),
        Paths.get("oc_minirisc", "register_file.v").toString(),
        Paths.get("oc_minirisc", "oc_minirisc.v").toString());
  }

  /*
   * oc_oc8051
   */

  //TODO: ERROR: [Internal] Descriptor for 'dpram_32x32.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram_32x32() {
    runTest("dpram_32x32",
        Paths.get("oc_oc8051", "dpram_32x32.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'dpram_32x32_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram_32x32_bb() {
    runTest("dpram_32x32_bb",
        Paths.get("oc_oc8051", "dpram_32x32_bb.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32.v").toString());

  }*/

  // TODO: ERROR: [Internal] Descriptor for 'dpram_256x8.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram_256x8() {
    runTest("dpram_256x8",
        Paths.get("oc_oc8051", "dpram_256x8.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'dpram_256x8_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram_256x8_bb() {
    runTest("dpram_256x8_bb",
        Paths.get("oc_oc8051", "dpram_256x8_bb.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8.v").toString());
  }*/

  @Test
  public void runTest_oc8051_acc() {
    runTest("oc8051_acc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_acc.v").toString());
  }

  @Test
  public void runTest_oc8051_alu() {
    runTest("oc8051_alu",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_alu.v").toString(),
        Paths.get("oc_oc8051", "oc8051_multiply.v").toString(),
        Paths.get("oc_oc8051", "oc8051_divide.v").toString());
  }

  @Test
  public void runTest_oc8051_alu_src_sel() {
    runTest("oc8051_alu_src_sel",
        "--include-path",
         Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_alu_src_sel.v").toString());
  }

  @Test
  public void runTest_oc8051_b_register() {
    runTest("oc8051_b_register",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_b_register.v").toString());
  }

  @Test
  public void runTest_oc8051_comp() {
    runTest("oc8051_comp",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_comp.v").toString());
  }

  @Test
  public void runTest_oc8051_cy_select() {
    runTest("oc8051_cy_select",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_cy_select.v").toString());
  }

  @Test
  public void runTest_oc8051_decoder() {
    runTest("oc8051_decoder",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_decoder.v").toString());
  }

  @Test
  public void runTest_oc8051_divide() {
    runTest("oc8051_divide",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_divide.v").toString());
  }

  @Test
  public void runTest_oc8051_dptr() {
    runTest("oc8051_dptr",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_dptr.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'oc8051_icache.oc8051_cache_ram.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_oc8051_icache() {
    runTest("oc8051_icache",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_icache.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32_bb.v").toString());
  }*/

  @Test
  public void runTest_oc8051_indi_addr() {
    runTest("oc8051_indi_addr",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_indi_addr.v").toString());
  }

  @Test
  public void runTest_oc8051_int() {
    runTest("oc8051_int",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_int.v").toString());
  }

  @Test
  public void runTest_oc8051_memory_interface() {
    runTest("oc8051_memory_interface",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_memory_interface.v").toString());
  }

  @Test
  public void runTest_oc8051_multiply() {
    runTest("oc8051_multiply",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_multiply.v").toString());
  }

  @Test
  public void runTest_oc8051_ports() {
    runTest("oc8051_ports",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_ports.v").toString());
  }

  @Test
  public void runTest_oc8051_psw() {
    runTest("oc8051_psw",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_psw.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'oc8051_ram_top.oc8051_idata.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_oc8051_ram_top() {
    runTest("oc8051_ram_top",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_ram_top.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8_bb.v").toString());
  }*/

  @Test
  public void runTest_oc8051_sfr() {
    runTest("oc8051_sfr",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_sfr.v").toString(),
        Paths.get("oc_oc8051", "oc8051_acc.v").toString(),
        Paths.get("oc_oc8051", "oc8051_b_register.v").toString(),
        Paths.get("oc_oc8051", "oc8051_sp.v").toString(),
        Paths.get("oc_oc8051", "oc8051_dptr.v").toString(),
        Paths.get("oc_oc8051", "oc8051_psw.v").toString(),
        Paths.get("oc_oc8051", "oc8051_ports.v").toString(),
        Paths.get("oc_oc8051", "oc8051_uart.v").toString(),
        Paths.get("oc_oc8051", "oc8051_int.v").toString(),
        Paths.get("oc_oc8051", "oc8051_tc.v").toString(),
        Paths.get("oc_oc8051", "oc8051_tc2.v").toString());
  }

  @Test
  public void runTest_oc8051_sp() {
    runTest("oc8051_sp",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_sp.v").toString());
  }

  @Test
  public void runTest_oc8051_tc() {
    runTest("oc8051_tc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_tc.v").toString());
  }

  @Test
  public void runTest_oc8051_tc2() {
    runTest("oc8051_tc2",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_tc2.v").toString());
  }

  @Test
  public void runTest_oc8051_uart() {
    runTest("oc8051_uart",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc8051_uart.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'oc_oc8051.oc8051_ram_top1.oc8051_idata.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_oc_oc8051() {
    runTest("oc_oc8051",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_oc8051").toString(),
        Paths.get("oc_oc8051", "oc_oc8051.v").toString(),
        Paths.get("oc_oc8051", "altsyncram.v").toString(),
        Paths.get("oc_oc8051", "oc8051_decoder.v").toString(),
        Paths.get("oc_oc8051", "oc8051_alu.v").toString(),
        Paths.get("oc_oc8051", "oc8051_ram_top.v").toString(),
        Paths.get("oc_oc8051", "oc8051_alu_src_sel.v").toString(),
        Paths.get("oc_oc8051", "oc8051_comp.v").toString(),
        Paths.get("oc_oc8051", "oc8051_cy_select.v").toString(),
        Paths.get("oc_oc8051", "oc8051_indi_addr.v").toString(),
        Paths.get("oc_oc8051", "oc8051_memory_interface.v").toString(),
        Paths.get("oc_oc8051", "oc8051_sfr.v").toString(),
        Paths.get("oc_oc8051", "oc8051_icache.v").toString(),
        Paths.get("oc_oc8051", "oc8051_multiply.v").toString(),
        Paths.get("oc_oc8051", "oc8051_divide.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8.v").toString(),
        Paths.get("oc_oc8051", "oc8051_acc.v").toString(),
        Paths.get("oc_oc8051", "oc8051_b_register.v").toString(),
        Paths.get("oc_oc8051", "oc8051_sp.v").toString(),
        Paths.get("oc_oc8051", "oc8051_dptr.v").toString(),
        Paths.get("oc_oc8051", "oc8051_psw.v").toString(),
        Paths.get("oc_oc8051", "oc8051_ports.v").toString(),
        Paths.get("oc_oc8051", "oc8051_uart.v").toString(),
        Paths.get("oc_oc8051", "oc8051_int.v").toString(),
        Paths.get("oc_oc8051", "oc8051_tc.v").toString(),
        Paths.get("oc_oc8051", "oc8051_tc2.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32.v").toString(),
        Paths.get("oc_oc8051", "dpram_256x8_bb.v").toString(),
        Paths.get("oc_oc8051", "dpram_32x32_bb.v").toString());
  }*/

  /*
   * oc_pci
   */

  //TODO: ERROR: [Internal] Descriptor for 'dpram40x256.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram40x256() {
    runTest("dpram40x256",
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'dpram40x256_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram40x256_bb() {
    runTest("dpram40x256_bb",
        Paths.get("oc_pci", "dpram40x256_bb.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString());
  }*/

  //TODO: ERROR: [Internal] Descriptor for
  // 'oc_pci.wishbone_slave_unit.fifos.wbw_fifo_storage.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_oc_pci() {
    runTest("oc_pci",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "oc_pci.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString(),
        Paths.get("oc_pci", "pci_rst_int.v").toString(),
        Paths.get("oc_pci", "pci_wbs_wbb3_2_wbb2.v").toString(),
        Paths.get("oc_pci", "pci_wb_slave_unit.v").toString(),
        Paths.get("oc_pci", "pci_target_unit.v").toString(),
        Paths.get("oc_pci", "pci_conf_space.v").toString(),
        Paths.get("oc_pci", "pci_io_mux.v").toString(),
        Paths.get("oc_pci", "pci_cur_out_reg.v").toString(),
        Paths.get("oc_pci", "pci_parity_check.v").toString(),
        Paths.get("oc_pci", "pci_pci_decoder.v").toString(),
        Paths.get("oc_pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_in_reg.v").toString(),
        Paths.get("oc_pci", "pci_out_reg.v").toString(),
        Paths.get("oc_pci", "pci_wb_slave.v").toString(),
        Paths.get("oc_pci", "pci_wbw_wbr_fifos.v").toString(),
        Paths.get("oc_pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("oc_pci", "pci_delayed_sync.v").toString(),
        Paths.get("oc_pci", "pci_delayed_write_reg.v").toString(),
        Paths.get("oc_pci", "pci_master32_sm_if.v").toString(),
        Paths.get("oc_pci", "pci_master32_sm.v").toString(),
        Paths.get("oc_pci", "pci_wb_master.v").toString(),
        Paths.get("oc_pci", "pci_pciw_pcir_fifos.v").toString(),
        Paths.get("oc_pci", "pci_target32_clk_en.v").toString(),
        Paths.get("oc_pci", "pci_target32_interface.v").toString(),
        Paths.get("oc_pci", "pci_target32_sm.v").toString(),
        Paths.get("oc_pci", "pci_target32_devs_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_trdy_crit.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "pci_sync_module.v").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_par_crit.v").toString(),
        Paths.get("oc_pci", "pci_perr_crit.v").toString(),
        Paths.get("oc_pci", "pci_perr_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_serr_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_serr_crit.v").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString(),
        Paths.get("oc_pci", "pci_wb_tpram.v").toString(),
        Paths.get("oc_pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_wb_decoder.v").toString(),
        Paths.get("oc_pci", "pci_frame_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ch_state_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_pci_tpram.v").toString(),
        Paths.get("oc_pci", "pci_pciw_fifo_control.v").toString());
  }*/

  @Test
  public void runTest_pci_async_reset_flop() {
    runTest("pci_async_reset_flop",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString());
  }

  @Test
  public void runTest_pci_cbe_en_crit() {
    runTest("pci_cbe_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_cbe_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_conf_space() {
    runTest("pci_conf_space",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_conf_space.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "pci_sync_module.v").toString());
  }

  @Test
  public void runTest_pci_constants() {
    runTest(null,
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_constants.v").toString());
  }

  @Test
  public void runTest_pci_cur_out_reg() {
    runTest("pci_cur_out_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_cur_out_reg.v").toString());
  }

  @Test
  public void runTest_pci_delayed_sync() {
    runTest("pci_delayed_sync",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_delayed_sync.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_pci_delayed_write_reg() {
    runTest("pci_delayed_write_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_delayed_write_reg.v").toString());
  }

  @Test
  public void runTest_pci_frame_crit() {
    runTest("pci_frame_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_frame_crit.v").toString());
  }

  @Test
  public void runTest_pci_frame_en_crit() {
    runTest("pci_frame_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_frame_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_frame_load_crit() {
    runTest("pci_frame_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_frame_load_crit.v").toString());
  }

  @Test
  public void runTest_pci_in_reg() {
    runTest("pci_in_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_in_reg.v").toString());
  }

  @Test
  public void runTest_pci_io_mux() {
    runTest("pci_io_mux",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_io_mux.v").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_pci_io_mux_ad_en_crit() {
    runTest("pci_io_mux_ad_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_io_mux_ad_load_crit() {
    runTest("pci_io_mux_ad_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_io_mux_ad_load_crit.v").toString());
  }

  @Test
  public void runTest_pci_irdy_out_crit() {
    runTest("pci_irdy_out_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_irdy_out_crit.v").toString());
  }

  @Test
  public void runTest_pci_mas_ad_en_crit() {
    runTest("pci_mas_ad_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_mas_ad_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_mas_ad_load_crit() {
    runTest("pci_mas_ad_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_mas_ad_load_crit.v").toString());
  }

  @Test
  public void runTest_pci_mas_ch_state_crit() {
    runTest("pci_mas_ch_state_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_mas_ch_state_crit.v").toString());
  }

  @Test
  public void runTest_pci_master32_sm() {
    runTest("pci_master32_sm",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_master32_sm.v").toString(),
        Paths.get("oc_pci", "pci_frame_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ch_state_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_master32_sm_if() {
    runTest("pci_master32_sm_if",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_master32_sm_if.v").toString());
  }

  @Test
  public void runTest_pci_out_reg() {
    runTest("pci_out_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_pci_par_crit() {
    runTest("pci_par_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_par_crit.v").toString());
  }

  @Test
  public void runTest_pci_parity_check() {
    runTest("pci_parity_check",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_parity_check.v").toString(),
        Paths.get("oc_pci", "pci_par_crit.v").toString(),
        Paths.get("oc_pci", "pci_perr_crit.v").toString(),
        Paths.get("oc_pci", "pci_perr_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_serr_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_serr_crit.v").toString());
  }

  @Test
  public void runTest_pci_pci_decoder() {
    runTest("pci_pci_decoder",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_pci_decoder.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'pci_pci_tpram.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_pci_pci_tpram() {
    runTest("pci_pci_tpram",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_pci_tpram.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString());
  }*/

  @Test
  public void runTest_pci_pcir_fifo_control() {
    runTest("pci_pcir_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_pci_pciw_fifo_control() {
    runTest("pci_pciw_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_pciw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for
  // 'pci_pciw_pcir_fifos.pciw_fifo_storage.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_pci_pciw_pcir_fifos() {
    runTest("pci_pciw_pcir_fifos", "--include-path", Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_pciw_pcir_fifos.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "pci_pci_tpram.v").toString(),
        Paths.get("oc_pci", "pci_pciw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString());
  }*/

  @Test
  public void runTest_pci_perr_crit() {
    runTest("pci_perr_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_perr_crit.v").toString());
  }

  @Test
  public void runTest_pci_perr_en_crit() {
    runTest("pci_perr_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_perr_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_rst_int() {
    runTest("pci_rst_int",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_rst_int.v").toString(),
        Paths.get("oc_pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_pci_serr_crit() {
    runTest("pci_serr_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_serr_crit.v").toString());
  }

  @Test
  public void runTest_pci_serr_en_crit() {
    runTest("pci_serr_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_serr_en_crit.v").toString());
  }

  @Test
  public void runTest_pci_sync_module() {
    runTest("pci_sync_module",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_sync_module.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_pci_synchronizer_flop() {
    runTest("pci_synchronizer_flop",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_pci_target32_clk_en() {
    runTest("pci_target32_clk_en",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_clk_en.v").toString());
  }

  @Test
  public void runTest_pci_target32_devs_crit() {
    runTest("pci_target32_devs_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_devs_crit.v").toString());
  }

  // TODO: ERROR: [Internal] 11 must be within range [0, 1)
  /*@Test
  public void runTest_pci_target32_interface() {
    runTest("pci_target32_interface",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_interface.v").toString(),
        Paths.get("oc_pci", "pci_pci_decoder.v").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString());
  }*/

  @Test
  public void runTest_pci_target32_sm() {
    runTest("pci_target32_sm",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_sm.v").toString(),
        Paths.get("oc_pci", "pci_target32_clk_en.v").toString(),
        Paths.get("oc_pci", "pci_target32_trdy_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_devs_crit.v").toString());
  }

  @Test
  public void runTest_pci_target32_stop_crit() {
    runTest("pci_target32_stop_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_stop_crit.v").toString());
  }

  @Test
  public void runTest_pci_target32_trdy_crit() {
    runTest("pci_target32_trdy_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target32_trdy_crit.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for
  // 'pci_target_unit.fifos.pciw_fifo_storage.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_pci_target_unit() {
    runTest("pci_target_unit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_target_unit.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString(),
        Paths.get("oc_pci", "pci_wb_master.v").toString(),
        Paths.get("oc_pci", "pci_pciw_pcir_fifos.v").toString(),
        Paths.get("oc_pci", "pci_delayed_sync.v").toString(),
        Paths.get("oc_pci", "pci_target32_interface.v").toString(),
        Paths.get("oc_pci", "pci_target32_sm.v").toString(),
        Paths.get("oc_pci", "pci_pci_tpram.v").toString(),
        Paths.get("oc_pci", "pci_pciw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "pci_pci_decoder.v").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString(),
        Paths.get("oc_pci", "pci_target32_clk_en.v").toString(),
        Paths.get("oc_pci", "pci_target32_trdy_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("oc_pci", "pci_target32_devs_crit.v").toString());
  }*/

  @Test
  public void runTest_pci_wb_addr_mux() {
    runTest("pci_wb_addr_mux",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("oc_pci", "pci_wb_decoder.v").toString());
  }

  @Test
  public void runTest_pci_wb_decoder() {
    runTest("pci_wb_decoder",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_decoder.v").toString());
  }

  @Test
  public void runTest_pci_wb_master() {
    runTest("pci_wb_master",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_master.v").toString());
  }

  @Test
  public void runTest_pci_wb_slave() {
    runTest("pci_wb_slave",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_slave.v").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for
  // 'pci_wb_slave_unit.fifos.wbw_fifo_storage.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_pci_wb_slave_unit() {
    runTest("pci_wb_slave_unit",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_slave_unit.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString(),
        Paths.get("oc_pci", "pci_wb_slave.v").toString(),
        Paths.get("oc_pci", "pci_wbw_wbr_fifos.v").toString(),
        Paths.get("oc_pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("oc_pci", "pci_delayed_sync.v").toString(),
        Paths.get("oc_pci", "pci_delayed_write_reg.v").toString(),
        Paths.get("oc_pci", "pci_master32_sm_if.v").toString(),
        Paths.get("oc_pci", "pci_master32_sm.v").toString(),
        Paths.get("oc_pci", "pci_async_reset_flop.v").toString(),
        Paths.get("oc_pci", "pci_wb_tpram.v").toString(),
        Paths.get("oc_pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "pci_wb_decoder.v").toString(),
        Paths.get("oc_pci", "pci_frame_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ch_state_crit.v").toString(),
        Paths.get("oc_pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("oc_pci", "pci_frame_en_crit.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString());
  }*/

  //TODO: ERROR: [Internal] Descriptor for 'pci_wb_tpram.ram1.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_pci_wb_tpram() {
    runTest("pci_wb_tpram",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wb_tpram.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString());
  }*/

  @Test
  public void runTest_pci_wbr_fifo_control() {
    runTest("pci_wbr_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_pci_wbs_wbb3_2_wbb2() {
    runTest("pci_wbs_wbb3_2_wbb2",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wbs_wbb3_2_wbb2.v").toString());
  }

  @Test
  public void runTest_pci_wbw_fifo_control() {
    runTest("pci_wbw_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'pci_wbw_wbr_fifos.wbw_fifo_storage.ram1.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_pci_wbw_wbr_fifos() {
    runTest("pci_wbw_wbr_fifos",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_pci").toString(),
        Paths.get("oc_pci", "pci_wbw_wbr_fifos.v").toString(),
        Paths.get("oc_pci", "altsyncram.v").toString(),
        Paths.get("oc_pci", "dpram40x256_bb.v").toString(),
        Paths.get("oc_pci", "pci_wb_tpram.v").toString(),
        Paths.get("oc_pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("oc_pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("oc_pci", "dpram40x256.v").toString());
  }*/

  /*
   * oc_rtc
   */

  @Test
  public void runTest_oc_rtc() {
    runTest("oc_rtc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_rtc").toString(),
        Paths.get("oc_rtc", "oc_rtc.v").toString());
  }

  /*
   * oc_sdram
   */

  @Test
  public void runTest_hostcont() {
    runTest("hostcont",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_sdram").toString(),
        Paths.get("oc_sdram", "hostcont.v").toString());
  }

  @Test
  public void runTest_micro() {
    runTest("micro",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_sdram").toString(),
        Paths.get("oc_sdram", "micro.v").toString());
  }

  @Test
  public void runTest_oc_sdram() {
    runTest("oc_sdram",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_sdram").toString(),
        Paths.get("oc_sdram", "oc_sdram.v").toString(),
        Paths.get("oc_sdram", "sdramcnt.v").toString(),
        Paths.get("oc_sdram", "hostcont.v").toString());
  }

  @Test
  public void runTest_sdramcnt() {
    runTest("sdramcnt",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_sdram").toString(),
        Paths.get("oc_sdram", "sdramcnt.v").toString());
  }

  /*
   * oc_vga_lcd
   */

  //TODO: ERROR: [Internal] Descriptor for 'dpram128x24.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram128x24() {
    runTest("dpram128x24",
        Paths.get("oc_vga_lcd", "dpram128x24.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24_bb.v").toString());
  }*/

  // TODO: Descriptor for 'dpram128x24_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_dpram128x24_bb() {
    runTest("dpram128x24_bb",
        Paths.get("oc_vga_lcd", "dpram128x24_bb.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'oc_vga_lcd.clut_mem.clut_mem.ram1.sub_wire0'
  //  has not been found
  /*@Test
  public void runTest_oc_vga_lcd() {
    runTest("oc_vga_lcd",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "oc_vga_lcd.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "vga_wb_slave.v").toString(),
        Paths.get("oc_vga_lcd", "vga_wb_master.v").toString(),
        Paths.get("oc_vga_lcd", "vga_csm_pb.v").toString(),
        Paths.get("oc_vga_lcd", "vga_cur_cregs.v").toString(),
        Paths.get("oc_vga_lcd", "vga_curproc.v").toString(),
        Paths.get("oc_vga_lcd", "vga_pgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo_dc.v").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32_bb.v").toString(),
        Paths.get("oc_vga_lcd", "vga_clkgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_tgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_colproc.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24_bb.v").toString(),
        Paths.get("oc_vga_lcd", "vga_vtim.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'spram512x24.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_spram512x24() {
    runTest("spram512x24",
        Paths.get("oc_vga_lcd", "spram512x24.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'spram512x24_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_spram512x24_bb() {
    runTest("spram512x24_bb",
        Paths.get("oc_vga_lcd", "spram512x24_bb.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'spram512x32.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_spram512x32() {
    runTest("spram512x32",
        Paths.get("oc_vga_lcd", "spram512x32.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'spram512x32_bb.sub_wire0' has not been found
  /*@Test
  public void runTest_spram512x32_bb() {
    runTest("spram512x32_bb",
        Paths.get("oc_vga_lcd", "spram512x32_bb.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32.v").toString());
  }*/

  @Test
  public void runTest_timescale_vga() {
    runTest(null, Paths.get("oc_vga_lcd", "timescale.v").toString());
  }

  @Test
  public void runTest_vga_clkgen() {
    runTest("vga_clkgen",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_clkgen.v").toString());
  }

  @Test
  public void runTest_vga_colproc() {
    runTest("vga_colproc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_colproc.v").toString());
  }

  /*//TODO: ERROR: [Internal] Descriptor for 'vga_csm_pb.clut_mem.ram1.sub_wire0' has not been found
  @Test
  public void runTest_vga_csm_pb() {
    runTest("vga_csm_pb",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_csm_pb.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x24_bb.v").toString());
  }*/

  @Test
  public void runTest_vga_cur_cregs() {
    runTest("vga_cur_cregs",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_cur_cregs.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'vga_curproc.cbuf.ram1.sub_wire0' has not been found
  /*@Test
  public void runTest_vga_curproc() {
    runTest("vga_curproc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_curproc.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32_bb.v").toString());
  }*/

  @Test
  public void runTest_vga_defines() {
    runTest(null,
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_defines.v").toString());
  }

  @Test
  public void runTest_vga_fifo() {
    runTest("vga_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo.v").toString());
  }

  //TODO: ERROR: [Internal] Descriptor for 'vga_fifo_dc.fifo_dc_mem.ram1.sub_wire0'
  // has not been found
  /*@Test
  public void runTest_vga_fifo_dc() {
    runTest("vga_fifo_dc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo_dc.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24.v").toString(),
        Paths.get("oc_vga_lcd", "dpram128x24_bb.v").toString());
  }*/

  // TODO: ERROR: [Internal] Descriptor for 'vga_pgen.hw_cursor1.cbuf.ram1.sub_wire0'
  //  has not been found
  /*@Test
  public void runTest_vga_pgen() {
    runTest("vga_pgen",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_pgen.v").toString(),
        Paths.get("oc_vga_lcd", "altsyncram.v").toString(),
        Paths.get("oc_vga_lcd", "vga_clkgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_tgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_colproc.v").toString(),
        Paths.get("oc_vga_lcd", "vga_curproc.v").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo.v").toString(),
        Paths.get("oc_vga_lcd", "vga_vtim.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32.v").toString(),
        Paths.get("oc_vga_lcd", "spram512x32_bb.v").toString());
  }*/

  @Test
  public void runTest_vga_tgen() {
    runTest("vga_tgen",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_tgen.v").toString(),
        Paths.get("oc_vga_lcd", "vga_vtim.v").toString());
  }

  @Test
  public void runTest_vga_vtim() {
    runTest("vga_vtim",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_vtim.v").toString());
  }

  @Test
  public void runTest_vga_wb_master() {
    runTest("vga_wb_master",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_wb_master.v").toString(),
        Paths.get("oc_vga_lcd", "vga_fifo.v").toString());
  }

  @Test
  public void runTest_vga_wb_slave() {
    runTest("vga_wb_slave",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_vga_lcd").toString(),
        Paths.get("oc_vga_lcd", "vga_wb_slave.v").toString(),
        Paths.get("oc_vga_lcd", "vga_cur_cregs.v").toString());
  }

  /*
   * oc_video_compression_systems_dct
   */

  @Test
  public void runTest_dct() {
    runTest("dct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_dct_cos_table() {
    runTest("dct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctub.v").toString());
  }

  @Test
  public void runTest_dct_mac() {
    runTest("dct_mac",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_dctu() {
    runTest("dctu",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_fdct() {
    runTest("fdct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "fdct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "zigzag.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_dctub() {
    runTest("dctub",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_oc_video_compression_systems_dct() {
    runTest("oc_video_compression_systems_dct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct",
            "oc_video_compression_systems_dct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "fdct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "zigzag.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_dct", "dct_mac.v").toString());
  }

  @Test
  public void runTest_zigzag() {
    runTest("zigzag",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_dct").toString(),
        Paths.get("oc_video_compression_systems_dct", "zigzag.v").toString());
  }

  /*
   * oc_video_compression_systems_huffman_dec
   */

  @Test
  public void runTest_oc_video_compression_systems_huffman_dec() {
    runTest("oc_video_compression_systems_huffman_dec",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_huffman_dec").toString(),
        Paths.get("oc_video_compression_systems_huffman_dec",
            "oc_video_compression_systems_huffman_dec.v").toString());
  }

  /*
   * oc_video_compression_systems_huffman_enc
   */

  @Test
  public void runTest_oc_video_compression_systems_huffman_enc() {
    runTest("oc_video_compression_systems_huffman_enc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_huffman_enc").toString(),
        Paths.get("oc_video_compression_systems_huffman_enc",
            "oc_video_compression_systems_huffman_enc.v").toString());
  }

  /*
   * oc_video_compression_systems_jpeg
   */

  @Test
  public void runTest_dct_jpeg() {
    runTest("dct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString());
  }

  @Test
  public void runTest_dct_mac_jpeg() {
    runTest("dct_mac",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString());
  }

  @Test
  public void runTest_dctu_jpeg() {
    runTest("dctu",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString());
  }

  @Test
  public void runTest_dctub_jpeg() {
    runTest("dctub",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString());
  }

  @Test
  public void runTest_div_su_jpeg() {
    runTest("div_su",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_su.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_uu.v").toString());
  }

  @Test
  public void runTest_div_uu_jpeg() {
    runTest("div_uu",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_uu.v").toString());
  }

  @Test
  public void runTest_fdct_jpeg() {
    runTest("fdct",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "fdct.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "zigzag.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString());
  }

  @Test
  public void runTest_jpeg_qnr() {
    runTest("jpeg_qnr",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_qnr.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_su.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_uu.v").toString());
  }

  @Test
  public void runTest_jpeg_rle() {
    runTest("jpeg_rle",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rle.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rle1.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rzs.v").toString());
  }

  @Test
  public void runTest_jpeg_rle1() {
    runTest("jpeg_rle1",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rle1.v").toString());
  }

  @Test
  public void runTest_jpeg_rzs() {
    runTest("jpeg_rzs",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rzs.v").toString());
  }

  @Test
  public void runTest_oc_video_compression_systems_jpeg() {
    runTest("oc_video_compression_systems_jpeg",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg",
            "oc_video_compression_systems_jpeg.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "fdct.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_qnr.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rle.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dct_mac.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "zigzag.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_su.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rle1.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "jpeg_rzs.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctub.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "div_uu.v").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "dctu.v").toString());
  }

  @Test
  public void runTest_zigzag_jpeg() {
    runTest("zigzag",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_video_compression_systems_jpeg").toString(),
        Paths.get("oc_video_compression_systems_jpeg", "zigzag.v").toString());
  }

  /*
   * oc_wb_dma
   */

  @Test
  public void runTest_oc_wb_dma() {
    runTest("oc_wb_dma",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "oc_wb_dma.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_rf.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_sel.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_de.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_if.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_rf.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_arb.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_inc30r.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_mast.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_slv.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }

  @Test
  public void runTest_wb_dma_ch_arb() {
    runTest("wb_dma_ch_arb",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_arb.v").toString());
  }

  @Test
  public void runTest_wb_dma_ch_pri_enc() {
    runTest("wb_dma_ch_pri_enc",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }

  @Test
  public void runTest_wb_dma_ch_rf() {
    runTest("wb_dma_ch_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_rf.v").toString());
  }

  @Test
  public void runTest_wb_dma_ch_sel() {
    runTest("wb_dma_ch_sel",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_sel.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_arb.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }

  @Test
  public void runTest_wb_dma_de() {
    runTest("wb_dma_de",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_de.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_inc30r.v").toString());
  }

  @Test
  public void runTest_wb_dma_inc30r() {
    runTest("wb_dma_inc30r",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_inc30r.v").toString());
  }

  @Test
  public void runTest_wb_dma_pri_enc_sub() {
    runTest("wb_dma_pri_enc_sub",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }

  @Test
  public void runTest_wb_dma_rf() {
    runTest("wb_dma_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_rf.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_ch_rf.v").toString());
  }

  @Test
  public void runTest_wb_dma_wb_if() {
    runTest("wb_dma_wb_if",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_if.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_mast.v").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_slv.v").toString());
  }

  @Test
  public void runTest_wb_dma_wb_mast() {
    runTest("wb_dma_wb_mast",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_mast.v").toString());
  }

  @Test
  public void runTest_wb_dma_wb_slv() {
    runTest("wb_dma_wb_slv",
        "--include-path",
        Paths.get(getTestDirPath(), "oc_wb_dma").toString(),
        Paths.get("oc_wb_dma", "wb_dma_wb_slv.v").toString());
  }

  /*
   * os_sdram16
   */

  @Test
  public void runTest_hostcont_sdram16() {
    runTest("hostcont",
        "--include-path",
        Paths.get(getTestDirPath(), "os_sdram16").toString(),
        Paths.get("os_sdram16", "hostcont.v").toString());
  }

  @Test
  public void runTest_micro_sdram16() {
    runTest("micro",
        "--include-path",
        Paths.get(getTestDirPath(), "os_sdram16").toString(),
        Paths.get("os_sdram16", "micro.v").toString());
  }

  @Test
  public void runTest_os_sdram16() {
    runTest("os_sdram16",
        "--include-path",
        Paths.get(getTestDirPath(), "os_sdram16").toString(),
        Paths.get("os_sdram16", "os_sdram16.v").toString(),
        Paths.get("os_sdram16", "sdramcnt.v").toString(),
        Paths.get("os_sdram16", "hostcont.v").toString(),
        Paths.get("os_sdram16", "micro.v").toString());
  }

  @Test
  public void runTest_sdramcnt_sdram16() {
    runTest("sdramcnt",
        "--include-path",
        Paths.get(getTestDirPath(), "os_sdram16").toString(),
        Paths.get("os_sdram16", "sdramcnt.v").toString());
  }

  /*
   * ts_mike_fsm
   */

  @Test
  public void runTest_ts_mike_fsm() {
    runTest("ts_mike_fsm", Paths.get("ts_mike_fsm", "ts_mike_fsm.v").toString());
  }

  /*
   * xbar_16x16
   */

  @Test
  public void runTest_xbar_16x16() {
    runTest("xbar_16x16", Paths.get("xbar_16x16", "xbar_16x16.v").toString());
  }

  @Override
  public String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "hdl-benchmarks", "hdl", "quip").toString()
        + File.separator;
  }
}
