/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;
import ru.ispras.verilog.parser.util.VerilogBenchmarkTest;

import java.io.File;
import java.nio.file.Paths;

/**
 * Texas97 benchmark jUnit test cases.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VerilogTexas97TestSuite extends VerilogBenchmarkTest {

  /* Blackjack */

  @Test
  public void runTest_Blackjack_dp() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("Blackjack", "include.lib").toString(),
        Paths.get("Blackjack", "dp.v").toString());
  }

  @Test
  public void runTest_Blackjack_p12() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("Blackjack", "include.lib").toString(),
        Paths.get("Blackjack", "p12.v").toString());
  }

  @Test
  public void runTest_Blackjack_p13() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("Blackjack", "include.lib").toString(),
        Paths.get("Blackjack", "p13.v").toString());
  }

  @Test
  public void runTest_Blackjack_sp() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("Blackjack", "include.lib").toString(),
        Paths.get("Blackjack", "sp.v").toString());
  }

  /* cache_coherence */

  @Test
  public void runTest_cache_coherence_three_processor() {
    runTest("protocol",
        "--library-file",
        getTestDirPath() + Paths.get("cache_coherence", "include.lib").toString(),
        Paths.get("cache_coherence", "three_processor.v").toString());
  }

  @Test
  public void runTest_cache_coherence_three_processor_bin() {
    runTest("protocol",
        "--library-file",
        getTestDirPath() + Paths.get("cache_coherence", "include.lib").toString(),
        Paths.get("cache_coherence", "three_processor_bin.v").toString());
  }

  @Test
  public void runTest_cache_coherence_two_processor() {
    runTest("protocol",
        "--library-file",
        getTestDirPath() + Paths.get("cache_coherence", "include.lib").toString(),
        Paths.get("cache_coherence", "two_processor.v").toString());
  }

  @Test
  public void runTest_cache_coherence_two_processor_bin() {
    runTest("protocol",
        "--library-file",
        getTestDirPath() + Paths.get("cache_coherence", "include.lib").toString(),
        Paths.get("cache_coherence", "two_processor_bin.v").toString());
  }

  /* DLX/PDLX */

  @Test
  public void runTest_Dlx_Pdlx_alu_Alu() {
    runTest("ALU",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "alu.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_alu_AluControl() {
    runTest("ALUControl",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "alu.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_control() {
    runTest("Control",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString(),
        Paths.get("DLX", "PDLX", "control.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_DataMem() {
    runTest("DataMem",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_InstMem() {
    runTest("InstMem",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_Register() {
    runTest("Register",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_OneBitReg() {
    runTest("OneBitReg",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_ThreeBitReg() {
    runTest("ThreeBitReg",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_FiveBitReg() {
    runTest("FiveBitReg",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_Mux() {
    runTest("Mux",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_Add4() {
    runTest("Add4",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_SignExt() {
    runTest("SignExt",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_Ztest() {
    runTest("ZTest",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_misc_Clock() {
    runTest("Clock",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_pdlx() {
    runTest("pdlx",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "alu.v").toString(),
        Paths.get("DLX", "PDLX", "pdlx.v").toString());
  }

  @Test
  public void runTest_Dlx_Pdlx_RegFile() {
    runTest("RegFile",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "PDLX").toString(),
        Paths.get("DLX", "PDLX", "misc.v").toString(),
        Paths.get("DLX", "PDLX", "regfile.v").toString());
  }

  /* DLX/SDLX */

  @Test
  public void runTest_Dlx_Sdlx_alu_Alu() {
    runTest("ALU",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "alu.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_alu_AluControl() {
    runTest("ALUControl",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "alu.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_control() {
    runTest("Control",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "control.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_DataMem() {
    runTest("DataMem",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_Register() {
    runTest("Register",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_OneBitReg() {
    runTest("OneBitReg",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_Mux() {
    runTest("Mux",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_Add4() {
    runTest("Add4",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_SignExt() {
    runTest("SignExt",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_ZTest() {
    runTest("ZTest",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_misc_Clock() {
    runTest("Clock",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_RegFile() {
    runTest("RegFile",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "misc.v").toString(),
        Paths.get("DLX", "SDLX", "regfile.v").toString());
  }

  @Test
  public void runTest_Dlx_Sdlx_sdlx() {
    runTest("sdlx",
        "--include-path",
        getTestDirPath() + Paths.get("DLX", "SDLX").toString(),
        Paths.get("DLX", "SDLX", "sdlx.v").toString());
  }

  /* ifetch */

  @Test
  public void runTest_ifetch_IFetch() {
    runTest("instrFetch", Paths.get("ifetch", "IFetch.v").toString());
  }

  @Test
  public void runTest_ifetch_IFetchControl1() {
    runTest("IFetchControl", Paths.get("ifetch", "IFetchControl1.v").toString());
  }

  @Test
  public void runTest_ifetch_IFetchControl2() {
    runTest("IFetchControl", Paths.get("ifetch", "IFetchControl2.v").toString());
  }

  @Test
  public void runTest_ifetch_IFetchControl3() {
    runTest("IFetchControl", Paths.get("ifetch", "IFetchControl3.v").toString());
  }

  /* MPEG */

  @Test
  public void runTest_Mpeg_mpeg() {
    runTest("main", Paths.get("MPEG", "mpeg.v").toString());
  }

  @Test
  public void runTest_Mpeg_mpeg1() {
    runTest("main", Paths.get("MPEG", "mpeg1.v").toString());
  }

  @Test
  public void runTest_Mpeg_packet() {
    runTest("read_packet", Paths.get("MPEG", "packet.v").toString());
  }

  @Test
  public void runTest_Mpeg_packstart() {
    runTest("test", Paths.get("MPEG", "packstart.v").toString());
  }

  @Test
  public void runTest_Mpeg_parsepack() {
    runTest("parse_pack_header", Paths.get("MPEG", "parsepack.v").toString());
  }

  @Test
  public void runTest_Mpeg_parsesys() {
    runTest("parse_system_header", Paths.get("MPEG", "parsesys.v").toString());
  }

  @Test
  public void runTest_Mpeg_prefixcode() {
    runTest("start_code_prefix", Paths.get("MPEG", "prefixcode.v").toString());
  }

  @Test
  public void runTest_Mpeg_test1() {
    runTest("fsm_for_pack_start", Paths.get("MPEG", "test1.v").toString());
  }

  @Test
  public void runTest_Mpeg_test2() {
    runTest("parse_system_header", Paths.get("MPEG", "test2.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_main() {
    runTest("main", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_input_buffer() {
    runTest("input_buffer", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_counter() {
    runTest("counter", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_fsm_for_pack_start() {
    runTest("fsm_for_pack_start", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_parse_pack_header() {
    runTest("parse_pack_header", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_fsm_for_syshead_start() {
    runTest("fsm_for_syshead_start", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test3_parse_system_header() {
    runTest("parse_system_header", Paths.get("MPEG", "test3.v").toString());
  }

  @Test
  public void runTest_Mpeg_test4() {
    runTest("read_packet", Paths.get("MPEG", "test4.v").toString());
  }

  @Test
  public void runTest_Mpeg_timestamp() {
    runTest("time_stamps", Paths.get("MPEG", "timestamp.v").toString());
  }

  /* p6bus/vl */

  @Test
  public void runTest_p6bus_vl_p62_L_L_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_L_L_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_L_L_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_L_L_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_L_S_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_L_S_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_L_S_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_L_S_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_L_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_L_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_L_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_L_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_Ls_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_LS_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_Ls_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_LS_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_S_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_S_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Ls_S_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_LS_S_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_L_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_L_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_L_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_L_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_Ls_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_LS_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_Ls_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_LS_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_Nd_V() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_ND_V.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_Nd_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_ND_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_Nd_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_ND_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_S_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_S_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_Nd_S_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_ND_S_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_S_S_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_S_S_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_S_S_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_S_S_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_V_Ls_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_V_LS_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_V_Ls_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_V_LS_V02.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_V_S_V() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_V_S_V.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_V_S_V01() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_V_S_V01.v").toString());
  }

  @Test
  public void runTest_p6bus_vl_p62_V_S_V02() {
    runTest("PENTIUM_PRO_SYSTEM",
        "--library-file",
        getTestDirPath() + Paths.get("p6bus", "vl", "include.lib").toString(),
        Paths.get("p6bus", "vl", "p62_V_S_V02.v").toString());
  }

  /* PCI_BUS/Verilog-MV-files */

  @Test
  public void runTest_Pci_Bus_Verilog_Mv_files_PciAbnorm() {
    runTest("bus",
        "--library-file",
        getTestDirPath() + Paths.get("PCI_BUS", "Verilog-MV-files", "include.lib").toString(),
        Paths.get("PCI_BUS", "Verilog-MV-files", "PCIabnorm.v").toString());
  }

  @Test
  public void runTest_Pci_Bus_Verilog_Mv_files_PciNorm() {
    runTest("bus",
        "--library-file",
        getTestDirPath() + Paths.get("PCI_BUS", "Verilog-MV-files", "include.lib").toString(),
        Paths.get("PCI_BUS", "Verilog-MV-files", "PCInorm.v").toString());
  }

  /* PI_BUS/multi_master */

  @Test
  public void runTest_Pi_Bus_multi_master_bus() {
    runTest("bus_cont",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master").toString(),
        "--library-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master", "include.lib").toString(),
        Paths.get("PI_BUS", "multi_master", "main.v").toString(),
        Paths.get("PI_BUS", "multi_master", "bus.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_multi_master_main() {
    runTest("main",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master").toString(),
        "--library-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master", "include.lib").toString(),
        Paths.get("PI_BUS", "multi_master", "main.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_multi_master_master2() {
    runTest("master_controller",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master", "include.lib").toString(),
        Paths.get("PI_BUS", "multi_master", "main.v").toString(),
        Paths.get("PI_BUS", "multi_master", "master2.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_multi_master_slave() {
    runTest("pi_state_machine",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "multi_master", "include.lib").toString(),
        Paths.get("PI_BUS", "multi_master", "main.v").toString(),
        Paths.get("PI_BUS", "multi_master", "slave.v").toString());
  }

  /* PI_BUS/single_master */

  @Test
  public void runTest_Pi_Bus_single_master_bus() {
    runTest("bus_cont",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "single_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "single_master", "include.lib").toString(),
        Paths.get("PI_BUS", "single_master", "main.v").toString(),
        Paths.get("PI_BUS", "single_master", "bus.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_single_master_main() {
    runTest("main",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "single_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "single_master", "include.lib").toString(),
        Paths.get("PI_BUS", "single_master", "main.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_single_master_main2() {
    runTest("main",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "single_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "single_master", "include.lib").toString(),
        Paths.get("PI_BUS", "single_master", "main.v").toString(),
        Paths.get("PI_BUS", "single_master", "bus.v").toString(),
        Paths.get("PI_BUS", "single_master", "master2.v").toString(),
        Paths.get("PI_BUS", "single_master", "slave.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_single_master_master2() {
    runTest("master_controller",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "single_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "single_master", "include.lib").toString(),
        Paths.get("PI_BUS", "single_master", "main.v").toString(),
        Paths.get("PI_BUS", "single_master", "master2.v").toString());
  }

  @Test
  public void runTest_Pi_Bus_single_master_slave() {
    runTest("pi_state_machine",
        "--include-path",
        getTestDirPath() + Paths.get("PI_BUS", "single_master").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PI_BUS", "single_master", "include.lib").toString(),
        Paths.get("PI_BUS", "single_master", "main.v").toString(),
        Paths.get("PI_BUS", "single_master", "slave.v").toString());
  }

  /* PPC60X_bus/src */

  @Test
  public void runTest_Ppc60x_bus_src_arbiter() {
    runTest("arb2",
        "--include-path",
        getTestDirPath() + Paths.get("PPC60X_bus", "src").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "arbiter.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_cpu() {
    runTest("cpu",
        "--include-path",
        getTestDirPath() + Paths.get("PPC60X_bus", "src").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "cpu.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_define() {
    runTest(null, Paths.get("PPC60X_bus", "src", "define.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_main1() {
    runTest("main",
        "--include-path",
        getTestDirPath() + Paths.get("PPC60X_bus", "src").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "arbiter.v").toString(),
        Paths.get("PPC60X_bus", "src", "cpu.v").toString(),
        Paths.get("PPC60X_bus", "src", "mem.v").toString(),
        Paths.get("PPC60X_bus", "src", "main1.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_main2() {
    runTest("main",
        "--include-path",
        getTestDirPath() + Paths.get("PPC60X_bus", "src").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "arbiter.v").toString(),
        Paths.get("PPC60X_bus", "src", "cpu.v").toString(),
        Paths.get("PPC60X_bus", "src", "mem.v").toString(),
        Paths.get("PPC60X_bus", "src", "main2.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_mem() {
    runTest("mem",
        "--include-path",
        getTestDirPath() + Paths.get("PPC60X_bus", "src").toString(),
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "mem.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_One() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "ONE.v").toString());
  }

  @Test
  public void runTest_Ppc60x_bus_src_Two() {
    runTest("main",
        "--library-file",
        getTestDirPath() + Paths.get("PPC60X_bus", "src", "include.lib").toString(),
        Paths.get("PPC60X_bus", "src", "TWO.v").toString());
  }

  @Override
  public String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "hdl-benchmarks", "hdl", "texas97").toString()
        + File.separator;
  }
}
