`ifndef _MY_FIFO_
`define _MY_FIFO_

`define NAME myfifo

`include "mem_2p.v"

`timescale 1ns/100ps

module `NAME (
DO,
DO_VAL,
IS_EMPTY,
IS_FULL,
CLK,
RST,
DI,
CE_RD,
CE_WR
);

//------Parameters declaration:---------
	parameter  ADDR_SIZE   = 4;
	parameter  DATA_SIZE   = 8;
//--------------------------------------

output [(DATA_SIZE-1):0] DO;      // Data Output
output DO_VAL; // Data Output Valid
output IS_EMPTY; // Fifo is empty
output IS_FULL; // Fifo is full

input  CLK;            // Clock
input  RST;            // Reset
input  [(DATA_SIZE-1):0] DI;      // Data Input
input  CE_RD;            // Read Chip Select Enable
input  CE_WR;            // Write Chip Select Enable

reg [(DATA_SIZE-1):0] DO;
reg  DO_VAL;
reg  DO_R_VAL;
reg  IS_EMPTY;
reg  IS_RR_FULL;
reg  IS_R_FULL;

assign IS_FULL = (IS_RR_FULL | IS_R_FULL);


reg [(ADDR_SIZE-1):0] RD_A;        //Read Address
reg [(ADDR_SIZE-1):0] WR_A;        //Write Address
reg [(ADDR_SIZE-1):0] RD_A_1;        //Read Address + 1
reg [(ADDR_SIZE-1):0] WR_A_1;        //Write Address + 1

wire [(DATA_SIZE-1):0] DO_tmp;
// reg  [(DATA_SIZE-1):0] DI_tmp;
// reg  [(DATA_SIZE-1):0] DI_R_tmp;

reg CE_RD_tmp;
reg CE_WR_tmp;
reg [(DATA_SIZE-1):0] DI_tmp;


mem_2p #(ADDR_SIZE,DATA_SIZE) mem(
.DO(DO_tmp),
.RD_A(RD_A),
.WR_A(WR_A),
.DI(DI_tmp),
.RD_CLK(CLK),
.WR_CLK(CLK),
.CE_N_RD(~CE_RD_tmp),
.CE_N_WR(~CE_WR_tmp),
.OE_N(1'b0)
);

always @(posedge CLK) begin
    if(RST) begin
        DO_VAL <= 1'b0;
        DO_R_VAL <= 1'b0;
        IS_EMPTY <= 1'b1;
        IS_R_FULL <= 1'b0;
        IS_RR_FULL <= 1'b0;

        RD_A <= {(1 << ADDR_SIZE){1'b1}};
        WR_A <= {(1 << ADDR_SIZE){1'b1}};
        RD_A_1 <= {(1 << ADDR_SIZE){1'b0}};
        WR_A_1 <= {(1 << ADDR_SIZE){1'b0}};

        CE_RD_tmp <= 1'b0;
        CE_WR_tmp <= 1'b0;
    end
end

always @(posedge CLK) begin
    if(CE_RD_tmp) begin
        DO_R_VAL <= 1'b1;
        // перенесли на такт позже чтобы занятость не спадала в том же такте
    end else begin
        DO_R_VAL <= 1'b0;
    end

    DO <= DO_tmp;
    DO_VAL <= DO_R_VAL;

    DI_tmp <= DI;
end


always @(posedge CLK) begin

    if(CE_RD && !IS_EMPTY) begin
        if(!CE_WR) begin
            if(RD_A_1 == WR_A) begin
               IS_EMPTY <= 1'b1;
            end
        end
        IS_R_FULL <= 1'b0;
        RD_A <= RD_A_1;
        RD_A_1 <= RD_A_1+1;

        CE_RD_tmp <= CE_RD;
    end else begin
        CE_RD_tmp <= 1'b0; // чтение не прошло
    end

    if(CE_WR && !IS_R_FULL) begin
        if(!CE_RD) begin
            if(WR_A_1 == RD_A) begin
                IS_R_FULL <= 1'b1;
            end
        end
        IS_EMPTY <= 1'b0;
        WR_A <= WR_A_1;
        WR_A_1 <= WR_A_1+1;

        CE_WR_tmp <= CE_WR;
    end else begin
        CE_WR_tmp <= 1'b0; // запись не прошла
    end

    IS_RR_FULL <= IS_R_FULL;

end



endmodule

`endif
