/*
 * Copyright 2019-2020 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;
import ru.ispras.verilog.parser.util.VerilogBenchmarkTest;

import java.io.File;
import java.nio.file.Paths;

/**
 * IWLS2005 benchmarks based test cases.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VerilogIwlsTestSuite extends VerilogBenchmarkTest {

  /*
   * faraday
   */

  /* DMA */

  @Test
  public void runTest_dma_ahbdec() {
    runTest("dma_ahbdec",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbdec.v").toString());
  }

  @Test
  public void runTest_dma_ahbmst() {
    runTest("dma_ahbmst",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbmst.v").toString());
  }

  @Test
  public void runTest_dma_ahbmux() {
    runTest("dma_ahbmux",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbmux.v").toString());
  }

  @Test
  public void runTest_dma_ahbslv() {
    runTest("dma_ahbslv",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbslv.v").toString());
  }

  @Test
  public void runTest_dma_chrf() {
    runTest("dma_chrf",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_chrf.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'dma_chsel.arb_chcsr_reg' has not been found
  /*@Test
  public void runTest_dma_chsel() {
    runTest("dma_chsel",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_chsel.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_rrarb.v").toString());
  }*/

  @Test
  public void runTest_dma_ctlrf() {
    runTest("dma_ctlrf",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ctlrf.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_chrf.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'dma_engine.upk_cnt0_fix' has not been found
  /*@Test
  public void runTest_dma_engine() {
    runTest("dma_engine",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_engine.v").toString());
  }*/

  @Test
  public void runTest_dma_fifo() {
    runTest("dma_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_fifo.v").toString());
  }

  @Test
  public void runTest_dma_rrarb() {
    runTest("dma_rrarb",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_rrarb.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'dma_top.de.upk_cnt0_fix' has not been found
  /*@Test
  public void runTest_dma_top() {
    runTest("dma_top",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DMA", "hdl").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_top.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbdec.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbmst.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbmux.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ahbslv.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_chrf.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_chsel.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_ctlrf.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_engine.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_fifo.v").toString(),
        Paths.get("faraday", "rtl", "DMA", "hdl", "dma_rrarb.v").toString());
  }*/

  /* DSP */

  @Test
  public void runTest_dsp_bdma() {
    runTest("BDMA",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BDMA", "m_bdma.v").toString());
  }

  @Test
  public void runTest_dsp_bdma_dummy() {
    runTest("BDMA",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "BDMA").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BDMA", "m_bdma_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_btb() {
    runTest("BTB",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb.v").toString());
  }

  @Test
  public void runTest_dsp_btb_dummy() {
    runTest("BTB", Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_btbmemx() {
    runTest("BTBmem",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_btbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 1465
  /*@Test
  public void runTest_dsp_codec_adc162h90a() {
    runTest("ADC162H90A_dtop",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "DAC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "MCLK_Delay.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ram_32_44.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "decode.v")
            .toString());
  }*/

  // TODO: ADDA162H90A_atop.v line 120:47 mismatched input ':' expecting RPAREN
  /*@Test
  public void runTest_dsp_codec_adda162h90a() {
    runTest("ADDA162H90A_atop",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADDA162H90A_atop.v")
            .toString());
  }*/

  // TODO: ERROR: Starting points limit has been exhausted: 1465
  /*@Test
  public void runTest_dsp_codec_dac162h90a() {
    runTest("DAC162H90A_dtop",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "DAC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "decode.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "fir_8x.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "MCLK_Delay.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ram_32_44.v")
            .toString());
  }*/

  @Test
  public void runTest_dsp_codec_decode() {
    runTest("codec_decode",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "decode.v")
            .toString());
  }

  @Test
  public void runTest_dsp_codec_fir() {
    runTest("fir_8x",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "fir_8x.v")
            .toString());
  }

  // TODO: ADDA162H90A_atop.v line 120:47 mismatched input ':' expecting RPAREN
  /*@Test
  public void runTest_dsp_codec_fs90b779() {
    runTest("FS90B779",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779_DAI.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FXADDA162H90A.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "DAC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADDA162H90A_atop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "MCLK_Delay.v")
            .toString());
  }*/

  @Test
  public void runTest_dsp_codec_fs90b779_dai() {
    runTest("FS90B779_DAI",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779_DAI.v")
            .toString());
  }

  @Test
  public void runTest_dsp_codec_fs90b779_dai_loop() {
    runTest("FS90B779_DAI",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779_DAI_loop.v")
            .toString());
  }

  // TODO: ADDA162H90A_atop.v line 120:47 mismatched input ':' expecting RPAREN
  /*@Test
  public void runTest_dsp_codec_fs90b779_sim() {
    runTest("FS90B779_sim",
        "--library-file",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A",
            "tasks.lib").toString(),
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADDA162H90A_atop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "DAC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FXADDA162H90A.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779_DAI.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779_sim.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FS90B779.v")
            .toString());
  }*/

  // TODO: ADDA162H90A_atop.v line 120:47 mismatched input ':' expecting RPAREN
  /*@Test
  public void runTest_dsp_codec_fxadda162h90a() {
    runTest("FXADDA162H90A",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FXADDA162H90A.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADDA162H90A_atop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ADC162H90A_dtop.v")
            .toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "DAC162H90A_dtop.v")
            .toString());
  }*/

  @Test
  public void runTest_dsp_codec_fxadda162h90a_syn() {
    runTest("FXADDA162H90A",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "FXADDA162H90A_syn.v")
            .toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 1441
  /*@Test
  public void runTest_dsp_codec_ram_32_44() {
    runTest("ram_32_44",
        Paths.get("faraday", "rtl", "DSP", "hdl", "CODEC", "FXADDA162H90A", "ram_32_44.v")
            .toString());
  }*/

  @Test
  public void runTest_dsp_dag_adder15() {
    runTest("CLA15",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_adder15.v").toString());
  }

  @Test
  public void runTest_dsp_dag_cla15() {
    runTest("CLA15_0",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "DAG").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_cla15.v").toString());
  }

  @Test
  public void runTest_dsp_dag_csa15() {
    runTest("CSA15",
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa15.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_dag_csa() {
    runTest("CSA_d", Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa_d.v").toString());
  }

  @Test
  public void runTest_dsp_dag_dag() {
    runTest("DAG",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "DAG").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_dag.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa_d.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM1reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM2reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL2.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_dag_ilm1reg() {
    runTest("ILM1REG",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "DAG").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM1reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_dag_ilm2reg() {
    runTest("ILM2REG",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "DAG").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM2reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_dag_lpn() {
    runTest("DLPN",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_lpn.v").toString());
  }

  @Test
  public void runTest_dsp_dag_modulol1() {
    runTest("MODULO1",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa_d.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL1.v").toString());
  }

  @Test
  public void runTest_dsp_dag_modulol2() {
    runTest("MODULO2",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa_d.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL2.v").toString());
  }

  @Test
  public void runTest_dsp_dec() {
    runTest("DEC",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DEC", "c_dec.v").toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 939
  /*@Test
  public void runTest_dsp_dft() {
    runTest("DSP_CORE_top",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DFT", "wrapper.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_dag.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "e_eu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_btbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_rtbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DEC", "c_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_psq.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BDMA", "m_bdma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "i_sice.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "IDMA", "m_idma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_emc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_memc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PIO", "pio.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_autoctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_scfg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_screg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_sport0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_rxctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_txctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_rxctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_scfg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_screg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_sport1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_txctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TM", "Timer.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_clkctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_dsp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_glogic.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_rego.v").toString());
  }*/

  @Test
  public void runTest_dsp_eu() {
    runTest("EU",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "e_eu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_alu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "CUN", "ec_cun.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mac.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_sht.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_ea_alu() {
    runTest("EA_ALU",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "ALU").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_alu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_ea_core() {
    runTest("EA_CORE",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "ALU").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_ea_dec() {
    runTest("EA_DEC",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "ALU").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString());
  }

  @Test
  public void runTest_dsp_ea_reg() {
    runTest("EA_REG",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "ALU").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_ea_cun() {
    runTest("EC_CUN",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "CUN").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "CUN", "ec_cun.v").toString());
  }

  @Test
  public void runTest_dsp_mac_adder() {
    runTest("EM_CLA36",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_adder.v").toString());
  }

  @Test
  public void runTest_dsp_mac_array() {
    runTest("EM_ARRAY",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_array.v").toString());
  }

  @Test
  public void runTest_dsp_mac_core() {
    runTest("EM_CORE",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "MAC").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString());
  }

  @Test
  public void runTest_dsp_mac_dec() {
    runTest("EM_DEC",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString());
  }

  @Test
  public void runTest_dsp_mac_mac() {
    runTest("EM_MAC",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mac.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_mac_mvovf() {
    runTest("EM_MVOVF",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "MAC").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString());
  }

  @Test
  public void runTest_dsp_mac_part() {
    runTest("EM_PART",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "MAC").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString());
  }

  @Test
  public void runTest_dsp_mac_psel() {
    runTest("PSEL",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_psel_part.v").toString());
  }

  @Test
  public void runTest_dsp_mac_ppart() {
    runTest("PPART",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_psel_part.v").toString());
  }

  @Test
  public void runTest_dsp_mac_reg() {
    runTest("EM_REG",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString());
  }

  @Test
  public void runTest_dsp_sht_array() {
    runTest("ES_ARRAY",
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString());
  }

  @Test
  public void runTest_dsp_sht_dec() {
    runTest("ES_DEC",
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString());
  }

  @Test
  public void runTest_dsp_sht_exp() {
    runTest("ES_EXP",
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString());
  }

  @Test
  public void runTest_dsp_sht_reg() {
    runTest("ES_REG",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "SHT").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString());
  }

  @Test
  public void runTest_dsp_sht_sht() {
    runTest("ES_SHT",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "EU", "SHT").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_sht.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  // TODO: [Internal] Descriptor for 'SICE.R_RST_1' has not been found
  /*@Test
  public void runTest_dsp_ice() {
    runTest("SICE",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "i_sice.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }*/

  @Test
  public void runTest_dsp_ice_dummy() {
    runTest("SICE",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "i_sice_dummy.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_ice_parser() {
    runTest("SPARSER",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "x_parser.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "x_parsm.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "idebn.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_ice_parsm_wriar() {
    runTest("WRIAR",
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "x_parsm.v").toString());
  }

  @Test
  public void runTest_dsp_ice_wridr() {
    runTest("WRIDR",
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "x_parsm.v").toString());
  }

  @Test
  public void runTest_dsp_ice_rdidr() {
    runTest("RDIDR",
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "x_parsm.v").toString());
  }

  @Test
  public void runTest_dsp_idma() {
    runTest("IDMA",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "IDMA").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "IDMA", "m_idma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_idma_dummy() {
    runTest("IDMA",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "IDMA", "m_idma_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_lib_idebn() {
    runTest("IDEBN", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "idebn.v").toString());
  }

  @Test
  public void runTest_dsp_lib_cm4k() {
    runTest("CM4k",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_cm4k.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_lib_cm8k() {
    runTest("CM8k",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_cm8k.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_lib_dm8k() {
    runTest("DM8k",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_dm8k.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_lib_ecm32kx24() {
    runTest("ECM32kx24",
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_ecm32kx24.v").toString());
  }

  @Test
  public void runTest_dsp_lib_edm8k() {
    runTest("EDM8k", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_edm8k.v").toString());
  }

  @Test
  public void runTest_dsp_lib_eeprom() {
    runTest("EEPROM", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_eeprom.v").toString());
  }

  @Test
  public void runTest_dsp_lib_eio2k() {
    runTest("EIO2k", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_eio2k.v").toString());
  }

  @Test
  public void runTest_dsp_lib_em4k() {
    runTest("EM4K", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_em4k.v").toString());
  }

  @Test
  public void runTest_dsp_lib_em8k() {
    runTest("EM8K", Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_em8k.v").toString());
  }

  @Test
  public void runTest_dsp_lib_pm4k() {
    runTest("PM4k",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "m_pm4k.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_memc_emc() {
    runTest("EMC",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "MEMC").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_emc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_memc_emc_dummy() {
    runTest("EMC",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "MEMC").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_emc_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_memc_memc() {
    runTest("MEMC",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "MEMC").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_memc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_pio_pio() {
    runTest("PIO",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "PIO").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PIO", "pio.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_pio_dummy() {
    runTest("PIO", Paths.get("faraday", "rtl", "DSP", "hdl", "PIO", "pio_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_psq_cntstk() {
    runTest("CNTstk",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "PSQ").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_cntstk.v").toString());
  }

  @Test
  public void runTest_dsp_psq_lpstk() {
    runTest("LPstk",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "PSQ").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_lpstk.v").toString());
  }

  @Test
  public void runTest_dsp_psq_pcstk() {
    runTest("PCstk",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "PSQ").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_pcstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_psq_psq() {
    runTest("PSQ",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_psq.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_cntstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_lpstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_pcstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_ststk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_psq_ststk() {
    runTest("STstk",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "PSQ").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_ststk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_autoctl() {
    runTest("AUTOctl",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT0").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_autoctl.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_compand() {
    runTest("COMPAND",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT0").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compress.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_expand.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_compress() {
    runTest("COMPRESS",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT0").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compress.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_expand() {
    runTest("EXPAND",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_expand.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_rxctl0() {
    runTest("RXctl0",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_rxctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_scfg0() {
    runTest("SCFG0",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_scfg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_screg0() {
    runTest("SCreg0",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_screg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_sport0() {
    runTest("SPORT0",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_sport0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compress.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_expand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_rxctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_screg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_scfg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_txctl0.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_dummy() {
    runTest("SPORT0",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT0").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_sport0_dummy.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString());
  }

  @Test
  public void runTest_dsp_sport0_txctl0() {
    runTest("TXctl0",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_txctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_rxctl1() {
    runTest("RXctl1",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_rxctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_scfg1() {
    runTest("SCFG1",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_scfg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_screg1() {
    runTest("SCreg1",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_screg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_sport1() {
    runTest("SPORT1",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_sport1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_rxctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_scfg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_screg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_txctl1.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_dummy() {
    runTest("SPORT1",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_sport1_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_sport1_txctl1() {
    runTest("TXctl1",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_txctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_tm_timer() {
    runTest("TM",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "TM").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TM", "Timer.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_tm_dummy() {
    runTest("TM", Paths.get("faraday", "rtl", "DSP", "hdl", "TM", "Timer_dummy.v").toString());
  }

  @Test
  public void runTest_dsp_top_clkctl() {
    runTest("CLKC",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "TOP").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_clkctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_top_core() {
    runTest("CORE",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa15.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_dag.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL2.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM1reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM2reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DEC", "c_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mac.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_sht.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "e_eu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_alu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "CUN", "ec_cun.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_cntstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_lpstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_pcstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_psq.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_ststk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_btbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_rtbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  // TODO: Descriptor for 'DSP_CORE.sice.R_RST_1' has not been found
  /*@Test
  public void runTest_dsp_top_dsp() {
    runTest("DSP_CORE",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_dsp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BDMA", "m_bdma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_btbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_rtbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa15.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_dag.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM1reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM2reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL2.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DEC", "c_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "e_eu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_alu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "CUN", "ec_cun.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mac.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_sht.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "i_sice.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "IDMA", "m_idma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_emc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_memc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PIO", "pio.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_psq.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_cntstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_lpstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_pcstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_ststk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_autoctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compress.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_expand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_scfg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_screg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_sport0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_rxctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_txctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_rxctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_scfg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_screg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_sport1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_txctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TM", "Timer.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_clkctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_glogic.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_rego.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }*/

  @Test
  public void runTest_dsp_top_glogic() {
    runTest("Glogic",
        "--include-path",
        String.format("%s%s%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "TOP").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_glogic.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_top_muxio() {
    runTest("MUX_IO", Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_muxio.v").toString());
  }

  @Test
  public void runTest_dsp_top_pin013() {
    runTest("PINs",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_pin.013.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_top_pin018() {
    runTest("PINs",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_pin.018.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_top_pin035() {
    runTest("PINs",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_pin.035.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "include", "x_def.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }

  @Test
  public void runTest_dsp_top_pinrtl() {
    runTest("PINs",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_pin_rtl.v").toString());
  }

  @Test
  public void runTest_dsp_top_rego() {
    runTest("REGo",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "SPORT1").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_rego.v").toString());
  }

  // TODO: ERROR: Starting points limit has been exhausted: 594
  /*@Test
  public void runTest_dsp_top_top() {
    runTest("TOP",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "hdl", "LIB").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "DSP", "include").toString()),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_top.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BDMA", "m_bdma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "c_btb.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_btbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "BTB", "m_rtbmemx.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_csa15.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_dag.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM1reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_ILM2reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DAG", "d_moduloL2.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "DEC", "c_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "e_eu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_alu.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "ALU", "ea_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "CUN", "ec_cun.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mac.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_mvovf.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_part.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "MAC", "em_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_array.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_dec.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_exp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_reg.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "EU", "SHT", "es_sht.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "IDMA", "m_idma.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "ICE", "i_sice.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_emc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "MEMC", "m_memc.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PIO", "pio.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_cntstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_lpstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_pcstk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_psq.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "PSQ", "c_ststk.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TM", "Timer.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_core.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_clkctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_dsp.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_glogic.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_muxio.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_pin.013.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "TOP", "t_rego.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_autoctl.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_compress.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_expand.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_rxctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_scfg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_screg0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_sport0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT0", "s_txctl0.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_rxctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_scfg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_screg1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_sport1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "SPORT1", "s_txctl1.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_gclk_lib.v").toString(),
        Paths.get("faraday", "rtl", "DSP", "hdl", "LIB", "x_lib.v").toString());
  }*/

  /* RISC */

  @Test
  public void runTest_risc_abc() {
    runTest("c0cont",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString());
  }

  @Test
  public void runTest_risc_abcd() {
    runTest("core",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcd.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString());
  }

  @Test
  public void runTest_risc_abcde() {
    runTest("jpt",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString());
  }

  @Test
  public void runTest_risc_abcdef() {
    runTest("lmi_lbc_cbi",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcdef.v").toString());
  }

  @Test
  public void runTest_risc_bcd() {
    runTest("fifo_ctl",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcd.v").toString());
  }

  @Test
  public void runTest_risc_bcde() {
    runTest("dmux",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString());
  }

  @Test
  public void runTest_risc_bcdef() {
    runTest("ejtag",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrst.v").toString());
  }

  @Test
  public void runTest_risc_bcdefg() {
    runTest("lmi_lbc",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxyz.v").toString());
  }

  @Test
  public void runTest_risc_cde() {
    runTest("lbc_data",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efgh.v").toString());
  }

  @Test
  public void runTest_risc_cdef() {
    runTest("ce_hl",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdef.v").toString());
  }

  @Test
  public void runTest_risc_cdefg() {
    runTest("lbc_cbus",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcd.v").toString());
  }

  @Test
  public void runTest_risc_cdefgh() {
    runTest("mpcs50",
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString());
  }

  @Test
  public void runTest_risc_def() {
    runTest("lbc_async",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "def.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString());
  }

  @Test
  public void runTest_risc_defg() {
    runTest("ce_hlw",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdef.v").toString());
  }

  // TODO: [Internal] Bit vector sizes do not match: 32 != 2.
  /*@Test
  public void runTest_risc_defgh() {
    runTest("lbc_lbus",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString());
  }*/

  @Test
  public void runTest_risc_defghi() {
    runTest("mux32", Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString());
  }

  // TODO: [Internal] Bit vector sizes do not match: 32 != 2.
  /*@Test
  public void runTest_risc_efg() {
    runTest("lbc_lbi",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "def.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString());
  }*/

  @Test
  public void runTest_risc_efgh() {
    runTest("fifo_data",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efgh.v").toString());
  }

  // TODO: [Internal] 0 must be > 0
  /*@Test
  public void runTest_risc_efghi() {
    runTest("macd",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrst.v").toString());
  }*/

  @Test
  public void runTest_risc_efghij() {
    runTest("lmi_watchb",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghij.v").toString());
  }

  @Test
  public void runTest_risc_fgh() {
    runTest("lmi_icache",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString());
  }

  @Test
  public void runTest_risc_fghi() {
    runTest("lx0",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcd.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxyz.v").toString());
  }

  // TODO: ERROR: [Internal] 0 must be > 0
  /*@Test
  public void runTest_risc_fghij() {
    runTest("macfile",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghij.v").toString());
  }*/

  @Test
  public void runTest_risc_fghijk() {
    runTest("lmi_watchd",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghijk.v").toString());
  }

  // TODO: ERROR: [Internal] Bit vector sizes do not match: 32 != 2.
  /*@Test
  public void runTest_risc_ghi() {
    runTest("lbc",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "def.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efg.v").toString());
  }*/

  // TODO: ERROR: [Internal] Bit vector sizes do not match: 32 != 2
  /*@Test
  public void runTest_risc_ghij() {
    runTest("lx1",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcd.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "def.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwxy.v").toString());
  }*/

  @Test
  public void runTest_risc_ghijk() {
    runTest("macu",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString());
  }

  @Test
  public void runTest_risc_ghijkl() {
    runTest("pcont_copif",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString());
  }

  @Test
  public void runTest_risc_hij() {
    runTest("ejtag_dataout",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hij.v").toString());
  }

  @Test
  public void runTest_risc_hijk() {
    runTest("ejtag_async",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijk.v").toString());
  }

  // TODO: ERROR: [Internal] 0 must be > 0
  /*@Test
  public void runTest_risc_hijkl() {
    runTest("macw",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrst.v").toString());
  }*/

  @Test
  public void runTest_risc_hijklm() {
    runTest("pcont_ralu",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString());
  }

  @Test
  public void runTest_risc_ijk() {
    runTest("jtag_tap",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijk.v").toString());
  }

  @Test
  public void runTest_risc_ijkl() {
    runTest("ejtag_control",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrst.v").toString());
  }

  @Test
  public void runTest_risc_ijklm() {
    runTest("lmi_align", Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString());
  }

  @Test
  public void runTest_risc_ijklmn() {
    runTest("reset_dist",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklmn.v").toString());
  }

  @Test
  public void runTest_risc_jkl() {
    runTest("c0dpath",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString());
  }

  @Test
  public void runTest_risc_jklm() {
    runTest("ejtag_datain", Paths.get("faraday", "rtl", "RISC", "hdl", "jklm.v").toString());
  }

  @Test
  public void runTest_risc_jklmn() {
    runTest("mcsa6",
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString());
  }

  @Test
  public void runTest_risc_jklmno() {
    runTest("rf_if",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString());
  }

  @Test
  public void runTest_risc_klmn() {
    runTest("ejtag_dclk",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmn.v").toString());
  }

  @Test
  public void runTest_risc_klmno() {
    runTest("mcst50",
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString());
  }

  @Test
  public void runTest_risc_klmnop() {
    runTest("regfile",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString());
  }

  @Test
  public void runTest_risc_lmno() {
    runTest("ejtag_dmadata",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmno.v").toString());
  }

  @Test
  public void runTest_risc_lmnop() {
    runTest("lmi_dcache_bus",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqrs.v").toString());
  }

  @Test
  public void runTest_risc_mno() {
    runTest("lmi_icache_sm",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString());
  }

  @Test
  public void runTest_risc_mnop() {
    runTest("ejtag_dmatch_4180",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnop.v").toString());
  }

  @Test
  public void runTest_risc_mnopq() {
    runTest("lmi_dcache_sm",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString());
  }

  @Test
  public void runTest_risc_nop() {
    runTest("ce_buf",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nop.v").toString());
  }

  @Test
  public void runTest_risc_nopq() {
    runTest("ejtag_imatch_4180",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopq.v").toString());
  }

  @Test
  public void runTest_risc_nopqr() {
    runTest("lmi_dcache_tag",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString());
  }

  @Test
  public void runTest_risc_opq() {
    runTest("dcont",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString());
  }

  @Test
  public void runTest_risc_opqr() {
    runTest("ejtag_pmatch",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqr.v").toString());
  }

  @Test
  public void runTest_risc_opqrs() {
    runTest("lmi_dcache",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqrs.v").toString());
  }

  @Test
  public void runTest_risc_pqr() {
    runTest("ralu",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString());
  }

  @Test
  public void runTest_risc_pqrs() {
    runTest("ejtag_scan",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrs.v").toString());
  }

  @Test
  public void runTest_pqrst() {
    runTest("mdiv",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrst.v").toString());
  }

  @Test
  public void runTest_risc_qrst() {
    runTest("ejtag_setclr", Paths.get("faraday", "rtl", "RISC", "hdl", "qrst.v").toString());
  }

  @Test
  public void runTest_risc_qrstu() {
    runTest("lmi_dram_bus",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuvw.v").toString());
  }

  @Test
  public void runTest_risc_rst() {
    runTest("copif_bus",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString());
  }

  @Test
  public void runTest_risc_rstu() {
    runTest("lmi_icache_bus",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString());
  }

  @Test
  public void runTest_risc_rstuv() {
    runTest("lmi_dram_cmp",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString());
  }

  @Test
  public void runTest_risc_stu() {
    runTest("cop0",
        "--include-path",
        Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString());
  }

  @Test
  public void runTest_risc_stuv() {
    runTest("lmi_icache_tag",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString());
  }

  @Test
  public void runTest_risc_stuvw() {
    runTest("lmi_dram",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString());
  }

  @Test
  public void runTest_risc_tuv() {
    runTest("copif",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString());
  }

  @Test
  public void runTest_risc_tuvw() {
    runTest("lmi_iram_bus",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxy.v").toString());
  }

  @Test
  public void runTest_risc_tuvwx() {
    runTest("mfa", Paths.get("faraday", "rtl", "RISC", "hdl", "tuvwx.v").toString());
  }

  @Test
  public void runTest_risc_uvw() {
    runTest("coplogic",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString());
  }

  @Test
  public void runTest_risc_uvwx() {
    runTest("lmi_iram_cmp",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString());
  }

  @Test
  public void runTest_risc_uvwxy() {
    runTest("lmi_watch",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghijk.v").toString());
  }

  @Test
  public void runTest_risc_vwx() {
    runTest("core_lmi",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwx.v").toString());
  }

  @Test
  public void runTest_risc_vwxy() {
    runTest("lmi_iram",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString());
  }

  @Test
  public void runTest_risc_vwxyz() {
    runTest(
        "pcont_cop0",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString());
  }

  @Test
  public void runTest_risc_wxy() {
    runTest("daddr",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString());
  }

  @Test
  public void runTest_risc_wxyz() {
    runTest("lmi_lbc_align", Paths.get("faraday", "rtl", "RISC", "hdl", "wxyz.v").toString());
  }

  @Test
  public void runTest_risc_xyz() {
    runTest("dalu",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString());
  }

  // TODO: ERROR: [Internal] Bit vector sizes do not match: 32 != 2.
  /*@Test
  public void runTest_risc_zzzz() {
    runTest("frisc",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "hdl").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "faraday", "rtl", "RISC", "include").toString()),
        Paths.get("faraday", "rtl", "RISC", "hdl", "zzzz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abc.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcd.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "abcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "bcdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cde.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdef.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "cdefgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "def.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "defghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efg.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "efghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fgh.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "fghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghi.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hij.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "hijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijk.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ijklmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jkl.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklm.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "jklmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmn.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "klmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "lmnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mno.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "mnopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nop.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "nopqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opq.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "opqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqr.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrs.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "pqrst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "qrstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rst.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "rstuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stu.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "stuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuv.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "tuvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvw.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "uvwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwx.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "vwxyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxy.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "wxyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "xyz.v").toString(),
        Paths.get("faraday", "rtl", "RISC", "hdl", "ghij.v").toString());
  }*/

  /*
   * iscas
   */

  @Test
  public void runTest_iscas_s27() {
    runTest("s27_bench", Paths.get("iscas", "rtl", "s27.v").toString());
  }

  @Test
  public void runTest_iscas_s208() {
    runTest("s208_1_bench", Paths.get("iscas", "rtl", "s208_1.v").toString());
  }

  @Test
  public void runTest_iscas_s298() {
    runTest("s298_bench", Paths.get("iscas", "rtl", "s298.v").toString());
  }

  @Test
  public void runTest_iscas_s344() {
    runTest("s344_bench", Paths.get("iscas", "rtl", "s344.v").toString());
  }

  @Test
  public void runTest_iscas_s349() {
    runTest("s349_bench", Paths.get("iscas", "rtl", "s349.v").toString());
  }

  @Test
  public void runTest_iscas_s382() {
    runTest("s382_bench", Paths.get("iscas", "rtl", "s382.v").toString());
  }

  @Test
  public void runTest_iscas_s386() {
    runTest("s386_bench", Paths.get("iscas", "rtl", "s386.v").toString());
  }

  @Test
  public void runTest_iscas_s400() {
    runTest("s400_bench", Paths.get("iscas", "rtl", "s400.v").toString());
  }

  @Test
  public void runTest_iscas_s420_1() {
    runTest("s420_1_bench", Paths.get("iscas", "rtl", "s420_1.v").toString());
  }

  @Test
  public void runTest_iscas_s444() {
    runTest("s444_bench", Paths.get("iscas", "rtl", "s444.v").toString());
  }

  @Test
  public void runTest_iscas_s510() {
    runTest("s510_bench", Paths.get("iscas", "rtl", "s510.v").toString());
  }

  @Test
  public void runTest_iscas_s526() {
    runTest("s526_bench", Paths.get("iscas", "rtl", "s526.v").toString());
  }

  @Test
  public void runTest_iscas_s526n() {
    runTest("s526n_bench", Paths.get("iscas", "rtl", "s526n.v").toString());
  }

  @Test
  public void runTest_iscas_s641() {
    runTest("s641_bench", Paths.get("iscas", "rtl", "s641.v").toString());
  }

  @Test
  public void runTest_iscas_s713() {
    runTest("s713_bench", Paths.get("iscas", "rtl", "s713.v").toString());
  }

  @Test
  public void runTest_iscas_s820() {
    runTest("s820_bench", Paths.get("iscas", "rtl", "s820.v").toString());
  }

  @Test
  public void runTest_iscas_s832() {
    runTest("s832_bench", Paths.get("iscas", "rtl", "s832.v").toString());
  }

  @Test
  public void runTest_iscas_s838() {
    runTest("s838_1_bench", Paths.get("iscas", "rtl", "s838_1.v").toString());
  }

  @Test
  public void runTest_iscas_s953() {
    runTest("s953_bench", Paths.get("iscas", "rtl", "s953.v").toString());
  }

  @Test
  public void runTest_iscas_s1196() {
    runTest("s1196_bench", Paths.get("iscas", "rtl", "s1196.v").toString());
  }

  @Test
  public void runTest_iscas_s1238() {
    runTest("s1238_bench", Paths.get("iscas", "rtl", "s1238.v").toString());
  }

  @Test
  public void runTest_iscas_s1423() {
    runTest("s1423_bench", Paths.get("iscas", "rtl", "s1423.v").toString());
  }

  @Test
  public void runTest_iscas_s1488() {
    runTest("s1488_bench", Paths.get("iscas", "rtl", "s1488.v").toString());
  }

  @Test
  public void runTest_iscas_s1494() {
    runTest("s1494_bench", Paths.get("iscas", "rtl", "s1494.v").toString());
  }

  @Test
  public void runTest_iscas_s5378() {
    runTest("s5378_bench", Paths.get("iscas", "rtl", "s5378.v").toString());
  }

  @Test
  public void runTest_iscas_s9234_1() {
    runTest("s9234_1_bench", Paths.get("iscas", "rtl", "s9234_1.v").toString());
  }

  @Test
  public void runTest_iscas_s13207() {
    runTest("s13207_bench", Paths.get("iscas", "rtl", "s13207.v").toString());
  }

  @Test
  public void runTest_iscas_s15850() {
    runTest("s15850_bench", Paths.get("iscas", "rtl", "s15850.v").toString());
  }

  // TODO: java.lang.OutOfMemoryError: GC overhead limit exceeded
  /*@Test
  public void runTest_iscas_s35932() {
    runTest("s35932_bench", Paths.get("iscas", "rtl", "s35932.v").toString());
  }

  @Test
  public void runTest_iscas_s38417() {
    runTest("s38417_bench", Paths.get("iscas", "rtl", "s38417.v").toString());
  }

  @Test
  public void runTest_iscas_s38584() {
    runTest("s38584_bench", Paths.get("iscas", "rtl", "s38584.v").toString());
  }*/

  /*
   * opencores
   */

  /* ac97_ctrl */

  @Test
  public void runTest_opencores_ac97_ctrl_cra() {
    runTest("ac97_cra",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_cra.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_dma_if() {
    runTest("ac97_dma_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_dma_req.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_dma_if.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_dma_req() {
    runTest("ac97_dma_req",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_dma_req.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_fifo_ctrl() {
    runTest("ac97_fifo_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_fifo_ctrl.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_in_fifo() {
    runTest("ac97_in_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_in_fifo.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_int() {
    runTest("ac97_int",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_int.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_out_fifo() {
    runTest("ac97_out_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_out_fifo.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_prc() {
    runTest("ac97_prc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_prc.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_fifo_ctrl.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_rf() {
    runTest("ac97_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_rf.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_rst() {
    runTest("ac97_rst",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_rst.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_sin() {
    runTest("ac97_sin",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_sin.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_soc() {
    runTest("ac97_soc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_soc.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_sout() {
    runTest("ac97_sout",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_sout.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_top() {
    runTest("ac97_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_top.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_fifo_ctrl.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_int.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_rst.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_dma_if.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_dma_req.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_cra.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_prc.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_rf.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_in_fifo.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_out_fifo.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_soc.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_sout.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_wb_if.v").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_sin.v").toString());
  }

  @Test
  public void runTest_opencores_ac97_ctrl_wb_if() {
    runTest("ac97_wb_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ac97_ctrl").toString(),
        Paths.get("opencores", "rtl", "ac97_ctrl", "ac97_wb_if.v").toString());
  }

  /* aes_core */

  @Test
  public void runTest_opencores_aes_cipher_top() {
    runTest("aes_cipher_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_cipher_top.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_key_expand_128.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_rcon.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_sbox.v").toString());
  }

  @Test
  public void runTest_opencores_aes_inv_cipher_top() {
    runTest("aes_inv_cipher_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_inv_cipher_top.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_inv_sbox.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_key_expand_128.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_rcon.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_sbox.v").toString());
  }

  @Test
  public void runTest_opencores_aes_inv_sbox() {
    runTest("aes_inv_sbox",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_inv_sbox.v").toString());
  }

  @Test
  public void runTest_opencores_aes_key_expand() {
    runTest("aes_key_expand_128",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_key_expand_128.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_rcon.v").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_sbox.v").toString());
  }

  @Test
  public void runTest_opencores_aes_rcon() {
    runTest("aes_rcon",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_rcon.v").toString());
  }

  @Test
  public void runTest_opencores_aes_sbox() {
    runTest("aes_sbox",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "aes_core").toString(),
        Paths.get("opencores", "rtl", "aes_core", "aes_sbox.v").toString());
  }

  /* des */

  @Test
  public void runTest_opencores_des_opt_des() {
    runTest("des",
        Paths.get("opencores", "rtl", "des", "area_opt", "des.v").toString(),
        Paths.get("opencores", "rtl", "des", "area_opt", "key_sel.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "crp.v").toString());
  }

  @Test
  public void runTest_opencores_des_opt_des3() {
    runTest("des3",
        Paths.get("opencores", "rtl", "des", "area_opt", "des3.v").toString(),
        Paths.get("opencores", "rtl", "des", "area_opt", "key_sel3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "crp.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString());
  }

  @Test
  public void runTest_opencores_des_opt_key_sel() {
    runTest("key_sel", Paths.get("opencores", "rtl", "des", "area_opt", "key_sel.v").toString());
  }

  @Test
  public void runTest_opencores_des_opt_key_sel3() {
    runTest("key_sel3", Paths.get("opencores", "rtl", "des", "area_opt", "key_sel3.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_crp() {
    runTest("crp",
        Paths.get("opencores", "rtl", "des", "common", "crp.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox1() {
    runTest("sbox1", Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox2() {
    runTest("sbox2", Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox3() {
    runTest("sbox3", Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox4() {
    runTest("sbox4", Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox5() {
    runTest("sbox5", Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox6() {
    runTest("sbox6", Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox7() {
    runTest("sbox7", Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString());
  }

  @Test
  public void runTest_opencores_des_common_sbox8() {
    runTest("sbox8", Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString());
  }

  @Test
  public void runTest_opencores_des_perf_des() {
    runTest("des",
        Paths.get("opencores", "rtl", "des", "perf_opt", "des.v").toString(),
        Paths.get("opencores", "rtl", "des", "perf_opt", "crp.v").toString(),
        Paths.get("opencores", "rtl", "des", "perf_opt", "key_sel.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString());
  }

  @Test
  public void runTest_opencores_des_perf_des3() {
    runTest("des3",
        Paths.get("opencores", "rtl", "des", "perf_opt", "des3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "crp.v").toString(),
        Paths.get("opencores", "rtl", "des", "perf_opt", "des.v").toString(),
        Paths.get("opencores", "rtl", "des", "perf_opt", "key_sel.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox8.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox7.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox6.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox5.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox4.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox3.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox2.v").toString(),
        Paths.get("opencores", "rtl", "des", "common", "sbox1.v").toString());
  }

  @Test
  public void runTest_opencores_des_opt_perf_sel() {
    runTest("key_sel", Paths.get("opencores", "rtl", "des", "perf_opt", "key_sel.v").toString());
  }

  /* ethernet */

  @Test
  public void runTest_opencores_ethernet_clockgen() {
    runTest("eth_clockgen",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_clockgen.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_cop() {
    runTest("eth_cop",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_cop.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_crc() {
    runTest("eth_crc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_crc.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_fifo() {
    runTest("eth_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_fifo.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_maccontrol() {
    runTest("eth_maccontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_receivecontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_maccontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_transmitcontrol.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_macstatus() {
    runTest("eth_macstatus",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_macstatus.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_miim() {
    runTest("eth_miim",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_miim.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_clockgen.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_outputcontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_shiftreg.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_outputcontrol() {
    runTest("eth_outputcontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_outputcontrol.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_random() {
    runTest("eth_random",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_random.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_receivecontrol() {
    runTest("eth_receivecontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_receivecontrol.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_register() {
    runTest("eth_register",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_register.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_registers() {
    runTest("eth_registers",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_registers.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_register.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_rxaddrcheck() {
    runTest("eth_rxaddrcheck",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxaddrcheck.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_rxcounters() {
    runTest("eth_rxcounters",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxcounters.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_rxethmac() {
    runTest("eth_rxethmac",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxaddrcheck.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_crc.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxethmac.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxcounters.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxstatem.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_rxstatem() {
    runTest("eth_rxstatem",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxstatem.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_shiftreg() {
    runTest("eth_shiftreg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_shiftreg.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_spram_256x32() {
    runTest("eth_spram_256x32",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_spram_256x32.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_top() {
    runTest("eth_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_top.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_clockgen.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_crc.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_fifo.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_maccontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_macstatus.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_miim.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_outputcontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_random.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_receivecontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_register.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_registers.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxaddrcheck.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxcounters.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxethmac.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_rxstatem.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_shiftreg.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_spram_256x32.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_transmitcontrol.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txcounters.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txstatem.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txethmac.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_wishbone.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_transmitcontrol() {
    runTest("eth_transmitcontrol",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_transmitcontrol.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_txcounters() {
    runTest("eth_txcounters",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txcounters.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_txethmac() {
    runTest("eth_txethmac",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txethmac.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_crc.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_random.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txcounters.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txstatem.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_txstatem() {
    runTest("eth_txstatem",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_txstatem.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_wishbone() {
    runTest("eth_wishbone",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_wishbone.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_fifo.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_spram_256x32.v").toString());
  }

  @Test
  public void runTest_opencores_ethernet_xilinx_dist_ram_16x32() {
    runTest("xilinx_dist_ram_16x32",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString(),
        Paths.get("opencores", "rtl", "ethernet", "xilinx_dist_ram_16x32.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_top.v").toString());
  }

  /* fpu */

  @Test
  public void runTest_opencores_fpu_except() {
    runTest("except", Paths.get("opencores", "rtl", "fpu", "except.v").toString());
  }

  @Test
  public void runTest_opencores_fpu_fpu() {
    runTest("fpu",
        Paths.get("opencores", "rtl", "fpu", "fpu.v").toString(),
        Paths.get("opencores", "rtl", "fpu", "except.v").toString(),
        Paths.get("opencores", "rtl", "fpu", "pre_norm.v").toString(),
        Paths.get("opencores", "rtl", "fpu", "pre_norm_fmul.v").toString(),
        Paths.get("opencores", "rtl", "fpu", "primitives.v").toString(),
        Paths.get("opencores", "rtl", "fpu", "post_norm.v").toString());
  }

  @Test
  public void runTest_opencores_fpu_post_norm() {
    runTest("post_norm", Paths.get("opencores", "rtl", "fpu", "post_norm.v").toString());
  }

  @Test
  public void runTest_opencores_fpu_pre_norm() {
    runTest("pre_norm", Paths.get("opencores", "rtl", "fpu", "pre_norm.v").toString());
  }

  @Test
  public void runTest_opencores_fpu_pre_norm_fmul() {
    runTest("pre_norm_fmul", Paths.get("opencores", "rtl", "fpu", "pre_norm_fmul.v").toString());
  }

  /* i2c */

  @Test
  public void runTest_opencores_i2c_bit_ctrl() {
    runTest("i2c_master_bit_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "i2c").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_bit_ctrl.v").toString());
  }

  @Test
  public void runTest_opencores_i2c_byte_ctrl() {
    runTest("i2c_master_byte_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "i2c").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_byte_ctrl.v").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_bit_ctrl.v").toString());
  }

  @Test
  public void runTest_opencores_i2c_master_top() {
    runTest("i2c_master_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "i2c").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_top.v").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_bit_ctrl.v").toString(),
        Paths.get("opencores", "rtl", "i2c", "i2c_master_byte_ctrl.v").toString());
  }

  /* mem_ctrl */

  @Test
  public void runTest_opencores_mc_adr_sel() {
    runTest("mc_adr_sel",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_adr_sel.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_incn_r.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'mc_cs_rf.this_cs' has not been found
  /*@Test
  public void runTest_opencores_mc_cs_rf() {
    runTest("mc_cs_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_cs_rf.v").toString());
  }*/

  @Test
  public void runTest_opencores_mc_dp() {
    runTest("mc_dp",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_dp.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_rd_fifo.v").toString());
  }

  @Test
  public void runTest_opencores_mc_incn_r() {
    runTest("mc_incn_r",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_incn_r.v").toString());
  }

  @Test
  public void runTest_opencores_mc_mem_if() {
    runTest("mc_mem_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_mem_if.v").toString());
  }

  @Test
  public void runTest_opencores_mc_obct() {
    runTest("mc_obct",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_obct.v").toString());
  }

  @Test
  public void runTest_opencores_mc_obct_top() {
    runTest("mc_obct_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_obct_top.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_obct.v").toString());
  }

  @Test
  public void runTest_opencores_mc_rd_fifo() {
    runTest("mc_rd_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_rd_fifo.v").toString());
  }

  @Test
  public void runTest_opencores_mc_refresh() {
    runTest("mc_refresh",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_refresh.v").toString());
  }

  // TODO: [Internal] Descriptor for 'mc_rf.u0.this_cs' has not been found
  /*@Test
  public void runTest_opencores_mc_rf() {
    runTest("mc_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_rf.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_cs_rf.v").toString());
  }*/

  @Test
  public void runTest_opencores_mc_timing() {
    runTest("mc_timing",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_timing.v").toString());
  }

  // TODO: Descriptor for 'mc_top.u0.u0.this_cs' has not been found
  /*@Test
  public void runTest_opencores_mc_top() {
    runTest("mc_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_top.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_adr_sel.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_cs_rf.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_dp.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_incn_r.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_mem_if.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_obct.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_obct_top.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_rd_fifo.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_refresh.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_rf.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_timing.v").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_wb_if.v").toString());
  }*/

  @Test
  public void runTest_opencores_mc_wb_if() {
    runTest("mc_wb_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "mem_ctrl").toString(),
        Paths.get("opencores", "rtl", "mem_ctrl", "mc_wb_if.v").toString());
  }

  /* pci */

  @Test
  public void runTest_opencores_pci_async_reset_flop() {
    runTest("pci_async_reset_flop",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString());
  }

  // TODO: ERROR: [Internal] 11 must be within range [0, 1)
  /*@Test
  public void runTest_opencores_pci_bridge32() {
    runTest("pci_bridge32",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_bridge32.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_constants.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_conf_space.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cur_out_reg.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_sync.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_write_reg.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_in_reg.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ch_state_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm_if.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_out_reg.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_par_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_parity_check.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_decoder.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_tpram.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pciw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pciw_pcir_fifos.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_rst_int.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_sync_module.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target_unit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_clk_en.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_devs_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_interface.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_trdy_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_decoder.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_master.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_slave.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_slave_unit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_tpram.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbs_wbb3_2_wbb2.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_wbr_fifos.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "timescale.v").toString());
  }*/

  @Test
  public void runTest_opencores_pci_cbe_en_crit() {
    runTest("pci_cbe_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cbe_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_conf_cyc_addr_dec() {
    runTest("pci_conf_cyc_addr_dec",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_conf_cyc_addr_dec.v").toString());
  }

  @Test
  public void runTest_opencores_pci_conf_space() {
    runTest("pci_conf_space",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_conf_space.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_sync_module.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_constants.v").toString());
  }

  @Test
  public void runTest_opencores_pci_cur_out_reg() {
    runTest("pci_cur_out_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cur_out_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_delayed_sync() {
    runTest("pci_delayed_sync",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_sync.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_delayed_write_reg() {
    runTest("pci_delayed_write_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_write_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_frame_crit() {
    runTest("pci_frame_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_frame_en_crit() {
    runTest("pci_frame_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_frame_load_crit() {
    runTest("pci_frame_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_load_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_in_reg() {
    runTest("pci_in_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_in_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_io_mux() {
    runTest("pci_io_mux",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_io_mux_ad_en_crit() {
    runTest("pci_io_mux_ad_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_io_mux_ad_load_crit() {
    runTest("pci_io_mux_ad_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_io_mux_ad_load_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_irdy_out_crit() {
    runTest("pci_irdy_out_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_irdy_out_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_mas_ad_en_crit() {
    runTest("pci_mas_ad_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_mas_ad_load_crit() {
    runTest("pci_mas_ad_load_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_load_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_mas_ch_state_crit() {
    runTest("pci_mas_ch_state_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ch_state_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_master32_sm() {
    runTest("pci_master32_sm",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ch_state_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_master32_sm_if() {
    runTest("pci_master32_sm_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm_if.v").toString());
  }

  @Test
  public void runTest_opencores_pci_out_reg() {
    runTest("pci_out_reg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_par_crit() {
    runTest("pci_par_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_par_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_parity_check() {
    runTest("pci_parity_check",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_parity_check.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_par_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_pci_decoder() {
    runTest("pci_pci_decoder",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_decoder.v").toString());
  }

  @Test
  public void runTest_opencores_pci_pci_tpram() {
    runTest("pci_pci_tpram",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_tpram.v").toString());
  }

  @Test
  public void runTest_opencores_pci_pcir_fifo_control() {
    runTest("pci_pcir_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_perr_crit() {
    runTest("pci_perr_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_perr_en_crit() {
    runTest("pci_perr_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_perr_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_ram_16x40d() {
    runTest("pci_ram_16x40d",
        "--include-path",
        String.format("%s%s%s",
            Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
            File.pathSeparator,
            Paths.get(getTestDirPath(), "opencores", "rtl", "ethernet").toString()
        ),
        Paths.get("opencores", "rtl", "pci", "pci_ram_16x40d.v").toString(),
        Paths.get("opencores", "rtl", "ethernet", "eth_top.v").toString());
  }

  @Test
  public void runTest_opencores_pci_rst_int() {
    runTest("pci_rst_int",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_rst_int.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_out_reg.v").toString());
  }

  @Test
  public void runTest_opencores_pci_serr_crit() {
    runTest("pci_serr_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_serr_en_crit() {
    runTest("pci_serr_en_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_serr_en_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_spoci_ctrl() {
    runTest("pci_spoci_ctrl",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_spoci_ctrl.v").toString());
  }

  @Test
  public void runTest_opencores_pci_sync_module() {
    runTest("pci_sync_module",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_sync_module.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_synchronizer_flop() {
    runTest("pci_synchronizer_flop",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_target32_clk_en() {
    runTest("pci_target32_clk_en",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_clk_en.v").toString());
  }

  @Test
  public void runTest_opencores_pci_target32_devs_crit() {
    runTest("pci_target32_devs_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_devs_crit.v").toString());
  }

  // TODO: ERROR: [Internal] 11 must be within range [0, 1)
  /*@Test
  public void runTest_opencores_pci_target32_interface() {
    runTest("pci_target32_interface",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_interface.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_decoder.v").toString());
  }*/

  @Test
  public void runTest_opencores_pci_target32_sm() {
    runTest("pci_target32_sm",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_clk_en.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_devs_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_trdy_crit.v").toString());
  }

  @Test
  public void runTest_opencores_pci_target32_stop_crit() {
    runTest("pci_target32_stop_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_clk_en.v").toString());
  }

  @Test
  public void runTest_opencores_pci_target32_trdy_crit() {
    runTest("pci_target32_trdy_crit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_trdy_crit.v").toString());
  }

  // TODO: ERROR: [Internal] 11 must be within range [0, 1)
  /*@Test
  public void runTest_opencores_pci_target_unit() {
    runTest("pci_target_unit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target_unit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_constants.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_clk_en.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_interface.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_devs_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_stop_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_target32_trdy_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_master.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_sync.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pcir_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pciw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pciw_pcir_fifos.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_decoder.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_pci_tpram.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "timescale.v").toString());
  }*/

  @Test
  public void runTest_opencores_pci_wb_addr_mux() {
    runTest("pci_wb_addr_mux",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_decoder.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wb_decoder() {
    runTest("pci_wb_decoder",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_decoder.v").toString());
  }

  // TODO: java.lang.IllegalArgumentException
  @Test
  public void runTest_opencores_pci_wb_master() {
    runTest("pci_wb_master",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_master.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wb_slave() {
    runTest("pci_wb_slave",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_slave.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wb_slave_unit() {
    runTest("pci_wb_slave_unit",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_slave_unit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_sync.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_async_reset_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_cbe_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_delayed_write_reg.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_frame_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_irdy_out_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_en_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ad_load_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_mas_ch_state_crit.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_master32_sm_if.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_addr_mux.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_decoder.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_slave.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_tpram.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_wbr_fifos.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wb_tpram() {
    runTest("pci_wb_tpram",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_tpram.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wbr_fifo_control() {
    runTest("pci_wbr_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbr_fifo_control.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wbs_wbb3_2_wbb2() {
    runTest("pci_wbs_wbb3_2_wbb2",
        Paths.get("opencores", "rtl", "pci", "pci_wbs_wbb3_2_wbb2.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wbw_fifo_control() {
    runTest("pci_wbw_fifo_control",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString());
  }

  @Test
  public void runTest_opencores_pci_wbw_wbr_fifos() {
    runTest("pci_wbw_wbr_fifos",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "pci").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_wbr_fifos.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_synchronizer_flop.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbr_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wbw_fifo_control.v").toString(),
        Paths.get("opencores", "rtl", "pci", "pci_wb_tpram.v").toString());
  }

  /* sasc */

  @Test
  public void runTest_opencores_sasc_brg() {
    runTest("sasc_brg",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "sasc").toString(),
        Paths.get("opencores", "rtl", "sasc", "sasc_brg.v").toString());
  }

  @Test
  public void runTest_opencores_sasc_fifo4() {
    runTest("sasc_fifo4",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "sasc").toString(),
        Paths.get("opencores", "rtl", "sasc", "sasc_fifo4.v").toString());
  }

  @Test
  public void runTest_opencores_sasc_top() {
    runTest("sasc_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "sasc").toString(),
        Paths.get("opencores", "rtl", "sasc", "sasc_top.v").toString(),
        Paths.get("opencores", "rtl", "sasc", "sasc_fifo4.v").toString());
  }

  /* simple_spi */

  @Test
  public void runTest_opencores_simple_spi_fifo4() {
    runTest("fifo4",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "spi").toString(),
        Paths.get("opencores", "rtl", "simple_spi", "fifo4.v").toString());
  }

  @Test
  public void runTest_opencores_simple_spi_top() {
    runTest("simple_spi_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "spi").toString(),
        Paths.get("opencores", "rtl", "simple_spi", "fifo4.v").toString(),
        Paths.get("opencores", "rtl", "simple_spi", "simple_spi_top.v").toString());
  }

  /* spi */

  @Test
  public void runTest_opencores_spi_clgen() {
    runTest("spi_clgen",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "spi").toString(),
        Paths.get("opencores", "rtl", "spi", "spi_clgen.v").toString());
  }

  @Test
  public void runTest_opencores_spi_shift() {
    runTest("spi_shift",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "spi").toString(),
        Paths.get("opencores", "rtl", "spi", "spi_shift.v").toString());
  }

  @Test
  public void runTest_opencores_spi_top() {
    runTest("spi_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "spi").toString(),
        Paths.get("opencores", "rtl", "spi", "spi_clgen.v").toString(),
        Paths.get("opencores", "rtl", "spi", "spi_shift.v").toString(),
        Paths.get("opencores", "rtl", "spi", "spi_top.v").toString());
  }

  /* ss_pcm */

  @Test
  public void runTest_opencores_ss_pcm_slv_top() {
    runTest("pcm_slv_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "ss_pcm").toString(),
        Paths.get("opencores", "rtl", "ss_pcm", "pcm_slv_top.v").toString());
  }

  /* systemcaes */

  // TODO: subbytes.v line 76:13 no viable alternative at input '['
  /*@Test
  public void runTest_systemcaes_aes() {
    runTest("aes",
        Paths.get("opencores", "rtl", "systemcaes", "aes.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "byte_mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "keysched.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "word_mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "sbox.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "subbytes.v").toString());
  }*/

  @Test
  public void runTest_systemcaes_byte_mixcolum() {
    runTest("byte_mixcolum",
        Paths.get("opencores", "rtl", "systemcaes", "byte_mixcolum.v").toString());
  }

  @Test
  public void runTest_systemcaes_keysched() {
    runTest("keysched", Paths.get("opencores", "rtl", "systemcaes", "keysched.v").toString());
  }

  @Test
  public void runTest_systemcaes_mixcolum() {
    runTest("mixcolum",
        Paths.get("opencores", "rtl", "systemcaes", "mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "byte_mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "word_mixcolum.v").toString());
  }

  @Test
  public void runTest_systemcaes_sbox() {
    runTest("sbox", Paths.get("opencores", "rtl", "systemcaes", "sbox.v").toString());
  }

  // TODO: subbytes.v line 76:13 no viable alternative at input '['
  /*@Test
  public void runTest_systemcaes_subbytes() {
    runTest("subbytes", Paths.get("opencores", "rtl", "systemcaes", "subbytes.v").toString());
  }*/

  @Test
  public void runTest_systemcaes_word_mixcolum() {
    runTest("word_mixcolum",
        Paths.get("opencores", "rtl", "systemcaes", "word_mixcolum.v").toString(),
        Paths.get("opencores", "rtl", "systemcaes", "byte_mixcolum.v").toString());
  }

  /* systemcdes */

  @Test
  public void runTest_systemcdes_des() {
    runTest("des",
        Paths.get("opencores", "rtl", "systemcdes", "des.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "desround.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "key_gen.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s1.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s2.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s3.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s4.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s5.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s6.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s7.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "s8.v").toString());
  }

  @Test
  public void runTest_systemcdes_desround() {
    runTest("desround",
        Paths.get("opencores", "rtl", "systemcdes", "desround.v").toString(),
        Paths.get("opencores", "rtl", "systemcdes", "key_gen.v").toString());
  }

  @Test
  public void runTest_systemcdes_key_gen() {
    runTest("key_gen", Paths.get("opencores", "rtl", "systemcdes", "key_gen.v").toString());
  }

  @Test
  public void runTest_systemcdes_s1() {
    runTest("s1", Paths.get("opencores", "rtl", "systemcdes", "s1.v").toString());
  }

  @Test
  public void runTest_systemcdes_s2() {
    runTest("s2", Paths.get("opencores", "rtl", "systemcdes", "s2.v").toString());
  }

  @Test
  public void runTest_systemcdes_s3() {
    runTest("s3", Paths.get("opencores", "rtl", "systemcdes", "s3.v").toString());
  }

  @Test
  public void runTest_systemcdes_s4() {
    runTest("s4", Paths.get("opencores", "rtl", "systemcdes", "s4.v").toString());
  }

  @Test
  public void runTest_systemcdes_s5() {
    runTest("s5", Paths.get("opencores", "rtl", "systemcdes", "s5.v").toString());
  }

  @Test
  public void runTest_systemcdes_s6() {
    runTest("s6", Paths.get("opencores", "rtl", "systemcdes", "s6.v").toString());
  }

  @Test
  public void runTest_systemcdes_s7() {
    runTest("s7", Paths.get("opencores", "rtl", "systemcdes", "s7.v").toString());
  }

  @Test
  public void runTest_systemcdes_s8() {
    runTest("s8", Paths.get("opencores", "rtl", "systemcdes", "s8.v").toString());
  }

  /* tv80 */

  @Test
  public void runTest_tv80_alu() {
    runTest("tv80_alu", Paths.get("opencores", "rtl", "tv80", "tv80_alu.v").toString());
  }

  @Test
  public void runTest_tv80_core() {
    runTest("tv80_core",
        Paths.get("opencores", "rtl", "tv80", "tv80_core.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_alu.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_mcode.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_reg.v").toString());
  }

  @Test
  public void runTest_tv80_mcode() {
    runTest("tv80_mcode", Paths.get("opencores", "rtl", "tv80", "tv80_mcode.v").toString());
  }

  @Test
  public void runTest_tv80_reg() {
    runTest("tv80_reg", Paths.get("opencores", "rtl", "tv80", "tv80_reg.v").toString());
  }

  @Test
  public void runTest_tv80s() {
    runTest("tv80s",
        Paths.get("opencores", "rtl", "tv80", "tv80s.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_alu.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_core.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_mcode.v").toString(),
        Paths.get("opencores", "rtl", "tv80", "tv80_reg.v").toString());
  }

  /* usb_funct */

  @Test
  public void runTest_usbf_crc5() {
    runTest("usbf_crc5",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc5.v").toString());
  }

  @Test
  public void runTest_usbf_crc16() {
    runTest("usbf_crc16",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc16.v").toString());
  }

  @Test
  public void runTest_usbf_ep_rf() {
    runTest("usbf_ep_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_ep_rf.v").toString());
  }

  @Test
  public void runTest_usbf_ep_rf_dummy() {
    runTest("usbf_ep_rf_dummy",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_ep_rf_dummy.v").toString());
  }

  @Test
  public void runTest_usbf_idma() {
    runTest("usbf_idma",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_idma.v").toString());
  }

  @Test
  public void runTest_usbf_mem_arb() {
    runTest("usbf_mem_arb",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_mem_arb.v").toString());
  }

  @Test
  public void runTest_usbf_pa() {
    runTest("usbf_pa",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pa.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc16.v").toString());
  }

  @Test
  public void runTest_usbf_pd() {
    runTest("usbf_pd",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pd.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc16.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc5.v").toString());
  }

  @Test
  public void runTest_usbf_pe() {
    runTest("usbf_pe",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pe.v").toString());
  }

  @Test
  public void runTest_usbf_pl() {
    runTest("usbf_pl",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pl.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc5.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_crc16.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_idma.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pa.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pd.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_pe.v").toString());
  }

  @Test
  public void runTest_usbf_rf() {
    runTest("usbf_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_rf.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_ep_rf_dummy.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_ep_rf.v").toString());
  }

  // TODO: ERROR: [Internal] 0 must be > 0
  /*@Test
  public void runTest_usbf_top() {
    runTest("usbf_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_top.v").toString());
  }*/

  @Test
  public void runTest_usbf_utmi_if() {
    runTest("usbf_utmi_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_utmi_if.v").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_utmi_ls.v").toString());
  }

  @Test
  public void runTest_usbf_utmi_ls() {
    runTest("usbf_utmi_ls",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_utmi_ls.v").toString());
  }

  @Test
  public void runTest_usbf_wb() {
    runTest("usbf_wb",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_funct").toString(),
        Paths.get("opencores", "rtl", "usb_funct", "usbf_wb.v").toString());
  }

  /* usb_phy */

  @Test
  public void runTest_usb_phy() {
    runTest("usb_phy",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_phy").toString(),
        Paths.get("opencores", "rtl", "usb_phy", "usb_phy.v").toString(),
        Paths.get("opencores", "rtl", "usb_phy", "usb_rx_phy.v").toString(),
        Paths.get("opencores", "rtl", "usb_phy", "usb_tx_phy.v").toString());
  }

  @Test
  public void runTest_usb_rx_phy() {
    runTest("usb_rx_phy",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_phy").toString(),
        Paths.get("opencores", "rtl", "usb_phy", "usb_rx_phy.v").toString());
  }

  @Test
  public void runTest_usb_tx_phy() {
    runTest("usb_tx_phy",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "usb_phy").toString(),
        Paths.get("opencores", "rtl", "usb_phy", "usb_tx_phy.v").toString());
  }

  /* vga_lcd */

  @Test
  public void runTest_vga_generic_dpram() {
    runTest("generic_dpram",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_dpram.v").toString());
  }

  @Test
  public void runTest_vga_generic_spram() {
    runTest("generic_spram",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_spram.v").toString());
  }

  @Test
  public void runTest_vga_clkgen() {
    runTest("vga_clkgen",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_clkgen.v").toString());
  }

  @Test
  public void runTest_vga_colproc() {
    runTest("vga_colproc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_colproc.v").toString());
  }

  @Test
  public void runTest_vga_csm_pb() {
    runTest("vga_csm_pb",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_csm_pb.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_spram.v").toString());
  }

  @Test
  public void runTest_vga_cur_cregs() {
    runTest("vga_cur_cregs",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_cur_cregs.v").toString());
  }

  @Test
  public void runTest_vga_curproc() {
    runTest("vga_curproc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_spram.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_curproc.v").toString());
  }

  @Test
  public void runTest_vga_enh_top() {
    runTest("vga_enh_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_enh_top.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_dpram.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_spram.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_clkgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_colproc.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_csm_pb.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo_dc.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_pgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_tgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_vtim.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_wb_master.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_wb_slave.v").toString());
  }

  @Test
  public void runTest_vga_fifo() {
    runTest("vga_fifo",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo.v").toString());
  }

  @Test
  public void runTest_vga_fifo_dc() {
    runTest("vga_fifo_dc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo_dc.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "generic_dpram.v").toString());
  }

  @Test
  public void runTest_vga_pgen() {
    runTest("vga_pgen",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_pgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_clkgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_colproc.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_tgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_vtim.v").toString());
  }

  @Test
  public void runTest_vga_tgen() {
    runTest("vga_tgen",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_tgen.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_vtim.v").toString());
  }

  @Test
  public void runTest_vga_vtim() {
    runTest("vga_vtim",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_vtim.v").toString());
  }

  @Test
  public void runTest_vga_wb_master() {
    runTest("vga_wb_master",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_wb_master.v").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_fifo.v").toString());
  }

  @Test
  public void runTest_vga_wb_slave() {
    runTest("vga_wb_slave",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "vga_lcd").toString(),
        Paths.get("opencores", "rtl", "vga_lcd", "vga_wb_slave.v").toString());
  }

  /* wb_conmax */

  @Test
  public void runTest_wb_conmax_arb() {
    runTest("wb_conmax_arb",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_arb.v").toString());
  }

  @Test
  public void runTest_wb_conmax_master_if() {
    runTest("wb_conmax_master_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_master_if.v").toString());
  }

  @Test
  public void runTest_wb_conmax_msel() {
    runTest("wb_conmax_msel",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_msel.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_arb.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_dec.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_enc.v").toString());
  }

  @Test
  public void runTest_wb_conmax_pri_dec() {
    runTest("wb_conmax_pri_dec",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_dec.v").toString());
  }

  @Test
  public void runTest_wb_conmax_pri_enc() {
    runTest("wb_conmax_pri_enc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_enc.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_dec.v").toString());
  }

  @Test
  public void runTest_wb_conmax_rf() {
    runTest("wb_conmax_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_rf.v").toString());
  }

  @Test
  public void runTest_wb_conmax_slave_if() {
    runTest("wb_conmax_slave_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_slave_if.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_arb.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_msel.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_dec.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_enc.v").toString());
  }

  @Test
  public void runTest_wb_conmax_top() {
    runTest("wb_conmax_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_conmax").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_top.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_arb.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_master_if.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_msel.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_dec.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_pri_enc.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_rf.v").toString(),
        Paths.get("opencores", "rtl", "wb_conmax", "wb_conmax_slave_if.v").toString());
  }

  /* wb_dma */

  @Test
  public void runTest_wb_dma_ch_arb() {
    runTest("wb_dma_ch_arb",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_arb.v").toString());
  }

  // TODO: descriptor has not been found
  /*@Test
  public void runTest_wb_dma_ch_pri_enc() {
    runTest("wb_dma_ch_pri_enc",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }*/

  @Test
  public void runTest_wb_dma_ch_rf() {
    runTest("wb_dma_ch_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_rf.v").toString());
  }

  // TODO: Descriptor for 'wb_dma_ch_sel.ch0_conf' has not been found
  /*@Test
  public void runTest_wb_dma_ch_sel() {
    runTest("wb_dma_ch_sel",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_sel.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_arb.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }*/

  @Test
  public void runTest_wb_dma_de() {
    runTest("wb_dma_de",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_de.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_inc30r.v").toString());
  }

  @Test
  public void runTest_wb_dma_inc30r() {
    runTest("wb_dma_inc30r",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_inc30r.v").toString());
  }

  // TODO: ERROR: [Internal] Descriptor for 'wb_dma_pri_enc_sub.ch_conf' has not been found
  /*@Test
  public void runTest_wb_dma_pri_enc_sub() {
    runTest("wb_dma_pri_enc_sub",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_pri_enc_sub.v").toString());
  }*/

  // TODO: Descriptor for 'wb_dma_rf.ch1_conf' has not been found
  /*@Test
  public void runTest_wb_dma_rf() {
    runTest("wb_dma_rf",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_rf.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_rf.v").toString());
  }*/

  // TODO: Descriptor for 'wb_dma_top.u0.ch1_conf' has not been found
  /*@Test
  public void runTest_wb_dma_top() {
    runTest("wb_dma_top",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_top.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_arb.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_pri_enc.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_rf.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_ch_sel.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_de.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_inc30r.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_pri_enc_sub.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_rf.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_if.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_mast.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_slv.v").toString());
  }*/

  @Test
  public void runTest_wb_dma_wb_if() {
    runTest("wb_dma_wb_if",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_if.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_mast.v").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_slv.v").toString());
  }

  @Test
  public void runTest_wb_dma_wb_mast() {
    runTest("wb_dma_wb_mast",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_mast.v").toString());
  }

  @Test
  public void runTest_wb_dma_wb_slv() {
    runTest("wb_dma_wb_slv",
        "--include-path",
        Paths.get(getTestDirPath(), "opencores", "rtl", "wb_dma").toString(),
        Paths.get("opencores", "rtl", "wb_dma", "wb_dma_wb_slv.v").toString());
  }

  @Override
  public String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "hdl-benchmarks", "hdl", "iwls05").toString()
        + File.separator;
  }
}
